package edu.qrg.vmodel.controllers.classic;
// Sun Java 1.3.1 -Sourcepath QRG/VModel/v4/code/src
// -----------------------------------------------------------------------------
// File name: VModelGUIModelFrame.java
//    System: VModel
//   Version: 4.0
//  Abstract: VModel is a visual representation system developed to provide 
//            computer support to students learning how to create, articulate,
//            and reason with scientific models.  There is a special emphasis on
//            supporting the reuse and integration of previously modeled
//            material in the course of making new models. 
//    Author: Leo C. Ureel II
//   Created: March 7, 1999 18:52:54
//   Purpose: The VModelGUIModelFrame represents the top-level VModel window.  
//            The event driven nature of this system gives this class a great 
//            deal of control over the flow and interpretation of data.
// -----------------------------------------------------------------------------
// (c)2001 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
// -----------------------------------------------------------------------------
// Modified: Friday, June 22, 2001 at 12:10:52 by ureel
// -----------------------------------------------------------------------------
import java.awt.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*; //EtchedBorder
import java.util.*;
import javax.swing.tree.*;
import java.awt.image.*;
import java.beans.*;
import java.awt.event.*;

// -----------------------------------------------------------------------------
// Class: VModelCAD
// -----------------------------------------------------------------------------
public class VModelCAD extends JScrollPane implements VModelConstants, ComponentListener {
	protected VModelGUI gui = null;
	public VModelGUI gui ( ) { return( gui ); }
	public VModelGUI gui ( VModelGUI g ) { return( gui = g ); }

	protected VModelModel model = null;
	public VModelModel model ( ) { return( model ); }
	public VModelModel model ( VModelModel m ) { return( model = m ); }

	protected VModelFrame frame = null;
	public VModelFrame frame ( ) { return( frame ); }
	public VModelFrame frame ( VModelFrame f ) { return( frame = f ); }

	protected VModelElement peer = null;
	public VModelElement peer ( ) { return( peer ); }
	public VModelElement peer ( VModelElement e ) { return( peer = e ); }

	protected JScrollBar hbar = null;
	public JScrollBar hbar ( ) { return( hbar ); }
	public JScrollBar hbar ( JScrollBar s ) { return( hbar = s ); }

	protected JScrollBar vbar = null;
	public JScrollBar vbar ( ) { return( vbar ); }
	public JScrollBar vbar ( JScrollBar s ) { return( vbar = s ); }

	protected JViewport view = null;
	public JViewport view ( ) { 
		if (view == null) view(new JViewport()); 
		return( view ); 
	} // reader view
	public JViewport view ( JViewport v ) { return( view = v ); }

	protected VModelPanel viewPane = null;
	public VModelPanel viewPane ( ) {
		if (viewPane == null) viewPane(new VModelPanel(gui())); 
		return( viewPane ); 
	} // reader viewPane
	public VModelPanel viewPane ( VModelPanel v ) { return( viewPane = v ); }

	protected VModelCADpane modelPane = null;
	public VModelCADpane modelPane ( ) { 
		if (modelPane == null) modelPane(new VModelCADpane(gui())); 
		return( modelPane ); 
	} // reader modelPane
	public VModelCADpane modelPane ( VModelCADpane c ) { return( modelPane = c ); }

	protected int modelheight = 0;
	public int modelheight ( ) { return( modelheight ); }
	public int modelheight ( int i ) { return( modelheight = i ); }

	public int modelwidth = 0;
	public int modelwidth ( ) { return( modelwidth ); }
	public int modelwidth ( int i ) { return( modelwidth = i ); }
		
	protected ComponentPanel componentpane = null;
	public ComponentPanel componentpane ( ) {
		if (componentpane == null) componentpane(new ComponentPanel(gui()));
		return(componentpane);
	} // reader componentpane
	public ComponentPanel componentpane ( ComponentPanel c ) { return(componentpane = c); }

	// Methods
	// --------------------------------------------------------------------------
		public void addVModelElement ( VModelElement newElement ) {
			int x = newElement.getInt("XOFFSET");
			int y = newElement.getInt("YOFFSET") - 42;
			if (newElement.tag().equalsIgnoreCase("MODEL")) newElement.tag("GROUP");
			model().add(newElement);
			VModelPeerElement grumble = makePeer(newElement);
			if (grumble != null) {
				grumble.setSize(grumble.getPreferredSize().width, grumble.getPreferredSize().height);
				componentpane().add(grumble); //, mumble.getInt("ID") - 1);
				if (grumble instanceof VModelPeerRelationship) {
					markerize((VModelPeerRelationship) grumble);
				} // end if relationship
				grumble.expandAll(newElement);
			} // end if grumble
		} // method addVModelElement
		
	public void markerize (VModelPeerRelationship r) {
		VModelElement p = r.peer();
		int x = p.getInt("XOFFSET");
		int y = p.getInt("YOFFSET");
		int inx = 0, iny = 0;
		if (p.input() == null) {
			if (r.inputMarker() == null) {
				inx = x - 42;
				if (inx < 0) inx = 0;
				iny = y + 3;
				if (iny < 0) iny = 0;
				r.inputMarker(new VModelPeerMarker(this.gui(), model(), p, r, inx, iny));
				componentpane().add(r.inputMarker());
			} else r.inputMarker().setVisible(true);
		} // end if no input
		if (p.output() == null) {
			if (r.outputMarker() == null) {
				inx = x + r.getSize().width + 10;
				if (inx < 0) inx = 0;
				iny = y + 3;
				if (iny < 0) iny = 0;
				r.outputMarker(new VModelPeerMarker(this.gui(), model(), p, r, inx, iny));
				componentpane().add(r.outputMarker());
			} else r.outputMarker().setVisible(true);
		} // end if no input
	} // method markerize

	public void setBackground(Color color) {
		super.setBackground(color);
		if (modelPane != null) modelPane().setBackground(color);
	} // method setBackground
			
	public void reshape(int x, int y, int w, int h) {
		super.reshape(x,y,w,h);
		reshaper();
	 	invalidate();
	} // method reshape
		
	public void reshaper ( ) {
		Dimension s = getSize();
		int width = s.width;
		int height = s.height;
		
		modelwidth((int) Math.max(modelwidth, width));
		modelheight((int) Math.max(modelheight, height));
		
		if (Math.floor(modelheight() * 1.0 / PAGE_HEIGHT) != (modelheight() * 1.0 / PAGE_HEIGHT))
			modelheight((int) (Math.floor(modelheight() * 1.0 / PAGE_HEIGHT) + 1) * PAGE_HEIGHT);
		
		if (Math.floor(modelwidth() * 1.0 / PAGE_WIDTH) != (modelwidth() * 1.0 / PAGE_WIDTH))
			modelwidth((int) (Math.floor(modelwidth() * 1.0 / PAGE_WIDTH) + 1) * PAGE_WIDTH);
			
		peer().set("MODELWIDTH", modelwidth);
		peer().set("MODELHEIGHT", modelheight);
		
		hbar().setUnitIncrement(modelwidth() / PAGE_WIDTH * 5);
		hbar().setBlockIncrement(modelwidth / PAGE_WIDTH * 50);
		vbar().setUnitIncrement(modelheight / PAGE_HEIGHT * 5);
		vbar().setBlockIncrement(modelheight / PAGE_HEIGHT * 50);
		
		modelPane().setSize(modelwidth(), modelheight());
		modelPane().setPreferredSize(modelPane().getSize());
		modelPane().revalidate();
		
		invalidate();
		validate();
	} // method reshaper
			
		public VModelPeerElement makePeer ( VModelElement element ) {
			VModelPeerElement result = null;
				if (element.peer() != null) {
					result = (VModelPeerElement) element.peer();
				} else if (element.typep("RELATIONSHIP")) {
					result = new VModelPeerRelationship(gui(), model(), element);
				} else if (element.typep("ATTRIBUTE") || element.typep("DESCRIBER")) {
					result = new VModelPeerDescriber(gui(), model(), element);
				} else if (element.tag().equalsIgnoreCase("OBJECT")) {
					result = new VModelPeerObject(gui(), model(), element);
				} else if (element.tag().equalsIgnoreCase("GROUP")) {
					result = new VModelPeerGroup(gui(), model(), element);
				} else if (element.tag().equalsIgnoreCase("MODEL")) {
					result = new VModelPeerGroup(gui(), model(), element);
				} else if (element.tag().equalsIgnoreCase("HISTORY")) {
					result = null;
				} else if (element.tag().equalsIgnoreCase("SESSION")) {
					result = null;
				} else if (element.tag().equalsIgnoreCase("ACTION")) {
					result = null;
				} else {
					QRGutils.debug("VModelCAD","makePeer element=" + element.tag());
					result = null;
				} // end if tag
			return( result );
		} // method makePeer
		
	// ComponentListener Methods
	// -----------------------------------------------------------------------------
	public void componentHidden(ComponentEvent e) {}
	public void componentMoved(ComponentEvent e) {
    	reshaper();
	} // method componentMoved
	public void componentResized(ComponentEvent e) {
		reshaper();
	} // method componentResized
	public void componentShown(ComponentEvent e) { }
		
	// Constructors
	// --------------------------------------------------------------------------
	public VModelCAD ( VModelGUI gui, VModelModel model, VModelElement peer ) {
		super();
		gui(gui);
		model(model);
		peer(peer);
//QRGutils.debug("VModelCAD", "VModelCAD peer=" + peer());
		
		hbar(getHorizontalScrollBar());
		vbar(getVerticalScrollBar());

		setOpaque(false);
		
		modelwidth(Math.max(peer().getInt("MODELWIDTH"), PAGE_WIDTH));
		modelheight(Math.max(peer().getInt("MODELHEIGHT"), PAGE_HEIGHT));
		
		peer().set("MODELWIDTH", modelwidth());
		peer().set("MODELHEIGHT", modelheight());
		
		modelPane().setSize(modelwidth(), modelheight());
		modelPane().setPreferredSize(modelPane().getSize());
		modelPane().setOpaque(false);
		viewPane().setOpaque(false);
		view().setOpaque(false);
		viewPane().add(modelPane(), BorderLayout.CENTER);
		view().setView(viewPane());
		setViewport(view());

		VModelElement mumble = null;
		for(int i = 0; i < model().allElements().size(); i++) {
			mumble = (VModelElement) model().allElements().elementAt(i);
			if (mumble.peer() instanceof VModelPeerRelationship) markerize((VModelPeerRelationship) mumble.peer());
		} // for i
	} // method VModelCAD
	
	// -----------------------------------------------------------------------------
	// Inner Classes
	// -----------------------------------------------------------------------------
	// Class PageRulersPanel
	public class PageRulersPanel extends VModelPanel implements VModelConstants {
		public void paint (Graphics g) {
			super.paint(g);
			int my = getSize().height;
			int mx = getSize().width;
			int page = 1;
			
			if (peer().tag().equalsIgnoreCase("GROUP")) {
				g.setColor(WHITE);
				for(int j = 0; j < modelheight; j += 10)
					g.drawLine(0,j,mx,j);
				for(int i = 0; i < modelwidth; i += 10)
					g.drawLine(i,0,i,my);
			} else {
				g.setColor(new Color(180,250,250));
				for(int j = 0; j < modelheight; j += PAGE_HEIGHT)
					g.drawLine(0,j,mx,j);
				for(int i = 0; i < modelwidth; i += PAGE_WIDTH)
					g.drawLine(i,0,i,my);
					
				g.setColor(new Color(130,250,250));
				for(int j = 0; j < modelheight; j += PAGE_HEIGHT)
					for(int i = 0; i < modelwidth; i += PAGE_WIDTH)
						g.drawString("Page: " + page++,i+20,j + 20);
			} // end if group
		} // method paint
		public PageRulersPanel (VModelGUI gui) {super(gui);}
	} // innerclass PJPanel
	
	// Class ComponentPanel
	public class ComponentPanel extends VModelPanel implements VModelConstants, MouseListener, MouseMotionListener {
		public int clickx = 0;
		public int clicky = 0;
		public int dragx = 0;
		public int dragy = 0;
		public boolean dragging = false;
		
		public ComponentPanel (VModelGUI gui ) {
			super(gui);
			setOpaque(false);
			setLayout(null);
			addMouseListener(this);
			addMouseMotionListener(this);
			VModelElement mumble = null;
			Component grumble = null;
			for(int i = 0; i < model().allElements().size(); i++) {
				mumble = (VModelElement) model().allElements().elementAt(i);
				if (mumble != model().modelElement()) {
					grumble = makePeer(mumble);
					if (grumble != null) {
						grumble.setSize(grumble.getPreferredSize().width, grumble.getPreferredSize().height);					
						add(grumble); //, mumble.getInt("ID") - 1);
						grumble.validate();
						if ((mumble.parent() != peer()) && !mumble.getBoolean("EXTENDED")) grumble.setVisible(false);
					} // end if grumble
				} // end if mumble
			} // end for i
		} // constructor
		
		public boolean isFocusCycleRoot( ) { return(true); }
		
		public Color getGroupColor (VModelElement e) {
			Color result = null;
			if (e == model().modelElement()) result = GREEN;
			else if ((e.peer() != null) && (((VModelPeerElement) e.peer()).isVisible()) && 
					    e.getBoolean("SKINNED")) 
				result = e.getColor("GROUPCOLOR");
			else if (e.parent() != null) result = getGroupColor(e.parent());
			return(result);
		} // method getGroupColor
		
		public boolean isSkinned (VModelElement e) {
			boolean result = false;
			if (e == model().modelElement()) result = false;
			else if ((e.peer() != null) && (((VModelPeerElement) e.peer()).isVisible()) && 
					    e.getBoolean("SKINNED")) 
				result = true;
			else if (e.parent() != null) result = isSkinned(e.parent());
			return(result);
		} // method getGroupColor
		
		public void print (Graphics g) {
			VModelPeerElement foo = null;
			VModelPeerElement bar = null;
			VModelPeerElement baz = null;
			VModelElement quux = null;
			VModelElement inputElement = null;
			VModelElement outputElement = null;
			VModelPeerMarker inputMarker = null;
			VModelPeerMarker outputMarker = null;
			int inx = 0, iny = 0, outx = 0, outy = 0, x1 = 0, y1 = 0, x2 = 0, y2 = 0, fx = 0,fy = 0,fw = 0,fh = 0, fx2 = 0, fy2 = 0;
			boolean skinned = false;
			Color gcolor = GREEN;
			Color vcolor = GOLD;
			Color bcolor = SLATEGREY;
			if (gcolor == BLACK) gcolor = GREEN;
			boolean isV = false;
			boolean isB = false;
			Polygon poly = null;
			for(int i = 0; i < model().allElements().size(); i++) {
				quux = (VModelElement) model().allElements().elementAt(i);
				foo = null;
				if (quux != model().modelElement()) foo = (VModelPeerElement) quux.peer();
				if (foo != null) {
					gcolor = getGroupColor(quux);
					if ((gcolor == null) || (gcolor == BLACK)) gcolor = GREEN;
					fx = foo.getLocation().x;
					fy = foo.getLocation().y;
					fw = foo.getSize().width;
					fh = foo.getSize().height;
					fx2 = fx + fw;
					fy2 = fy + fh;
					skinned = isSkinned(quux);
					if (skinned && foo.isVisible()) {
						x1 = fx - 3;
						y1 = fy - 3;
						x2 = fw + 6;
						y2 = fh + 6;
						//if (skinned) gcolor = quux.getColor("GROUPCOLOR");
						//else gcolor = quux.parent().getColor("GROUPCOLOR");
//						if (gcolor == BLACK) {
//							gcolor = this.gui().groupColor();
//							quux.parent().set("GROUPCOLOR", gcolor.getRGB());
//						} // end if gcolor
						g.setColor(gcolor);
						if (foo instanceof VModelPeerDescriber) g.fillRoundRect(x1,y1, x2, y2, 50, 50);
						else g.fillRect(x1,y1,x2, y2);
					} // end if quux skinned
					isV = quux.getBoolean("VARIABLE");
					isB = quux.getBoolean("BOUND");
					if (isV || isB) {
						x1 = fx - 5;
						y1 = fy - 5;
						x2 = fw + 10;
						y2 = fh + 10;
						if (foo.isVisible()) {
							poly = new Polygon();
							poly.addPoint(x1 + (x2/2), y1);
							poly.addPoint(x1, y1 + (y2/2));
							poly.addPoint(x1 + (x2/2), y1 + y2);
							poly.addPoint(x1 + x2, y1 + (y2/2));
							if (isV) g.setColor(vcolor);
							else g.setColor(bcolor);
							g.fillPolygon(poly);
							if (isV) g.setColor(bcolor);
							else g.setColor(vcolor);
							g.drawPolygon(poly);
							if (isB) {
								try {
									for(int b = 0; b < quux.bindings().size(); b++) {
									inputElement = (VModelElement) quux.bindings().elementAt(b);
										if (inputElement != null) {
											inx = fx+(fw/2);
											iny = fy + fh/2;
											bar = (VModelPeerElement) inputElement.peer();;
											while((bar == null) || (!bar.isVisible())) {
												inputElement = inputElement.parent();
												if (inputElement == peer()) break;
												bar = (VModelPeerElement) inputElement.peer();
											} // while !baz
											if (bar != null) {
												x1 = bar.getLocation().x+(bar.getSize().width/2);
												y1 = bar.getLocation().y+(bar.getSize().height/2);
												g.setColor(bcolor);
												g.drawLine(x1-2, y1, inx - 2, iny);
												g.drawLine(x1+2, y1, inx + 2, iny);
												g.drawLine(x1, y1-2, inx, iny - 2);
												g.drawLine(x1, y1+2, inx, iny + 2);
												g.drawLine(x1-3, y1, inx - 3, iny);
												g.drawLine(x1+3, y1, inx + 3, iny);
												g.drawLine(x1, y1-3, inx, iny - 3);
												g.drawLine(x1, y1+3, inx, iny + 3);
												g.setColor(vcolor);
												g.drawLine(x1, y1, inx, iny);
												g.drawLine(x1-1, y1, inx - 1, iny);
												g.drawLine(x1+1, y1, inx + 1, iny);
												g.drawLine(x1, y1-1, inx, iny - 1);
												g.drawLine(x1, y1+1, inx, iny + 1);
											} // end if bar
										} // end if inputElement
									} // for b
								} catch (Exception err) { }
							} // end if isV
						} // end if isVisible
					}  // if variable or bound
				}  // if foo
			} // for i
			for(int i = 0; i < model().allElements().size(); i++) {
				quux = (VModelElement) model().allElements().elementAt(i);
				foo = null;
				if (quux != model().modelElement()) foo = (VModelPeerElement) quux.peer();
				if ((foo != null) && (foo instanceof VModelPeerRelationship)) {
					VModelElement quuxie = quux;
					Container foobie = foo;
					while((foobie == null) || (!foobie.isVisible())) {
						quuxie = quuxie.parent();
						if (quuxie == peer()) break;
						foobie = (VModelPeerElement) quuxie.peer();
					} // while !baz
					if (foobie != null) {
						gcolor = getGroupColor(quux);
						if ((gcolor == null) || (gcolor == BLACK)) gcolor = GREEN;
						fx = foobie.getLocation().x;
						fy = foobie.getLocation().y;
						fw = foobie.getSize().width;
						fh = foobie.getSize().height;
						fx2 = fx + fw;
						fy2 = fy + fh;
						skinned = isSkinned(quux);
						g.setColor(quux.getColor("COLOR"));
						try {
							inputElement = quux.input();
							outputElement = quux.output();
							inputMarker = ((VModelPeerRelationship) foo).inputMarker();
							outputMarker = ((VModelPeerRelationship) foo).outputMarker();
							if (!foo.isVisible()) {
								inx = fx + (fw/2);
								iny = fy + (fh/2);
							} else if (quux.getString("ORIENTATION").equalsIgnoreCase("NORTH")) {
								inx = fx+(fw/2);
								iny = fy2;
							} else if (quux.getString("ORIENTATION").equalsIgnoreCase("SOUTH")) {
								inx = fx+(fw/2);
								iny = fy;
							} else if (quux.getString("ORIENTATION").equalsIgnoreCase("EAST")) {
								iny = fy+(fh/2);
								inx = fx;
							} else if (quux.getString("ORIENTATION").equalsIgnoreCase("WEST")) {
								iny = fy + (fh/2);
								inx = fx2;
							} // end if direction
							if (inputMarker != null) {
								bar = inputMarker;
								while((bar == null) || (!bar.isVisible())) {
									inputElement = inputElement.parent();
									if (inputElement == peer()) break;
									bar = (VModelPeerElement) inputElement.peer();
								} // while !baz
								if ((bar != null) && (bar.peer() != peer()) && (bar.isVisible())) {
									x1 = bar.getLocation().x+(bar.getSize().width/2);
									y1 = bar.getLocation().y+(bar.getSize().height/2);
									Color skinColor = null;
									if (quux.getBoolean("EXTENDED") && skinned) skinColor = gcolor;
									drawConnectingLine(g,x1,y1,inx,iny,quux.getColor("COLOR"),skinColor);
								} // end if bar
							} // end if inputElement
							if (inputElement != null) {
								bar = (VModelPeerElement) inputElement.peer();
								while((bar == null) || (!bar.isVisible())) {
									inputElement = inputElement.parent();
									if (inputElement == peer()) break;
									bar = (VModelPeerElement) inputElement.peer();
								} // while !baz
								if ((bar != null) && (bar.peer() != peer()) && (bar.isVisible())) {
									x1 = bar.getLocation().x+(bar.getSize().width/2);
									y1 = bar.getLocation().y+(bar.getSize().height/2);
									Color skinColor = null;
									if (quux.getBoolean("EXTENDED") && skinned) skinColor = gcolor;
									drawConnectingLine(g,x1,y1,inx,iny,quux.getColor("COLOR"),skinColor);
								} // end if bar
							} // end if inputElement
							if (!foo.isVisible()) {
								outx = fx + (fw/2);
								outy = fy + (fh/2);
							} else if (quux.getString("ORIENTATION").equalsIgnoreCase("NORTH")) {
								outx = fx+(fw/2);
								outy = fy;
							} else if (quux.getString("ORIENTATION").equalsIgnoreCase("SOUTH")) {
								outx = fx+(fw/2);
								outy = fy2;
							} else if (quux.getString("ORIENTATION").equalsIgnoreCase("EAST")) {
								outy = fy+(fh/2);
								outx = fx2;
							} else if (quux.getString("ORIENTATION").equalsIgnoreCase("WEST")) {
								outy = fy + (fh/2);
								outx = fx;
							} // end if direction
							if (outputMarker != null) {
								baz = outputMarker;
								while( baz == null) {
									outputElement = outputElement.parent();
									baz = (VModelPeerElement) outputElement.peer();
								} // while !baz
								if (baz != null) {
									x1 = baz.getLocation().x+(baz.getSize().width/2);
									y1 = baz.getLocation().y+(baz.getSize().height/2);
									Color skinColor = null;
									if (quux.getBoolean("EXTENDED") && skinned) skinColor = gcolor;
									drawConnectingLine(g,x1,y1,outx,outy,quux.getColor("COLOR"),skinColor);
								} // end if baz
							} // end if outputElement
							if (outputElement != null) {
								baz = (VModelPeerElement) outputElement.peer();
								while((baz == null) || (!baz.isVisible())) {
									outputElement = outputElement.parent();
									if (outputElement == peer()) break;
									baz = (VModelPeerElement) outputElement.peer();
								} // while !baz
								if ((baz != null) && (baz.isVisible())) {
									x1 = baz.getLocation().x+(baz.getSize().width/2);
									y1 = baz.getLocation().y+(baz.getSize().height/2);
									Color skinColor = null;
									if (quux.getBoolean("EXTENDED") && skinned) skinColor = gcolor;
									drawConnectingLine(g,x1,y1,outx,outy,quux.getColor("COLOR"),skinColor);
								} // end if baz
							} // end if outputElement
							if (foo.isVisible()) {
//QRGutils.debug("VModelCAD","print 1 foo=" + foo);
								g.drawLine(inx, iny, outx, outy);
								g.drawLine(inx - 1, iny, outx - 1, outy);
								g.drawLine(inx + 1, iny, outx + 1, outy);
								g.drawLine(inx,iny - 1, outx, outy - 1);
								g.drawLine(inx, iny + 1, outx, outy + 1);
							} // end if visible
							// This is inefficient need to fix!
							if (foo.isVisible() && !model().checkConnectable(quux)) {
								g.setColor(RED);
								int x = foo.getLocation().x;
								int y = foo.getLocation().y;
								int w = foo.getSize().width;
								int h = foo.getSize().height;
								g.drawLine(x, y + h, x + w, y + h);
							}
						} catch (Exception err) { }
					} // if relationship
				} // if mumble
			} // end for i
		} // print

		public void drawConnectingLine (Graphics g, int x1, int y1, int x2, int y2, Color lineColor, Color skinColor) {
			if (skinColor != null) {
				g.setColor(skinColor);
				g.drawLine(x1 - 2, y1, x2 - 2, y2);
				g.drawLine(x1 + 2, y1, x2 + 2, y2);
				g.drawLine(x1, y1 - 2, x2, y2 - 2);
				g.drawLine(x1, y1 + 2, x2, y2 + 2);
				g.drawLine(x1 - 3, y1, x2 - 3, y2);
				g.drawLine(x1 + 3, y1, x2 + 3, y2);
				g.drawLine(x1, y1 - 3, x2, y2 - 3);
				g.drawLine(x1, y1 + 3, x2, y2 + 3);
			} // end if skinned
			g.setColor(lineColor);
			g.drawLine(x1, y1, x2, y2);
			g.drawLine(x1 - 1, y1, x2 - 1, y2);
			g.drawLine(x1 + 1, y1, x2 + 1, y2);
			g.drawLine(x1, y1 - 1, x2, y2 - 1);
			g.drawLine(x1, y1 + 1, x2, y2 + 1);
		} // method drawConnectingLine

		public void paint (Graphics g) {
//			super.paint(g);

print(g);
super.paint(g);

/*			VModelElement mumble = null;
			for(int i=0;i<model.model.elements.size();i++) {
				mumble = (VModelElement)model.model.elements.elementAt(i);
				if (mumble.tag().equalsIgnoreCase("RELATIONSHIP")) {
					g.setColor(BLACK);
					g.fillRect(mumble.getInt("xoffset"), mumble.getInt("yoffset"), 100, 15);
					g.setColor(WHITE);
					g.drawString(mumble.getString("LABEL"), mumble.getInt("xoffset")+5, 5+mumble.getInt("yoffset"));
				} else {
					g.setColor(BLUE);
					g.fillRect(mumble.getInt("xoffset"), mumble.getInt("yoffset"), 100, 50);
					g.setColor(WHITE);
					g.drawString(mumble.getString("LABEL"), mumble.getInt("xoffset")+5, 5+mumble.getInt("yoffset"));
				}
			} // end for i
*/			
			if (dragging) stretch(g);
		} // method paint
		
		public void stretch(Graphics g) {
			int x1,x2,y1,y2;
			x1 = (int) Math.min(clickx,dragx);
			x2 = (int) Math.max(clickx,dragx);
			y1 = (int) Math.min(clicky,dragy);
			y2 = (int) Math.max(clicky,dragy);
			g.setColor(Color.red);
			boolean o = isOpaque();
			if (!o) setOpaque(true);
			g.setXORMode(getBackground());
			g.drawRect(x1, y1, x2 - x1, y2 - y1);
			setOpaque(o);
		} // method stretch
		
		public Rectangle dragRectangle() {
			int x1,x2,y1,y2;
			x1 = (int) Math.min(clickx,dragx);
			x2 = (int) Math.max(clickx,dragx);
			y1 = (int) Math.min(clicky,dragy);
			y2 = (int) Math.max(clicky,dragy);
			return(new Rectangle(x1, y1, x2 - x1, y2 - y1));
		} // method dragRectangle
		
		public void deselectAll() {
			if (getComponentCount()>0) {
				((VModelPeerElement) getComponent(0)).deselectAll();
			} // if component count
		} // method deselectAll
		
		public void mouseEntered(MouseEvent e) {e.consume();}
		
		public void mouseExited(MouseEvent e) {e.consume();}
		
		public void mousePressed(MouseEvent e) {
			clickx = e.getX();
			clicky = e.getY();
			deselectAll();
			repaint();
			e.consume();
		} // method mousePressed
		
		public void mouseReleased(MouseEvent e) {
			VModelPeerElement comp = null;
			if (dragging) {
				Rectangle r1 = dragRectangle();
				Rectangle r2 = new Rectangle();
				if (!e.isShiftDown()) deselectAll();
				for(int i = 0; i < componentpane().getComponentCount(); i++) {
					comp = (VModelPeerElement) componentpane().getComponent(i);
					r2 = comp.getBounds(r2);
					if (r1.intersects(r2)) comp.selected(true);
				} // for i
				dragging = false;
				repaint();
			} // end if dragging
			e.consume();
		} // method mousePressed
		
		public void mouseClicked(MouseEvent e) {
			if (VModelPeerOntology.selectedOntology() != null) {
					int x = e.getX();
					int y = e.getY();
					int oldx, oldy, dx, dy, x2, y2;
					//makeGUIEntity(x,y);
					VModelElement newElement = this.gui().cloneElement(VModelPeerOntology.selectedOntology());
					if (!newElement.elements().isEmpty()) model().groupInit(newElement);
					oldx = newElement.getInt("XOFFSET");
					oldy = newElement.getInt("YOFFSET");
					dx = x - oldx;
					dy = y - oldy;
					newElement.set("XOFFSET", x);
					newElement.set("YOFFSET", y);
					addVModelElement(newElement);
					if (!e.isShiftDown()) VModelPeerOntology.selectedTree(null);
					validate();
					((Component) model().peer()).repaint();
//				} // end if
			} else {
				QRGutils.debug("ComponentPaneMouseListener", "selectedontology is null");
				deselectAll();
//				if (VModelPeerOntology.selectedOntology() != null) this.gui().statusbar(VModelPeerOntology.selectedOntology().getString("DESCRIPTION"));
			} // end if selectedontology
			e.consume();
		} // method mouseClicked
		
		public void mouseMoved(MouseEvent e) {e.consume();}
		
		public void mouseDragged(MouseEvent e) {
			int x = e.getX();
			int y = e.getY();
			int h = hbar().getValue();
			int v = vbar().getValue();
			int newh = h;
			int newv = v;
			if (x > h + view().getSize().width) newh = h + x - ( h + view().getSize().width);
			if (y > v + view().getSize().height) newv = v + y - ( v + view().getSize().height);
			if (x < h) newh = x;
			if (y < v) newv = y;
			if (newh < 0) newh = 0;
			if (newv < 0) newv = 0;
			if (newh > modelwidth() - view().getSize().width) newh = modelwidth() - view().getSize().width;
			if (newv > modelheight() - view().getSize().height) newv = modelheight() - view().getSize().height;
			hbar().setValue(newh);
			vbar().setValue(newv);
			dragx = x;
			dragy = y;
			dragging = true;
			repaint();
			e.consume();
		} // method mouseClicked
		
	} // innerclass ComponentPanel

	// -----------------------------------------------------------------------------
	// Class: VModelGUIModelCAD
	// -----------------------------------------------------------------------------
	public class VModelCADpane extends JLayeredPane implements VModelConstants, ComponentListener {
		protected VModelGUI gui = null;
		public VModelGUI gui ( ) { return( gui ); }
		public VModelGUI gui ( VModelGUI g ) { return( gui = g ); }

		public PageRulersPanel pagerulers = null;
		public VModelPanel background = null;
			
		public VModelCADpane (VModelGUI gui) {
			super();
			gui(gui);
			setOpaque(false);
			componentpane().setOpaque(false);
			add(componentpane(), 20);
			componentpane().show();
			moveToBack(componentpane());
	
			pagerulers = new PageRulersPanel(gui());
			pagerulers.setOpaque(false);
			add(pagerulers, 20);
			pagerulers.show();
			moveToBack(pagerulers);
	
			background = new VModelPanel(gui);
			background.setOpaque(true);
			setBackground(LIGHTBLUE);
			add(background,10);
			background.show();
			moveToBack(background);
			
			show();
		} // constructor
		
		public void setBackground(Color color) {
			super.setBackground(color);
			background.setBackground(color);
		} // method setBackground
			
		public void reshape(int x, int y, int w, int h) {
			super.reshape(x,y,w,h);
			reshaper();
	    	invalidate();
		} // method reshape
		
		public void reshaper ( ) {
			Dimension s = getSize();
			int width = s.width;
			int height = s.height;
			
			modelwidth((int) Math.max(modelwidth(), width));
			modelheight((int) Math.max(modelheight(), height));
			
			if (Math.floor(modelheight() * 1.0 / PAGE_HEIGHT) != (modelheight() * 1.0 / PAGE_HEIGHT))
				modelheight((int) (Math.floor(modelheight() * 1.0 / PAGE_HEIGHT) + 1) * PAGE_HEIGHT);
			
			if (Math.floor(modelwidth() * 1.0 / PAGE_WIDTH) != (modelwidth() * 1.0 / PAGE_WIDTH))
				modelwidth((int) (Math.floor(modelwidth() * 1.0 / PAGE_WIDTH) + 1) * PAGE_WIDTH);
			
			background.setSize(modelwidth(), modelheight());
			background.setPreferredSize(background.getSize());
			background.revalidate();
			
			pagerulers.setSize(modelwidth(), modelheight());
			pagerulers.setPreferredSize(pagerulers.getSize());
			pagerulers.revalidate();
			
			componentpane().setSize(modelwidth(), modelheight());
			componentpane().setPreferredSize(componentpane().getSize());
			componentpane().revalidate();
			
			invalidate();
		} // method reshaper
			
		// ComponentListener Methods
		// -----------------------------------------------------------------------------
		public void componentHidden(ComponentEvent e) {}
		public void componentMoved(ComponentEvent e) {
	    	reshaper();
		} // method componentMoved
		public void componentResized(ComponentEvent e) {
			reshaper();
		} // method componentResized
		public void componentShown(ComponentEvent e) { }

	} // innerclass VModelGUIModelCAD

} // class VModelGUIModelScrollPane


// -----------------------------------------------------------------------------
// End of File: VModelGUIModelFrame.java        (c) 2001 Northwestern University
