package edu.qrg.vmodel.controllers.classic;
// Sun Java 1.3.1 -Sourcepath QRG/VModel/v4/code/src
// -----------------------------------------------------------------------------
// File name: VModelElement.java
//    System: VModel
//   Version: 4.0
//    Author: Leo C. Ureel II
//   Created: March 7, 1999 18:52:54
//   Purpose: A named object having attributes and possibly containing other 
//            elements.
// -----------------------------------------------------------------------------
// (c)2001 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
// -----------------------------------------------------------------------------
//  Modified: Thursday, June 21, 2001 at 12:10:52 by ureel
// -----------------------------------------------------------------------------
import java.lang.*;
import java.util.*;
import java.io.*;
import java.awt.*;
import javax.swing.*;
import java.net.*;

// -----------------------------------------------------------------------------
// Notes:
// * 
// -----------------------------------------------------------------------------

// -----------------------------------------------------------------------------
// Class: VModelElement 
// Description: A named object having attributes and possibly containing other 
//              elements.
// Methods:
// -----------------------------------------------------------------------------
public class VModelElement implements Cloneable, VModelConstants {
	// Properties & Accessors
	// --------------------------------------------------------------------------
	protected static Vector allVModelElements = null; // class allocated
	public static Vector allVModelElements ( ) { 
		if (allVModelElements == null) allVModelElements(new Vector());
		return(allVModelElements); 
	} // reader allVModelElements
	public static Vector allVModelElements ( Vector v ) { 
		return(allVModelElements = v); 
	} // writer allVModelElements
	
	protected static int count = 0; // class allocated
	public static int count ( ) { return(count); }
	public static int count ( int c ) { return(count = c); }
	
	protected int id = -1;
	public int id ( ) { return(id); }
	public int id ( int i ) { return(id = i); }
	
	protected String tag = "";
	public String tag ( ) { return(tag); }
	public String tag ( String t ) { return( tag = t.toUpperCase() ); }
	
	protected Vector attributes = null;
	public Vector attributes ( ) {
		if (attributes == null) attributes = new Vector();
		return(attributes);
	} // reader attributes
	public Vector attributes ( Vector a ) { return(attributes = a); }
	
	protected Vector elements = null;
	public Vector elements ( ) {
		if (elements == null) elements = new Vector();
		return(elements);
	} // reader elements
	public Vector elements ( Vector e ) { return(elements = e); }
	
	protected Vector connections = null;
	public Vector connections ( ) {
		if (connections == null) connections = new Vector();
		return(connections);
	} // reader connections
	public Vector connections ( Vector c ) { return(connections = c); }
	
	protected Vector bindings = null;
	public Vector bindings ( ) {
		if (bindings == null) bindings = new Vector();
		return(bindings);
	} // reader bindings
	public Vector bindings ( Vector b ) { return(bindings = b); }
	
	protected VModelElement input = null;
	public VModelElement input ( ) { return(input); }
	public VModelElement input ( VModelElement i ) { return(input = i); }
	
	protected VModelElement output = null;
	public VModelElement output ( ) { return(output); }
	public VModelElement output ( VModelElement o ) { return(output = o); }
	
	protected VModelElement parent = null;
	public VModelElement parent ( ) { return(parent); }
	public VModelElement parent ( VModelElement p ) { return(parent = p); }
	
	protected static VModelElement lastElementRead = null;
	public static VModelElement lastElementRead ( ) { return(lastElementRead); }
	public static VModelElement lastElementRead ( VModelElement l ) { 
	   return(lastElementRead = l); 
	} // writer VModelElement
	
	protected Object peer = null;
	public Object peer ( ) { return(peer); }
	public Object peer ( Object p ) { 
	   return(peer = p); 
	} // writer peer
	
	//  attributes.
	// -----------------------------------------------------------------------
	public VModelAttribute findAttribute ( String attribute ) {
		VModelAttribute result = null;
		VModelAttribute candidate = null;
		if ( attributes() != null ) {
			int i = 0;
			while( i < attributes().size( ) ) {
				candidate = (VModelAttribute) attributes().elementAt(i);
				if ( attribute.equalsIgnoreCase(candidate.tag( )) ) {
					result = candidate;
					i = attributes().size( );
				} // end if
				i++;
			} // while
		} // end if
		return(result);
	} // method findAttribute
	
	public VModelAttribute findAttribute( String etag, String atag ) {
		VModelAttribute result = null;
		VModelElement element = null;
		element = findElement(etag);
		if (element != null) result = element.findAttribute(atag);
		return(result);
	} // method findAttribute
	
	// Find child elements.
	// -----------------------------------------------------------------------
	public static VModelElement find ( int id ) {
		VModelElement result = null;
		VModelElement foo = null;
			for(int i = 0; i < allVModelElements().size(); i++) {
				foo = (VModelElement) allVModelElements().elementAt(i);
				if (foo.id() == id) { result = foo; break; }
			} // for i
		return(result);
	} // end if find
	
	public VModelElement findElement ( String tag ) {
		VModelElement result = null;
		if ( elements() != null ) {
			VModelElement candidate = null;
			int i = 0;
			while( i < elements().size( ) ) {
				candidate = (VModelElement) elements().elementAt(i);
				if ( tag.equalsIgnoreCase(candidate.tag( )) ) {
					result = candidate;
					i = elements().size( );
				} else {
					result = candidate.findElement( tag );
					if (result != null) break;
				} // end if else
				i++;
			} // while
		} // end if
		return(result);
	} // method findElement
	
	public VModelElement findElement ( VModelElement element ) {
		VModelElement result = null;
		VModelElement foo = null;
		for(int i = 0; i < elements().size(); i++) {
			foo = (VModelElement) elements().elementAt(i);
			if (foo == element) {
				result = foo;
				break;
			} else if (foo.elements() != null) {
				foo = foo.findElement(element);
				if (foo != null) {
					result = foo;
					break;
				} // end if
			} // end else if
		} // for i
		return result;
	} // method findElement
	
	public VModelElement findElement ( VModelAttribute attribute ) {
		VModelElement result = null;
		VModelElement foo = null;
		for(int i = 0; i < elements().size(); i++) {
			foo = (VModelElement) elements().elementAt(i);
			if ((foo.get(attribute.tag()) != null) &&
				 (foo.getString(attribute.tag()).equalsIgnoreCase((String)attribute.value()))) {
				result = foo;
				break;
			} else if (foo.elements() != null) {
				foo = foo.findElement(attribute);
				if (foo != null) {
					result = foo;
					break;
				} // end if
			} // end else if
		} // for i
		return result;
	} // method findElement
	
	public VModelElement findElement ( String tag, String type ) {
		VModelElement result = null;
		VModelElement foo = null;
		String bar = "";
		for(int i = 0; i < elements().size(); i++) {
			foo = (VModelElement) elements().elementAt(i);
			if (foo.tag().equalsIgnoreCase(tag)) {
				bar = foo.getString("TYPE");
				if (bar.equalsIgnoreCase(type) ||
					 (bar.equalsIgnoreCase("FILE") && 
					  foo.getString("LABEL").equalsIgnoreCase(type))) {
					result = foo;
					break;
				} else if (foo.elements() != null) {
					foo = foo.findElement(tag, type);
					if (foo != null) {
						result = foo;
						break;
				   } // end if
				} // end else if
			} else if (foo.elements() != null) {
				foo = foo.findElement(tag, type);
				if (foo != null) {
					result = foo;
					break;
				} // end if
			} // end else if
		} // for i
		return result;
	} // method findElement
	
	// Get attribute values.
	// -----------------------------------------------------------------------
	public Object get ( String tag ) {
		Object result = null;
		VModelAttribute attribute = null;
		attribute = findAttribute(tag);
		if ( attribute != null ) { result = attribute.value( ); }
		return(result);
	} // method get
	
	public Color getColor ( String tag ) {
		Color result = Color.black;
		Object mumble = get(tag);
		String value = (String) mumble;
		if (mumble instanceof Color) result = (Color) mumble;
		else if (value != null) {
			value = value.trim();
			if (value==null || value.equals("")) result = Color.black;
			else {
				try {
					int foo = Integer.parseInt(value);
					result = new Color(foo);
				} catch (NumberFormatException e) {
					result = QRGutils.matchColor(value);
				} // end try/catch
			} // end if else
		} // if value
		return result;
	} // method getColor
	
	public int getRGBColor ( String tag ) {
		int result = 0;
		Color color = getColor(tag);
		if (color != null) result = color.getRGB();
		return(result);
	} // method getRGBColor
	
	public int getInt ( String tag ) {
		int result = 0;
		String value = (String) get(tag);
		if (value != null) {
			value = value.trim();
			if (value==null || value.equals("")) result = 0;
			else {
				try {result = Integer.parseInt(value);}
				catch(NumberFormatException e) {}
			} // if value
		} // if value
		return result;
	} // method getInt
	
	public double getDouble ( String tag ) {
		double result = 0.0;
		String value = (String) get(tag);
		if (value != null) {
			value = value.trim();
			try {result = Double.valueOf(value).doubleValue();}
			catch(NumberFormatException e) {}
		} // if value
		return(result);
	} // method getDouble
	
	public String getString ( String tag ) {
		String result = "";
		String value = (String) get(tag);
		if (value != null) {
			value = value.trim();
			result = QRGutils.replaceEscapeChars(value);
		} // if value
		return(result);
	} // method getDouble
	
	public boolean containsString ( String tag, String target ) {
		boolean result = false;
		String value = getString(tag);
		result =  (value.indexOf(target)>=0);
		return(result);
	} // method getDouble
	
	public boolean getBoolean ( String tag ) {
		boolean result = false;
		String value = getString(tag);
		if (value != null) {
			value = value.trim();
			result = value.equalsIgnoreCase("TRUE");
		} // if value
		return(result);
	} // method getDouble
	
	// Get attribute values from a child element.
	// -----------------------------------------------------------------------
	public Object get ( String etag, String atag ) {
		Object result = null;
		VModelElement element = null;
		element = findElement(etag);
		if ( element != null ) result = element.get(atag);
		return(result);
	} // method get
	
	public Color getColor ( String etag, String atag ) {
		Color result = null;
		VModelElement element = null;
		element = findElement(etag);
		if ( element != null ) result = element.getColor(atag);
		return(result);
	} // method getColor
	
	public int getRGBColor ( String etag, String atag ) {
		int result = 0;
		VModelElement element = null;
		element = findElement(etag);
		if ( element != null ) result = element.getRGBColor(atag);
		return(result);
	} // method getRGBColor
	
	public int getInt ( String etag, String atag ) {
		int result = 0;
		VModelElement element = null;
		element = findElement(etag);
		if ( element != null ) result = element.getInt(atag);
		return(result);
	} // method get
	
	public double getDouble ( String etag, String atag ) {
		double result = 0;
		VModelElement element = null;
		element = findElement(etag);
		if ( element != null ) result = element.getDouble(atag);
		return(result);
	} // method getDouble
	
	public String getString ( String etag, String atag ) {
		String result = "";
		VModelElement element = null;
		element = findElement(etag);
		if ( element != null ) result = element.getString(atag);
		return(result);
	} // method getString
	
	public boolean containsString ( String etag, String atag, String target ) {
		boolean result = false;
		String value = getString(etag, atag);
		result =  (value.indexOf(target)>=0);
		return(result);
	} // method getDouble
	
	public boolean getBoolean ( String etag, String atag ) {
		boolean result = false;
		VModelElement element = null;
		element = findElement(etag);
		if ( element != null ) result = element.getBoolean(atag);
		return(result);
	} // method getString
	
	// Set attribute values.
	// -----------------------------------------------------------------------
	public boolean removeAttribute ( String tag ) {
		boolean result = false;
		VModelAttribute attribute = null;
		attribute = findAttribute(tag);
		if ( attribute != null ) 
			result = attributes( ).removeElement(attribute);
		return(result);
	} // method set
	
	public Object set ( String tag, Object value ) {
		Object result = null;
		VModelAttribute attribute = null;
		attribute = findAttribute(tag);
		if ( attribute != null ) result = attribute.value(value);
		else {
			if ( attributes == null ) attributes = new Vector( );
			attribute = new VModelAttribute(tag, value);
			attributes( ).addElement(attribute);
			result = attribute.value( );
		} // end if else
		return(result);
	} // method set
	
	public Object set ( String tag, int value ) {
	   return(set(tag, "" + value));
	} // method set
	
	public Object set ( String tag, boolean value ) {
	   return(set(tag, "" + value));
	} // method set
	
	public Object set ( String tag, double value ) {
	   return(set(tag, "" + value));
	} // method set
	
	public Object set ( String tag, Color value ) {
	   return(set(tag, "" + value.getRGB()));
	} // method set
	
	// Set attribute values of a child element.
	// -----------------------------------------------------------------------
	public Object set ( String etag, String atag, Object value ) {
		Object result = null;
		VModelElement element = null;
		element = findElement(etag);
		if (element == null) {
			element = new VModelElement(etag);
			elements( ).addElement(element);
		} // end if
		result = element.set(atag, value);
		return(result);
	} // method set
	
	public Object set ( String etag, String atag, int value ) {
	   return(set(etag, atag, "" + value));
	} // method set
	
	public Object set ( String etag, String atag, double value ) {
	   return(set(etag, atag, "" + value));
	} // method set
	
	public Object set ( String etag, String atag, boolean value ) {
	   return(set(etag, atag, "" + value));
	} // method set
	
	public Object set ( String etag, String atag, Color value ) {
	   return(set(etag, atag, "" + value.getRGB()));
	} // method set
	
	// ownership methods.
	// -----------------------------------------------------------------------
	public boolean descendsFrom ( VModelElement element ) {
		boolean result = (parent() == element);
		while((result == false) && (parent() != null)) {
		   result = parent().descendsFrom(element);
		} // end while
		return(result);
	} // method descendsFrom
	
	public boolean descendsFrom ( String tag ) {
		boolean result = false;
		if (parent() != null) result = (parent().tag().equalsIgnoreCase(tag));
		while((result == false) && (parent() != null)) {
		   result = parent().descendsFrom(tag);
		} // end while
		return(result);
	} // method descendsFrom
	
	public boolean hasDescendent ( VModelElement element ) {
	   return(element.descendsFrom(this));
	} // method hasDescendent
	
	public boolean hasDescendent ( String tag ) {
		boolean result = false;
		VModelElement mumble = null;
		for(int i = 0; i < elements().size(); i++) {
			mumble = (VModelElement) elements().elementAt(i);
			result = mumble.tag().equalsIgnoreCase(tag);
			if (result) break;
		} // for i
		if (!result) {
			for(int i = 0; i < elements().size(); i++) {
				mumble = (VModelElement) elements().elementAt(i);
				result = mumble.hasDescendent(tag);
				if (result) break;
			} // for i
		} // if result
		return(result);
	} // method hasDescendent
	
	public boolean typep ( String t ) {
		boolean result = false;
		result = tag().equalsIgnoreCase(t);
		if (!result) result = getString("TYPE").equalsIgnoreCase(t);
		if (!result) result = (getString("TYPECHAIN").indexOf(t.toUpperCase()) >= 0);
		if (!result) result = (getString("INHERITANCECHAIN").indexOf(t.toUpperCase()) >= 0); // deprecated
		return(result);
	} // method typep
	
	// Stringification.
	// -----------------------------------------------------------------------
	public String stringifyAttributes ( int indent ) {
		String result = "";
		int j = 0;
		if (attributes() != null) {
			j = 0;
			while( j < attributes().size( ) ) {
				if (j > 0) result += "\r\n" + QRGutils.space(indent);
				else result += " "; 
				result += (VModelAttribute) attributes().elementAt(j);
				j++;
			} // while
		} // end if
		return(result);
	} // method toString
	
	public String stringifyElements ( int indent ) {
		String result = "";
		Object foo = null;
		if (elements() != null) {
			for(int j = 0; j < elements().size( ); j++ ) {
				foo = elements().elementAt(j);
				if (foo != null) {
					if (foo instanceof VModelElement) result += ((VModelElement) foo).toString(indent);
					else result += foo + "\r\n";
				} else {
				   QRGutils.debug("VModelElement","stringifyElements foo=null");
				} // end if foo else
			} // for j
		} else elements(new Vector( ));
		return(result);
	} // method toString
	
	public String toString ( int indent ) {
		String result = "";
		int i = indent, j = 0;
		result = QRGutils.space(i) + "<" + tag.toUpperCase();
		result += stringifyAttributes( result.length( ) + 1 );
		if ((elements != null) && (elements.size() > 0)) {
			result += ">\r\n";
			result += stringifyElements( i + 3 );
			result += QRGutils.space(i) + "</" + tag + ">\r\n";
		} else result += " />\r\n";
		return(result);
	} // method toString
	
	public String toString ( ) { 
		savePrep();
		return(toString( 0 )); 
	} // method toString
	
	// Add child elements.
	// -----------------------------------------------------------------------
	public VModelElement addElement ( VModelElement element ) { // deprecated
		if(element!=null) {
			elements().addElement(element);
			element.parent(this);
		} // end if element
		return element;
	} // method addElement
	
	public VModelElement add ( VModelElement element ) { 
	   return(addElement(element));
	} // method add 
	
	// Remove child elements.
	// -----------------------------------------------------------------------
	public boolean remove ( VModelElement element ) { 
		boolean result = false;
		if (element != null) {
			result = elements().removeElement(element);
			if (result && (element.parent() == this)) 
				element.parent(null);
		} // end if element
		return(result);
	} // method add 
	
	// Cloning.
	// -----------------------------------------------------------------------
	public Object clone() {
		Object result = new VModelElement(tag());
		savePrep();
		((VModelElement)result).copyAttributes(this);
		((VModelElement)result).copyElements(this);
		((VModelElement)result).set("KEY",((VModelElement)result).id());
		((VModelElement)result).set("ID",id());
		((VModelElement)result).parent(parent());
		return(result);
	} // method clone
	
	public void copyAttributes(VModelElement e) {
		for(int j = 0; j < e.attributes().size(); j++) {
			VModelAttribute attribute = (VModelAttribute) e.attributes().elementAt(j);
			if (attribute.value() instanceof String) set(new String(attribute.tag()), new String((String)attribute.value()));
			else set(new String(attribute.tag()), attribute.value());
		} // for j
	} // method copyAttributes
	
	public void copyElements ( VModelElement e ) {
		// Does not copy properties - only attributes and elements.
		VModelElement mumble = null;
		VModelElement grumble = null;
		for(int i = 0; i < e.elements().size(); i++) {
			mumble = (VModelElement)e.elements().elementAt(i);
			grumble = (VModelElement) mumble.clone();
			add(grumble);
		} // for i
	} // method copyElements
	
	// Methods for Loading.
	// -----------------------------------------------------------------------
	public static int readAttributes(InputStream inp, VModelElement element) {
		int c = 0;
		String attributeTag = "", attributeValue = "";
		while((c != -1) && (c != 0x2F)) {
			attributeTag = "";
			attributeValue="";
			c = QRGutils.gobbleWhiteSpace(inp);
			attributeTag = "";
			while (((c >= 0x41) && (c <=0x5A))||
			       ((c >= 0x61) && (c <= 0x7A)) || 
			        (c == 0x5F) || (c == 0x2D)) {
				attributeTag += (char) c;
				try {c = inp.read( );}
				catch (Exception e) {c = -1;}
			} // while
			if ((c == 0x3E) || (c == -1) || (c == 0x2F)) return(c);
			if ((c != -1) && (c != 0x3D)) c = QRGutils.gobbleWhiteSpace(inp);
			if ((c != 0X3D)) return(c); // error
			c = QRGutils.gobbleWhiteSpace(inp);
			if ((c != 0x22)) return(c); //error
			try {c = inp.read();}
			catch (Exception e) {c = -1;}
			while ((c != -1) && (c != 0x22)) {
				attributeValue = attributeValue + (char) c;
				try {c = inp.read();}
				catch (Exception e) {c = -1;}
			} // end while
			if (attributeTag.equalsIgnoreCase("MOD-ELEMENT")) attributeTag = "ALLOW-CONNECTIONS";
			element.set(attributeTag, attributeValue);
		} // end while
		return c;
	} // method readAttributes
	
	public static int gobbleUntilTag ( InputStream inp ) {
		int c = 0;
		try {while ((c != -1) && (c != 0x3C)) c = inp.read();}
		catch (Exception e) {c = -1;}
		return(c);
	} // method gobbleUntilTag
	
	public static boolean legalFirstChar ( int c ) {
		return (((c >= 0x41) && (c <=0x5A)) ||
		        ((c >= 0x61) && (c <= 0x7A)) || 
		         (c == 0x5F) || (c == 2F));
	} // method legalFirstChar
	
	public static int gobbleUntilLegalFirstChar (InputStream inp) {
		int c = 0;
		boolean flag = true;
		try {
			while((c!=-1) && flag && (c != 2F)) {
				c = gobbleUntilTag(inp);
				if (c != -1) {
				try {c = inp.read();}
				catch (Exception e) {c = -1;}
				flag = (!legalFirstChar(c));
				if (c == 0x2F) flag = false;
		   		} // end if c != 1
			} // end while
		} catch ( Exception e ) {c = -1;}
		return(c);
	} // method gobbleUntilLegalFirstChar
	
	public static int readElement(InputStream inp, VModelElement parent) throws Exception {
		int c = 0;
		String elementTag = "";
		VModelElement element = null;
		boolean flag = true;
		lastElementRead = null;
		if (inp != null) {
			c = gobbleUntilLegalFirstChar(inp);
			if (c != 0x2F) {
				if ((c != 0x2F) && (c != -1)) {
					elementTag = "";
					while (((c >= 0x41) && (c <=0x5A))||
					       ((c >= 0x61) && (c <= 0x7A)) || 
					        (c == 0x5F)) {
						elementTag += (char) c;
						try {c = inp.read();}
						catch (Exception e) {c = -1;}
					} // while
					element = new VModelElement(elementTag, parent, true);
					if (c != 0x3E) c = readAttributes(inp, element);
					while((c != -1) && (c != 0x2F)) c = readElement(inp, element);
				} // end if
				if (c == 0x2F) c = 0x20;
				if (element != null) element.set("KEY",element.id()); 
				lastElementRead = element;
			} // end if
		} else QRGutils.debug("VModelElement"," readElement inp=null");
		return c;
	} // method readElement
	
	public static VModelElement load ( InputStream inp ) {
		VModelElement result = null;
		try {
			readElement(inp, null);
			result = lastElementRead;
		} catch (Exception e) {
		   QRGutils.debug("VModelElement","VModelElement load Error: \n" + e);
		   e.printStackTrace();
		} // try - catch
		return(result);
	} // method load
	
	public static VModelElement load ( String filename ) {
		VModelElement result = null;
			InputStream inp = null;
			URL url = null;
		try {
			if (QRGutils.isApplet()) {
   			url = new URL(filename);
				if (url != null) inp = url.openStream();
				if (inp != null) inp = new DataInputStream(inp);
			} else inp = new FileInputStream(filename);
			if (inp != null) {
				readElement(inp, null);
				result = lastElementRead;
				inp.close();
			} // end if inp
		} catch ( Exception e ) {
		   QRGutils.debug("VModelElement","VModelElement load ( String filename ) Error: \n" + e);
			e.printStackTrace();
		} // catch
		return(result);
	} // method load
	
	// Methods for saving.
	// -----------------------------------------------------------------------
	public void savePrep( ) {
		String foo = "";
		Object mumble = null;
		//set("KEY", id());
		//set("ID", id());
		//if (typep("ATTRIBUTE")) tag("ATTRIBUTE"); // back change
		foo = "";
		for(int i = 0; i < bindings().size(); i++)
		   foo += " " + ((VModelElement)bindings().elementAt(i)).id();
		if (!foo.equals("")) set("BINDINGS", foo);
		foo = "";
		for(int j = 0; j < connections().size(); j++)
		   foo += " " + ((VModelElement)connections().elementAt(j)).id();
		if (!foo.equals("")) set("CONNECTIONS", foo);
		for(int k = 0; k < elements().size(); k++) {
		   mumble = elements().elementAt(k);
		   if (mumble instanceof VModelElement) ((VModelElement) mumble).savePrep();
		} // for k
	} // method savePrep
	
	public void save ( String filename ) {
		String header = "<!DOCTYPE VMODEL SYSTEM \"vmodel.dtd\">\r\n";
		save( filename, header);
	} // method save
	
	public void save ( String filename, String header ) {
		VModelElement result = null;
		try {
			FileWriter fw = new FileWriter(filename);
			PrintWriter pw = new PrintWriter(fw);
			pw.println(
			"<?xml version = \"1.0\" encoding =\"UTF-8\" standalone = \"no\"?>\r\n"
			+ header );
			pw.print(this);
			pw.close();
			fw.close();
			result = this;
		} catch ( Exception e ) {
			QRGutils.debug("Save Element", " Error: \n" + e);
			e.printStackTrace();
		   //  Potential leak here.
		} // try catch
	} // method save
	
	public void save ( FileOutputStream fout ) {
		try {save(fout.getFD());}
		catch ( Exception e ) {
		   QRGutils.debug("Save Element", " Error: \n" + e);
			e.printStackTrace();
		} // try catch
	} // method save
	
	public void save ( FileDescriptor fout ) {
		try {
			FileWriter fw = new FileWriter(fout);
			PrintWriter pw = new PrintWriter(fw);
			pw.print(this);
			pw.close();
			fw.close();
		} catch ( Exception e ) {
		   QRGutils.debug("Save Element", " Error: \n" + e);
			e.printStackTrace();
		   //  Potential leak here.
		} // try catch
	} // method save
	
	// Constructors
	// --------------------------------------------------------------------------
	public VModelElement() {
	   id(count++);
	   set("KEY", id());
	   set("TYPECHAIN", " ENTITY * ");
	   allVModelElements().addElement(this);
	} // default constructor VModelElement
	
	public VModelElement( String tag ) {
		this();
		tag(tag);
	   set("TYPECHAIN", " " + tag + " " + getString("TYPECHAIN"));
	} // constructor VModelElement
	
	public VModelElement( String tag, VModelElement parent, boolean addFlag ) {
		this(tag);
		parent(parent);
		if (addFlag && (parent() != null)) parent().add(this);
	} // constructor VModelElement
	
	public VModelElement( String tag, VModelElement parent ) {
   	this(tag, parent, true);
	} // constructor VModelElement
	
} // class VModelElement

// -----------------------------------------------------------------------------
// End of File: VModelElement.java              (c) 2001 Northwestern University
