package edu.qrg.vmodel.controllers.classic;
// Sun Java 1.3.1 -Sourcepath QRG/VModel/v4/code/src
// -----------------------------------------------------------------------------
// File name: VModelOntology.java
//    System: VModel
//   Version: 4.0
//    Author: Leo C. Ureel II
//   Created: March 7, 1999 18:52:54
//   Purpose: Keeps track of all loaded ontology files.
// -----------------------------------------------------------------------------
// (c)2001 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
// -----------------------------------------------------------------------------
//  Modified: Tuesday 08 January 2001 at 12:10:52 by ureel
// -----------------------------------------------------------------------------
import java.util.*;
import javax.swing.*;
import java.lang.*;
import java.text.*;
import java.util.*;
import java.io.*;
import java.awt.*;

// -----------------------------------------------------------------------------
// Notes:
// * 
// -----------------------------------------------------------------------------

// -----------------------------------------------------------------------------
// Class: VModelOntology 
// -----------------------------------------------------------------------------
public class VModelOntology implements VModelConstants {
	// Properties & Accessors
	// --------------------------------------------------------------------------
	protected static Vector ontologies = null;
	public static Vector ontologies ( ) { 
		if (ontologies == null) ontologies(new Vector());
		return(ontologies); 
	} // reader ontologies
	public static Vector ontologies ( Vector o ) { return(ontologies = o); }
	
	protected static VModelSystem system = null;
	public static VModelSystem system ( ) { return(system); }
	public static VModelSystem system ( VModelSystem s ) { return(system = s); }

	protected static boolean dirty = false;
	public static boolean dirty ( ) { return(dirty); }
	public static boolean dirty ( boolean d ) { return(dirty = d); }

	// Methods.
	// -----------------------------------------------------------------------
	public static boolean removeFromOntology( VModelElement ontology, VModelElement element ) {
		boolean result = false;
		VModelEvent event = new VModelEvent(REMOVE_FROM_ONTOLOGY, ontology, "Remove an element from an ontology.", ontology, element);
		result = ontology.remove(element);
		if (result) VModelEvent.signal(event);
		return(result);
	} // method removeFromOntology
	
	public static VModelElement addToOntology( VModelElement ontology, VModelElement element ) {
		VModelElement result = null;
		VModelEvent event = new VModelEvent(ADD_TO_ONTOLOGY, ontology, "Add an element to an ontology.", ontology, element);
		result = ontology.add(element);
		if (result != null) VModelEvent.signal(event);
		return(result);
	} // method addToOntology
	
	public static VModelElement addToLibrary ( VModelElement thing ) {
		VModelElement result = null;
		VModelElement library = null;
		VModelElement element = null;
		String name = "";
		String label = "";
		String path = null;
		if (thing != null) {
			if (thing.tag().equalsIgnoreCase("OBJECT")) name = "OBJECTS-ONTOLOGY-LIBRARY";
			else if (thing.tag().equalsIgnoreCase("GROUP")) name = "GROUPS-ONTOLOGY-LIBRARY";
			else if (thing.tag().equalsIgnoreCase("MODEL")) name = "MODELS-ONTOLOGY-LIBRARY";
			library = find(name);
			if (library != null) {
				label = QRGutils.replaceSpaces(thing.getString("LABEL"));
				if (label.length() == 0) label = "???";
				element = find(library, thing.tag(), label);
				if (element != null) removeFromOntology(library, element);
				//element = (VModelElement) thing.clone(); //new VModelElement(thing.tag());
				element = new VModelElement(thing.tag());
				element.set("TYPE", "FILE");
				element.set("ALIAS", thing.getString("LABEL"));
				element.set("LABEL", "???");
				if (library.get("VPATH") != null) {
					path = QRGutils.QRGroot() + library.getString("VPATH");
					element.set("PATH", library.getString("VPATH"));
					element.set("FILEPATH", library.getString("VPATH") + label + "." + VMODEL_SUFFIX);
					thing.set("PATH", library.getString("VPATH"));
					thing.set("FILEPATH", library.getString("VPATH") + label + "." + VMODEL_SUFFIX);
				} else {
					path = QRGutils.QRGroot() + QRGutils.QRGuserpath() + library.getString("PATH");
					element.set("PATH", library.getString("PATH"));
					element.set("FILEPATH", library.getString("PATH") + label + "." + VMODEL_SUFFIX);
					thing.set("PATH", library.getString("PATH"));
					thing.set("FILEPATH", library.getString("PATH") + label + "." + VMODEL_SUFFIX);
				} // end if library path
				element.set("FILENAME", label + "." + VMODEL_SUFFIX);
				element.set("FULLFILEPATH", path + label + "." + VMODEL_SUFFIX);
				element.set("FULLPATH", path);
				addToOntology(library, element);
				thing.set("TYPE", label);
				thing.set("ALIAS", element.get("ALIAS"));
				thing.set("FILENAME", label + "." + VMODEL_SUFFIX);
				thing.set("FULLFILEPATH", path + label + "." + VMODEL_SUFFIX);
				thing.set("FULLPATH", path);
				dirty(true); // instead of or in addition to save?
				thing.save(path + label + "." + VMODEL_SUFFIX);
			} // end if library
		} // end if thing
		return result;
	} // method addToLibrary

	// Search
	// -----------------------------------------------------------------------
	public static VModelElement find ( String tag, String type ) {
		VModelElement result = null;
		VModelElement mumble = null;
		VModelElement grumble = null;
		for(int i = 0; i < ontologies().size(); i++) {
			mumble = (VModelElement) ontologies().elementAt(i);
			if (mumble != null) {
				result = find(mumble, tag, type);
			} // end if mumble
			if (result != null) break;
		} // for i
		return(result);
	} // method findByType
	
	public static VModelElement find ( VModelElement ontology, String tag, String type ) {
		VModelElement result = null;
		VModelElement grumble = null;
		if (ontology != null) {
			result = ontology.findElement(tag, type);
			if ((result != null) && (result.typep("FILE"))) {
				grumble = VModelElement.load(result.getString("FULLFILEPATH"));
				if (grumble != null) {
					// load or cache?
					// removeFromOntology(ontology, result);
					// addToOntology(ontology, grumble);
					result = grumble;
				} // end if grumble
				//((VModelElement)result).parent(parent());
			} // end if result
		} // end if ontology
		return(result);
	} // method findByType
	
	public static VModelElement find ( String ontology, String tag, String type ) {
		VModelElement result = null;
		VModelElement mumble = null;
		for(int i = 0; i < ontologies().size(); i++) {
			mumble = (VModelElement) ontologies().elementAt(i);
			if ((mumble != null) && (mumble.getString("LABEL").equalsIgnoreCase(ontology)))
				result = find(mumble, tag, type);
			if (result != null) break;
		} // for i
		return(result);
	} // method findByType
	
	public static VModelElement find ( VModelElement element, String library ) {
		VModelElement result = null;
		VModelElement foo = null;
		String tag = "ONTOLOGY";
		String type = "LIBRARY";
		String bar = "";
		if (element == null) {
			for(int i = 0; i < ontologies().size(); i++) {
				result = find((VModelElement) ontologies().elementAt(i), library);
				if (result != null) break;
			} // for i
		} else {
			for(int i = 0; i < element.elements.size(); i++) {
				foo = (VModelElement) element.elements.elementAt(i);
				if (foo.tag().equalsIgnoreCase(tag)) {
					bar = (String) foo.get("TYPE");
					if (type.equalsIgnoreCase(foo.getString("TYPE")) && 
						 library.equalsIgnoreCase(foo.getString("LIBRARY"))) {
						result = foo;
						break;
					} if (foo.elements() != null) {
						foo = find(foo, library);
						if (foo != null) {
							result = foo;
							break;
						} // end if
					} // end else if
				} else if (foo.elements() != null) {
					foo = find(foo, library);
					if (foo != null) {
						result = foo;
						break;
					} // end if
				} // end else if
			} // for i
		} // end if element
		return result;
	} // method find

	public static VModelElement find ( String library ) {
		VModelElement result = find((VModelElement) null, library);
		return result;
	} // method find

/*
What is the rght thing to do here?

Create an ontology element based on the tag of thing.
Type is file.
Add in appropriate file info.
Save the object to the appropriate directory.

What about url info?

Should we save these things in a user ontology???
(This might be necessary for the Applet version.)
Of course the applet version wouldn't alow users to save ontologies,
but perhaps the system ontology should save the urls of known objects
instead of relying on the autoloading of directories.

Hmmmm...............


*/


	// File IO.
	// -----------------------------------------------------------------------
	public static void loadOntologyFiles ( String path ) {
		VModelElement mumble = null;
		if (QRGutils.isApplet()) {
			mumble = VModelElement.load(path + "build.ont");
			if (mumble != null) {
				ontologies().addElement(mumble);
				loadLibraries(mumble);
			} // end if mumble
		} else {
			File dir = new File(path);
			String files[] = dir.list(new ontFileFilter());
			VModelElement grumble = null;
			for(int i = 0; i < files.length; i++) {
				mumble = VModelElement.load(path+files[i]);
				if (mumble != null) {
					ontologies().addElement(mumble);
					loadLibraries(mumble);
				} // end if mumble
			} // for i
		} // end if isApplet
	} // method loadOntologyFiles
	
	public static void loadLibraries ( VModelElement element ) {
		String path = null;
		File dir = null;
		VModelElement mumble = null;

		if (element.tag().equalsIgnoreCase("ONTOLOGY") && 
			 element.getString("TYPE").equalsIgnoreCase("LIBRARY")) {
			
			if (element.get("VPATH") != null)
				path = QRGutils.QRGroot() + element.getString("VPATH");
			else
				path = QRGutils.QRGroot() + QRGutils.QRGuserpath() + element.getString("PATH");
			dir = new File(path);
			String files[] = dir.list(new vmfFileFilter());
if (files != null ) {
			for(int i = 0; i < files.length; i++) {
				mumble = new VModelElement(element.getString("TAG"));
				mumble.set("LABEL", QRGutils.stripExtension(files[i]));
				mumble.set("TYPE", "FILE");
				mumble.set("FULLFILEPATH", path + files[i]);
				mumble.set("FULLPATH", path);
				mumble.set("MODE", "ALL");
				addToOntology(element, mumble);
			} // for i
}			
		} else {
if ( element != null ) {
			for(int i = element.elements().size() - 1; i >= 0; i--) {
				loadLibraries((VModelElement)element.elements().elementAt(i));
			} // for i
}			
		} // end if type
	} // method loadLibraries

	public static VModelElement loadOntology ( String filename ) {
		VModelElement result = VModelElement.load( filename );
		return(result);
	} // method loadOntology

	public void savePrep ( ) {
		// Does nothing at this time.
	} // method savePrep
	
	// Ontologies.
	// --------------------------------------------------------------------------
	public VModelOntology (VModelSystem system) {
		system(system);
		loadOntologyFiles(QRGutils.QRGroot()+ONTOLOGYPATH);
	} // VModelOntology	

} // class VModelOntology

// -----------------------------------------------------------------------------
// End of File: VModelOntology.java             (c) 2001 Northwestern University
