package edu.qrg.vmodel.controllers.classic;
// Sun Java 1.3.1 -Sourcepath QRG/VModel/v4/code/src
// -----------------------------------------------------------------------------
// File name: VModelGUIModelFrame.java
//    System: VModel
//   Version: 4.0
//  Abstract: VModel is a visual representation system developed to provide 
//            computer support to students learning how to create, articulate,
//            and reason with scientific models.  There is a special emphasis on
//            supporting the reuse and integration of previously modeled
//            material in the course of making new models. 
//    Author: Leo C. Ureel II
//   Created: March 7, 1999 18:52:54
//   Purpose: The VModelGUIModelFrame represents the top-level VModel window.  
//            The event driven nature of this system gives this class a great 
//            deal of control over the flow and interpretation of data.
// -----------------------------------------------------------------------------
// (c)2001 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
// -----------------------------------------------------------------------------
// Modified: Friday, June 22, 2001 at 12:10:52 by ureel
// -----------------------------------------------------------------------------
import java.awt.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*; //EtchedBorder
import java.util.*;
import javax.swing.tree.*;
import java.awt.image.*;
import java.beans.*;
import java.awt.event.*;
import javax.swing.plaf.metal.*;

// -----------------------------------------------------------------------------
// Class: VModelPeerElement
// -----------------------------------------------------------------------------
public class VModelPeerDescriber extends VModelPeerElement implements VModelConstants {
	// Properies & Accessors
	// --------------------------------------------------------------------------
	protected VModelPanel attributePane = null;
	public VModelPanel attributePane ( ) {
		if (attributePane == null) {
			label(new JTextField(peer().getString("LABEL")));
			JLabel typelabel = new JLabel(peer().getString("TYPE"));
			VModelPanel lpanel = makeLinksPanel(gui());
			attributePane(new VModelPanel(gui()));
			typelabel.setForeground(WHITE);
			lpanel.setBackground(peer().getColor("COLOR"));
			lpanel.setSize(typelabel.getPreferredSize().width+lpanel.getPreferredSize().width, 18);
			//lpanel.setPreferredSize(lpanel.getSize());
			lpanel.add(typelabel, 0);
			lpanel.revalidate();
			lpanel.setOpaque(false);
			label().setBorder(new LineBorder(peer().getColor("COLOR"), 2));
			label().addFocusListener(this);
			label().addKeyListener(new TextKeyAdapter());
			attributePane.setBackground(peer().getColor("COLOR"));
			attributePane.setOpaque(false);
			JLabel a = new JLabel(gui().getImage("spacer.gif")) { public void paint(Graphics g) {}};
			a.setSize(15,15);
			a.setPreferredSize(a.getSize());
			JLabel b = new JLabel(gui().getImage("spacer.gif")) { public void paint(Graphics g) {}};
			b.setSize(15,15);
			b.setPreferredSize(a.getSize());
			attributePane.add(a, BorderLayout.EAST);
			attributePane.add(b, BorderLayout.WEST);
			VModelPanel foobar = new VModelPanel(gui());
			foobar.setOpaque(false);
			foobar.add(label(), BorderLayout.CENTER);
			foobar.add(lpanel, BorderLayout.SOUTH);
			attributePane.add(foobar, BorderLayout.CENTER);
			attributePane.revalidate();
			attributePane.validate();
			attributePane.setLocation(peer().getInt("XOFFSET"), peer().getInt("YOFFSET"));
			attributePane.setSize(lpanel.getPreferredSize().width, attributePane.getPreferredSize().height);
			//attributePane.setPreferredSize(getSize());
			attributePane.revalidate();
			attributePane.validate();
		} // end if result
		return(attributePane);
	} // reader attributePane
	public VModelPanel attributePane ( VModelPanel p ) { return(attributePane = p); }

	// Methods
	// --------------------------------------------------------------------------
	public void paint( Graphics g ) {
		paintBorder(g);
		Color oldColor = g.getColor();
		int i;
		if (active())  g.setColor(peer().getColor("COLOR"));
		else g.setColor(GREY);

		g.fillRoundRect(4, 4, getSize().width - 8, getSize().height - 8, 40, 40);
		g.setColor(oldColor);
		paintChildren(g);
	} // method paint

	public void print (Graphics g) {
		if (isOpaque()) super.print(g);
		else {
			int i;
			if (active())  g.setColor(peer().getColor("COLOR"));
			else g.setColor(GREY);
			g.fillRoundRect(4, 4, getSize().width - 8, getSize().height - 8, 40, 40);
			printComponents(g);
		} // end if isOpaque
	} // method print

	// VModelPeerElement Methods
	// --------------------------------------------------------------------------
	public void makePopupMenu ( ) {
		super.makePopupMenu();
		if (!peer().getBoolean("VARIABLE")) menu().add(menuitemVariablize());
		else menu().add(menuitemUnVariablize());
		if (peer().getBoolean("BOUND")) menu().add(menuitemUnBind());
		if (!peer().connections().isEmpty()) {
			menu().addSeparator();
			menu().add(menuitemDisconnectAll());
			menu().add(menuitemDisconnectConnections());
		} // end if connections
	} // method makePopupMenu

	public void showPopupMenu (int x, int y ) {
		makePopupMenu();
		super.showPopupMenu(x,y);
	} // method showPopupMenu
		
	public int displayMode ( int m ) {
		peer().set("DISPLAY_MODE", ATTRIBUTE_MODE);
		return(ATTRIBUTE_MODE);
	} // method displayMode

	// Constructors
	// --------------------------------------------------------------------------
	public VModelPeerDescriber (VModelGUI gui, VModelModel model, VModelElement peer ) {
		super(gui, model, peer);
		setOpaque(false);
		//makePopupMenu();
		displayMode();
		add(attributePane(), BorderLayout.CENTER);
	} // constructor
	
} // class VModelPeerDescriber

// -----------------------------------------------------------------------------
// End of File: VModelPeerGroup.java        (c) 2001 Northwestern University
