package edu.qrg.vmodel.controllers.classic;
// Sun Java 1.3.1 -Sourcepath QRG/VModel/v4/code/src
// -----------------------------------------------------------------------------
// File name: VModelGUIModelFrame.java
//    System: VModel
//   Version: 4.0
//  Abstract: VModel is a visual representation system developed to provide 
//            computer support to students learning how to create, articulate,
//            and reason with scientific models.  There is a special emphasis on
//            supporting the reuse and integration of previously modeled
//            material in the course of making new models. 
//    Author: Leo C. Ureel II
//   Created: March 7, 1999 18:52:54
//   Purpose: The VModelGUIModelFrame represents the top-level VModel window.  
//            The event driven nature of this system gives this class a great 
//            deal of control over the flow and interpretation of data.
// -----------------------------------------------------------------------------
// (c)2001 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
// -----------------------------------------------------------------------------
// Modified: Friday, June 22, 2001 at 12:10:52 by ureel
// -----------------------------------------------------------------------------
import java.awt.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*; //EtchedBorder
import java.util.*;
import javax.swing.tree.*;
import java.awt.image.*;
import java.beans.*;
import java.awt.event.*;
import edu.qrg.vmodel.controllers.common.*;

// -----------------------------------------------------------------------------
// Class: VModelPeerElement
// -----------------------------------------------------------------------------
public class VModelPeerElement extends VModelPanel implements VModelConstants, VModelPeer, FocusListener {
	// Properties & Accessors
	// --------------------------------------------------------------------------
	protected VModelElement peer = null;
	public VModelElement peer ( ) { return(peer); }
	public VModelElement peer ( VModelElement p ) { return(peer = p); }
	
	protected VModelModel model = null;
	public VModelModel model ( ) { return(model); }
	public VModelModel model ( VModelModel m ) { return(model = m); }
	
	protected JPopupMenu menu = null;
	public JPopupMenu menu ( ) { 
		if (menu == null) menu(new JPopupMenu());
		return(menu); 
	} // reader menu
	public JPopupMenu menu ( JPopupMenu m ) { return(menu = m); }
	
	protected boolean selected = false;
	public boolean selected ( ) { return(selected); }
	public boolean selected ( boolean s ) { 
		Container parent = null;
		selected = s;
		if (s && isVisible()) {
			((VModelFrame) model().peer()).selectedElements().addElement(this);
			parent = getParent();
			if (parent != null) {
				boolean lock = focuslock;
				focuslock = true;
				parent.remove(this);
				parent.add(this,0);
				grabfocus();
				focuslock = lock;
			} // end if parent
		} else ((VModelFrame) model().peer()).selectedElements().removeElement(this);
//		if (!s) losefocus(); 
		((Component) model().modelElement().peer()).repaint(); 
		return(s); 
	} // writer selected

	protected boolean active = true;
	public boolean active ( ) { return(active); }
	public boolean active ( boolean a ) { 
		active = a;
		repaint(); 
		return(a); 
	} // writer active
	
// Earwax cafe
// 1564 N Milwaukee Ave.
		
	protected VModelPeerElement self = null;
	public VModelPeerElement self ( ) { return( self ); }
	public VModelPeerElement self ( VModelPeerElement s ) { return( self = s ); }
	
	public JTextField label = null;
	public JTextField label () { return(label); }
	public JTextField label ( JTextField l ) { return(label = l); }
	public JTextField labela = null;
	public JTextField labelb = null;

	// Methods
	// --------------------------------------------------------------------------
	public void grabfocus ( ) {
		boolean lock = focuslock;
		focuslock = true;
		if ((label != null) && (!label.hasFocus())) label.requestFocus(); 
		focuslock = lock;
	} // method grabfocus

//	public void losefocus ( ) {
//		if (label != null) label.transferFocus(); 
//	} // method grabfocus


	public boolean isAncestorSelected ( ) {
		boolean result = false;
		VModelElement parent = null;
		VModelPeerElement parentPeer = null;
		if ((!(self() instanceof VModelPeerMarker)) && (peer() != null)) {
			parent = peer().parent();
			if ((model() != null) && (parent != model().modelElement())) {
				parentPeer = (VModelPeerElement) parent.peer();
				if (parentPeer != null) {
					if (parentPeer.selected()) result = true;
					else result = parentPeer.isAncestorSelected();
				} // end if parentPeer
			} // end if parent
		} // end if peer
		return(result);
	} // method isAncestorSelected

	public void expandOne ( VModelElement e ) {
		if ((e != null) && (e.tag().equalsIgnoreCase("OBJECT") || e.tag().equalsIgnoreCase("GROUP"))) {
			VModelPeerElement p = (VModelPeerElement) e.peer();
			if (p != null) {
				int newx = 0, newy = 0;
				if (!e.elements().isEmpty()) {
					e.set("HAS-EXTENDED-ELEMENTS", true);
					e.set("SKINNED", true);
				} // end if !isEmpty
				VModelElement mumbleFrotz = null;
				for(int j = 0; j < e.elements().size(); j++) {
					mumbleFrotz = (VModelElement) e.elements().elementAt(j);
					mumbleFrotz.set("EXTENDED",true);
					newx = mumbleFrotz.getInt("DX") + p.getLocation().x;
					newy = mumbleFrotz.getInt("DY") + p.getLocation().y;
					mumbleFrotz.set("XOFFSET", newx);
					mumbleFrotz.set("YOFFSET", newy);
					VModelPeerElement grumbleFrotz = (VModelPeerElement) mumbleFrotz.peer();
					if (grumbleFrotz == null) grumbleFrotz = makePeer(mumbleFrotz);
					if (grumbleFrotz != null) {
						grumbleFrotz.setLocation(newx, newy);
						grumbleFrotz.setVisible(true);
					} // end if grumbleFrotz
				} // for j
			} // end if p
		} // end if e
	} // method expandAll
	
	public void expandAll ( VModelElement e ) {
		if ((e != null) && (e.tag().equalsIgnoreCase("OBJECT") || e.tag().equalsIgnoreCase("GROUP"))) {
			VModelPeerElement p = (VModelPeerElement) e.peer();
			if (p != null) {
				int newx = 0, newy = 0;
				if (!e.elements().isEmpty()) e.set("HAS-EXTENDED-ELEMENTS", true);
				VModelElement mumbleFrotz = null;
				for(int j = 0; j < e.elements().size(); j++) {
					mumbleFrotz = (VModelElement) e.elements().elementAt(j);
					mumbleFrotz.set("EXTENDED",true);
					newx = mumbleFrotz.getInt("DX") + p.getLocation().x;
					newy = mumbleFrotz.getInt("DY") + p.getLocation().y;
					mumbleFrotz.set("XOFFSET", newx);
					mumbleFrotz.set("YOFFSET", newy);
					VModelPeerElement grumbleFrotz = (VModelPeerElement) mumbleFrotz.peer();
					if (grumbleFrotz == null) grumbleFrotz = makePeer(mumbleFrotz);
					if (grumbleFrotz != null) {
						if (grumbleFrotz.getParent() == null) {
							((VModelPeerModel) model().modelElement().peer()).cadPane().componentpane.add(grumbleFrotz);
						} // end if grumbleFrotz
						grumbleFrotz.setLocation(newx, newy);
						grumbleFrotz.setVisible(true);
						grumbleFrotz.setSize(grumbleFrotz.getPreferredSize().width, grumbleFrotz.getPreferredSize().height);
					} // end if grumbleFrotz
					expandAll(mumbleFrotz);
				} // for j
			} // end if p
		} // end if e
	} // method expandAll
	
	public void contractAll ( VModelElement e ) {
		if ((e != null) && (e.tag().equalsIgnoreCase("OBJECT") || e.tag().equalsIgnoreCase("GROUP"))) {
			VModelPeerElement p = (VModelPeerElement) e.peer();
			if (p != null) {
				e.set("HAS-EXTENDED-ELEMENTS", false);
				e.set("SKINNED", false);
				VModelElement mumbleFrotz = null;
				for(int j = 0; j < e.elements().size(); j++) {
					mumbleFrotz = (VModelElement) e.elements().elementAt(j);
					mumbleFrotz.set("EXTENDED",false);
					mumbleFrotz.set("DX", mumbleFrotz.getInt("XOFFSET") - p.getLocation().x);
					mumbleFrotz.set("DY", mumbleFrotz.getInt("YOFFSET") - p.getLocation().y);
					VModelPeerElement grumbleFrotz = (VModelPeerElement) mumbleFrotz.peer();
					if (grumbleFrotz == null) grumbleFrotz = makePeer(mumbleFrotz);
					if (grumbleFrotz != null) grumbleFrotz.setVisible(false);
					contractAll(mumbleFrotz);
				} // for j
			} // end if p
		} // end if e
	} // method contractAll
	
		public VModelPeerElement makePeer ( VModelElement element ) {
			VModelPeerElement result = null;
				if (element.peer() != null) {
					result = (VModelPeerElement) element.peer();
				} else if (element.typep("RELATIONSHIP")) {
					result = new VModelPeerRelationship(gui(), model(), element);
				} else if (element.typep("ATTRIBUTE") || element.typep("DESCRIBER")) {
					result = new VModelPeerDescriber(gui(), model(), element);
				} else if (element.tag().equalsIgnoreCase("OBJECT")) {
					result = new VModelPeerObject(gui(), model(), element);
				} else if (element.tag().equalsIgnoreCase("GROUP")) {
					result = new VModelPeerGroup(gui(), model(), element);
				} else if (element.tag().equalsIgnoreCase("MODEL")) {
					result = new VModelPeerGroup(gui(), model(), element);
				} else if (element.tag().equalsIgnoreCase("HISTORY")) {
					result = null;
				} else if (element.tag().equalsIgnoreCase("SESSION")) {
					result = null;
				} else if (element.tag().equalsIgnoreCase("ACTION")) {
					result = null;
				} else {
					QRGutils.debug("VModelCAD","makePeer element=" + element.tag());
					result = null;
				} // end if tag
			return( result );
		} // method makePeer
		
	public boolean ancestorSelected ( VModelElement e ) {
		boolean result = false;
		VModelElement parent = e.parent();
		if ((parent != null) && (parent != model().modelElement())) {
			if (((VModelPeerElement) parent.peer() != null) && (((VModelPeerElement) parent.peer()).selected()))
				result = true;
			else
				result = ancestorSelected(parent);
		} // end if parent
		return(result);
	} // method ancestorSelected
		
	public void groupSelected ( VModelPeerElement group ) {
		VModelPeerElement element = null;
		VModelElement ep = null;
		group.peer().set("HAS-EXTENDED-ELEMENTS", true);
		group.peer().set("SKINNED", true);
		VModelElement mumbleFrotz = null;
		
//		contractAll(group.peer());

		//group.cadPane().componentpane = null;

		for(int i = ((VModelFrame) model().peer()).selectedElements().size() - 1; i>=0; i--) {
			element =  (VModelPeerElement) ((VModelFrame) model().peer()).selectedElements().elementAt(i);
			ep = element.peer();
			if ((element != group) && (!ancestorSelected(ep))) {
				ep.set("SKINNED", false);
//				contractAll(ep);
//				element.selected(false);
				//ep.set("EXTENDED",false);
				ep.set("DX", ep.getInt("XOFFSET") - group.getLocation().x);
				ep.set("DY", ep.getInt("YOFFSET") - group.getLocation().y);
				model().addToGroup(group.peer(), ep);
//				element.setVisible(false);
			} else element.selected(false);
			//group.cadPane().componentpane().add(element);
			//element.peer(null);
		} // for i

//		contractAll(group.peer());
		
		group.displayMode(BOX_MODE);
		group.getParent().repaint();
		((VModelFrame) model().peer()).repaint();
	} // method groupSelected

	protected static VModelPeerElement linkpoint = null;
	public VModelPeerElement linkpoint ( ) { return(linkpoint); }
	public VModelPeerElement linkpoint ( VModelPeerElement l ) {
		if (l == null) {
			linkpoint = null;
		} else if (linkpoint == null) {	
			selectOne();
			linkpoint = l;
		} else if (l != linkpoint) {
			VModelElement newElement = model().makeEntity(VModelPeerOntology.selectedOntology());
			VModelElement in = linkpoint.peer();
			VModelElement out = self().peer();
			if (newElement.typep("DESCRIBER")) {
				if (in.typep("ATTRIBUTE") && out.typep("OBJECT")) {
					in = out;
					out = linkpoint.peer();
				}
			} // end if describer
			int x1 = ((VModelPeerElement) in.peer()).getX();
			int y1 = ((VModelPeerElement) in.peer()).getY();
			int x2 = ((VModelPeerElement) out.peer()).getX();
			int y2 = ((VModelPeerElement) out.peer()).getY();
			int x = (x1 + x2)/2;
			int y = (y1 + y2)/2;
			newElement.set("XOFFSET", x);
			newElement.set("YOFFSET", y);
			if (Math.abs(y2 - y1) > Math.abs(x2 - x1)) {
				if (y2 > y1) newElement.set("ORIENTATION", "SOUTH");
				else newElement.set("ORIENTATION", "NORTH");
			} else {
				if (x2 > x1) newElement.set("ORIENTATION", "EAST");
				else newElement.set("ORIENTATION", "WEST");
			} // end if magnitude
			//if (modelCAD() != null) model().addToGroup(modelCAD().peer(), newElement);
			//else model().addToGroup(model().modelElement(), newElement);
			model().addToGroup(model().modelElement(), newElement);
			model().connect(newElement, in, out);
			Component grumble = null;
			grumble = new VModelPeerRelationship(gui(), model(), newElement);
			if (grumble != null) {
				self().getParent().add(grumble); //, mumble.getInt("ID") - 1);
				grumble.setVisible(true);
				grumble.setSize(grumble.getPreferredSize().width,grumble.getPreferredSize().height);
				grumble.validate();
			} // end if grumble
			//add(grumble, mumble.getInt("ID"));
			linkpoint.deselectAll();
			deselectAll();
			//modelCAD().validate();
			//modelCAD().repaint();
			linkpoint(null);
			VModelPeerOntology.selectedTree(null);
		} else {
			VModelPeerOntology.selectedTree(null);
			linkpoint.deselectAll();
			deselectAll();
		}
		((Component) model().peer()).repaint();
		return(linkpoint);
	} // method linkpoint

	public int displayMode ( ) {
		int result = -1;
		if (peer().get("DISPLAY_MODE") != null)
			result = peer().getInt("DISPLAY_MODE");
		else {
			if (peer().typep("RELATIONSHIP"))
				result = RELATIONSHIP_MODE;
			else if ((peer().typep("ATTRIBUTE")) || (peer().typep("DESCRIBER"))) {
				//QRGutils.debug("VModelPeerElement","displayMode ATTRIBUTE_MODE peer=" + peer());
				peer().tag("ATTRIBUTE");
				result = ATTRIBUTE_MODE;
			} else if (peer().tag().equalsIgnoreCase("OBJECT")) {
				if (peer().elements().size() > 0) {
					result = BOX_MODE;
				} else result = OBJECT_MODE;
			} else if (peer().tag().equalsIgnoreCase("GROUP"))
				result = BOX_MODE;
				//result = GROUP_MODE;
			else if (peer().tag().equalsIgnoreCase("MODEL"))
				result = MODEL_MODE;
		} // end if displayMode
		return( result );
	} // method displayMode

	public int displayMode ( int m ) {
		peer().set("DISPLAY_MODE", m);
		return(m);
	} // method displayMode
		
	public VModelPanel makeLinksPanel ( VModelGUI gui ) {
		VModelPanel linkpanel = new VModelPanel(gui);
		linkpanel.setLayout(new FlowLayout());
		ImageIcon chain = null;
		JLabel left = null;
		((FlowLayout)linkpanel.getLayout()).setVgap(2);
		if (peer().getBoolean("ALLOW-CONNECTIONS")) {
			if (peer().containsString("ALLOWED-INPUTS", "RELATOR") ||
				peer().containsString("ALLOWED-OUTPUTS", "RELATOR")) {
				if (peer().containsString("ALLOWED-INPUTS", "RELATOR")) {
					if (peer().containsString("ALLOWED-OUTPUTS", "RELATOR")) 
						chain = gui().getImage("black-both.gif");
					else 
						chain = gui().getImage("black-up.gif");
				} else  chain = gui().getImage("black-down.gif");
				left = new JLabel(chain);
				left.setSize(16,16);
				left.setBorder(new LineBorder(peer().getColor("COLOR"),2));
				left.addMouseListener(new LinkListener(this));
				linkpanel.add(left);
				linkpanel.setSize(linkpanel.getPreferredSize().width+14, 18);
				//linkpanel.setPreferredSize(linkpanel.getSize());
				linkpanel.revalidate();
			} // end if RELATOR
			if (peer().containsString("ALLOWED-INPUTS", "QUANTITY-RELATIONSHIP") ||
				peer().containsString("ALLOWED-OUTPUTS", "QUANTITY-RELATIONSHIP")) {
				if (peer().containsString("ALLOWED-INPUTS", "QUANTITY-RELATIONSHIP")) {
					if (peer().containsString("ALLOWED-OUTPUTS", "QUANTITY-RELATIONSHIP")) 
						chain = gui().getImage("blue-both.gif");
					else 
						chain = gui().getImage("blue-up.gif");
				} else  chain = gui().getImage("blue-down.gif");
				left = new JLabel(chain);
				left.setSize(16,16);
				left.setBorder(new LineBorder(peer().getColor("COLOR"),2));
				left.addMouseListener(new LinkListener(this));
				linkpanel.add(left);
				linkpanel.setSize(linkpanel.getPreferredSize().width+14, 18);
				//linkpanel.setPreferredSize(linkpanel.getSize());
				linkpanel.revalidate();
			}
			if (peer().containsString("ALLOWED-INPUTS", "DESCRIBER") ||
				peer().containsString("ALLOWED-OUTPUTS", "DESCRIBER")) {
				if (peer().containsString("ALLOWED-INPUTS", "DESCRIBER")) {
					if (peer().containsString("ALLOWED-OUTPUTS", "DESCRIBER")) 
						chain = gui().getImage("turq-both.gif");
					else 
						chain = gui().getImage("turq-up.gif");
				} else  chain = gui().getImage("turq-down.gif");
				left = new JLabel(chain);
				left.setSize(16,16);
				left.setBorder(new LineBorder(peer().getColor("COLOR"),2));
				left.addMouseListener(new LinkListener(this));
				linkpanel.add(left);
				linkpanel.setSize(linkpanel.getPreferredSize().width+14, 18);
				//linkpanel.setPreferredSize(linkpanel.getSize());
				linkpanel.revalidate();
			}
			if (peer().containsString("ALLOWED-INPUTS", "CONTROLLER") ||
				peer().containsString("ALLOWED-OUTPUTS", "CONTROLLER")) {
				if (peer().containsString("ALLOWED-INPUTS", "CONTROLLER")) {
					if (peer().containsString("ALLOWED-OUTPUTS", "CONTROLLER")) 
						chain = gui().getImage("dkred-both.gif");
					else 
						chain = gui().getImage("dkred-up.gif");
				} else  chain = gui().getImage("dkred-down.gif");
				left = new JLabel(chain);
				left.setSize(16,16);
				left.setBorder(new LineBorder(peer().getColor("COLOR"),2));
				left.addMouseListener(new LinkListener(this));
				linkpanel.add(left);
				linkpanel.setSize(linkpanel.getPreferredSize().width+14, 18);
				//linkpanel.setPreferredSize(linkpanel.getSize());
				linkpanel.revalidate();
			}
		}

		linkpanel.setSize(linkpanel.getPreferredSize().width+2, 20);
		//linkpanel.setPreferredSize(linkpanel.getSize());
		return(linkpanel);
	} // method hookupsPanel
	
	public boolean selectOne ( ) {
		boolean lock = focuslock;
		focuslock = true;
//		grabfocus();
		deselectAll();
		linkpoint(null);
		focuslock = lock;
		return(selected(true));
	} // method selectOne
	public void selectAll ( ) {
		VModelElement foo = null;
		if (peer() != null) {
			for(int i = 0; i < model().allElements().size(); i++) {
				foo = (VModelElement) model().allElements().elementAt(i);
				if ((foo != null) && (foo.peer() != null) && (!foo.typep("HISTORY")) && (foo != model().modelElement()))
					((VModelPeerElement) foo.peer()).selected(true);
			} // for i
		} // end if peer
	} // method selectOne
	public static void selectAll ( VModelModel model ) {
		VModelElement foo = null;
//		QRGutils.debug("VModelPeerElement", "selectAll size=" + model.allElements().size());
		for(int i = 0; i < model.allElements().size(); i++) {
			foo = (VModelElement) model.allElements().elementAt(i);
			if ((foo != null) && (foo.peer() != null) && (!foo.typep("HISTORY")) && (foo != model.modelElement()))
				((VModelPeerElement) foo.peer()).selected(true);
		} // for i
	} // method selectOne
	public boolean deselectAll ( ) {
		VModelPeerElement foo = null;
		for(int i = ((VModelFrame) model().peer()).selectedElements().size() - 1; i >= 0; i--) {
			foo = (VModelPeerElement) ((VModelFrame) model().peer()).selectedElements().elementAt(i);
			foo.selected(false);
		} // for i
		return(selected());
	} // method deselectAll
	
	// FocusListener methods
	// -----------------------------------------------------------------------
	public static boolean focuslock = false;
	public void focusGained(FocusEvent e) {
/*
		if ((!focuslock) && (!selected())) {
			boolean lock = focuslock;
			focuslock = true;
			selectOne();
			focuslock = lock;
		} // end if
*/
	} // method focusGained
	
	public void focusLost(FocusEvent e) {
		JTextField widget = (JTextField) e.getSource();
		if (peer() != null) peer().set("LABEL", widget.getText());
		//selected(false);		
	} // method focusLost
		
	// InnerClass TextKeyAdapter
	// -----------------------------------------------------------------------
	public class TextKeyAdapter extends KeyAdapter {
		public void keyTyped(KeyEvent e) {
			int kcode = e.getKeyCode();
			int kmod = e.getModifiers();
			JTextField widget = (JTextField) e.getSource();
			if ((widget != null) && (widget.getText().length() == 0)) widget.setText(" ");
			if ((kmod == VKM_COMMAND) || (kmod == VKM_CONTROL) || (kmod == VKM_OPTION)) {
//				QRGutils.debug("keyTyped","VKM_COMMAND");
				gui().keyPressed(e);
				e.consume();
			} else if (kcode == KeyEvent.VK_TAB) {
				e.consume();
				widget.transferFocus();
			} // end if kcode
		} // method keyTyped
		
		public void keyPressed(KeyEvent e) {
			int kcode = e.getKeyCode();
			int kmod = e.getModifiers();
			JTextField widget = (JTextField) e.getSource();
			peer().set("LABEL", widget.getText());
			gui().selectedModel().dirty(true);
		} // Method keyPressed
		
		public void keyReleased(KeyEvent e) {
			int width = self().getPreferredSize().width;
			if (width < 64) width = 64;
			self().setSize( width, self().getHeight());
			self().revalidate();
			self().getParent().repaint();
		} // Method keyReleased
	} // innerclass TextKeyAdapter

	// InnerClass PeerMouseMotionAdapter
	// -----------------------------------------------------------------------
		public static boolean growondragflag = false;
		public static int clickX = 0;
		public static int clickY = 0;
		public static boolean drug = false;
	public class PeerMouseMotionAdapter implements MouseListener, MouseMotionListener {
		public void mouseEntered(MouseEvent e) {
		} // method mouseEntered

		public void mouseExited(MouseEvent e) {
		} // method mouseExited

		public void mouseClicked(MouseEvent e) {
		} // method MouseClicked

		public void mouseReleased(MouseEvent e) {
			VModelElement elm = null;
			VModelPeerElement elmp = null;
			if (drug && selected() && (model() != null)) {
				for(int j = 0; j < model().allElements().size(); j++) {
					elm = (VModelElement) model().allElements().elementAt(j);
					if ((elm != null) && (elm != model().modelElement()) &&
						 (elm.typep("GROUP") || elm.typep("OBJECT") || elm.typep("ATTRIBUTE") || elm.typep("RELATIONSHIP"))) {
						elmp = (VModelPeerElement) elm.peer();
						if ((elmp != null) && elmp.isVisible()) {
							if ((self() instanceof VModelPeerMarker) && (elmp != null) && 
								 (!(elmp instanceof VModelPeerMarker)) &&
								 (self().peer() != elm) &&
								 (self().getBounds().intersects(elmp.getBounds()))) {
								if (self() == ((VModelPeerMarker) self()).relationship().inputMarker()) {
									model().connectInput(peer(), elm);
									((VModelPeerMarker) self()).relationship().inputMarker(null);
									self().getParent().remove(self());
								} else {
									model().connectOutput(peer(), elm);
									((VModelPeerMarker) self()).relationship().outputMarker(null);
									self().getParent().remove(self());
								} // end if inputMarker
								break;
							} else if (elm.getBoolean("VARIABLE") || elm.tag().equalsIgnoreCase("GROUP")) {
//								if ((elmp != null) && (!elmp.selected()) && (self().getBounds().intersects(elmp.getBounds()))) {
								if ((elmp != null) && (!elmp.selected()) && (elmp.getBounds().contains(e.getX() + self().getLocation().x, e.getY() + self().getLocation().y))) {
									VModelPeerElement vava = null;
									VModelElement voom = null;
									for(int s = 0; s < ((VModelFrame) model().peer()).selectedElements().size(); s++) {
									vava = (VModelPeerElement) ((VModelFrame) model().peer()).selectedElements().elementAt(s);
										voom = vava.peer();
										if (voom != null) {
											vava.setLocation(voom.getInt("XOFFSET"), voom.getInt("YOFFSET"));
										} // end if voom
									} // for s									
									if (elm.getBoolean("VARIABLE")) {
										// need to write a function bindSelected()
										for(int s = 0; s < ((VModelFrame) model().peer()).selectedElements().size(); s++) {
										vava = (VModelPeerElement) ((VModelFrame) model().peer()).selectedElements().elementAt(s);
											voom = vava.peer();
											if (voom != null) {
												if (!(vava instanceof VModelPeerRelationship)) model().bind(elm, voom);
											} // end if voom
										} // for s
									} else {
										groupSelected(elmp);
									} // end if
									break;
								} // end if elmp
							} // end if elm
						} // end if isVisible
					} // end if elm
				} // for j
				for(int k=0; k < ((VModelFrame) model().peer()).selectedElements().size(); k++) {
					elmp = (VModelPeerElement) ((VModelFrame) model().peer()).selectedElements().elementAt(k);
					if ((elmp != null) && (!(elmp instanceof VModelPeerMarker)) && (elmp.isVisible())) {
						elm = elmp.peer();
						elm.set("XOFFSET", elmp.getLocation().x);
						elm.set("YOFFSET", elmp.getLocation().y);
					} // end if elmp
				} // for k
				((VModelFrame) gui().selectedModel().peer()).dirty(true);
			} // end if
			drug = false;
			((Component) model().peer()).repaint();
			growondragflag = true;
			e.consume();
		} // method mouseReleased

		public void mousePressed(MouseEvent e) {
			//clickX = self().getX();//+e.getX();
			//clickY = self().getY();//+e.getY();
			clickX = e.getX();
			clickY = e.getY();
			if (active()) {
				if (!QRGutils.isRightClick(e)) {
					if (e.isControlDown()) selected(!selected());
					else if (e.isShiftDown()) selected(true);
					else if (!selected()) selectOne();
				} else {
					deselectAll();
					showPopupMenu(e.getX(), e.getY());
				} // end if rightClick
			} // if active
			e.consume();
		} // method mouseClicked

		public void mouseMoved(MouseEvent e) {
		} // method mouseMoved

		public void mouseDragged(MouseEvent e) {

			if (active && selected()) {
				boolean reshapeflag = false;
				int x = e.getX();
				int y = e.getY();
				int dx = x - clickX;
				int dy = y - clickY;
				int compx = 0; //self().getX();
				int compy = 0; //self().getY();
				int compw = 0; //self().getWidth();
				int comph = 0; //self().getHeight();
				int vieww = ((VModelPeerModel) model().modelElement().peer()).cadPane().view().getWidth();
				int viewh = ((VModelPeerModel) model().modelElement().peer()).cadPane().view().getHeight();
				int scrollx = ((VModelPeerModel) model().modelElement().peer()).cadPane().hbar().getValue();
				int scrolly = ((VModelPeerModel) model().modelElement().peer()).cadPane().vbar().getValue();
				int scrolldx = 0;
				int scrolldy = 0;
				int vw = ((VModelPeerModel) model().modelElement().peer()).cadPane().vbar().getWidth();
				int hh = ((VModelPeerModel) model().modelElement().peer()).cadPane().hbar().getHeight();
				int newx = 0;
				int newy = 0;
				VModelPeerElement c = null;
				drug = true;
				for(int i = 0; i < ((VModelFrame) model().peer()).selectedElements().size(); i++) {
					c = (VModelPeerElement) (((VModelFrame) model().peer()).selectedElements().elementAt(i));
					if (c.selected()) {
						compx = c.getX();
						compy = c.getY();
						compw = c.getWidth();
						comph = c.getHeight();
						newx = compx + dx;
						newy = compy + dy;

						if (newx < 0) {
							newx = 0;
							dx = newx - compx;
						} // end if
						if (newy < 0) {
							newy = 0;
							dy = newy - compy;
						} // end if

						if (!growondragflag) {
							if (newx > (((VModelPeerModel) model().modelElement().peer()).cadPane().modelwidth() - compw)) {
								newx = ((VModelPeerModel) model().modelElement().peer()).cadPane().modelwidth() - compw;
								dx = newx - compx;
							} // end if
							if (newy > (((VModelPeerModel) model().modelElement().peer()).cadPane().modelheight() - comph)) {
								newy = ((VModelPeerModel) model().modelElement().peer()).cadPane().modelheight() - comph;
								dy = newy - compy;
							} // end if
						} // end if growondragflag

					} // end if selected
				} // for i

				for(int i = 0; i < ((VModelFrame) model().peer()).selectedElements().size(); i++) {
					c = (VModelPeerElement) (((VModelFrame) model().peer()).selectedElements().elementAt(i));
					if (c.selected()) {
						compx = c.getX();
						compy = c.getY();
						compw = c.getWidth();
						comph = c.getHeight();
						newx = compx + dx;
						newy = compy + dy;

						if ((!reshapeflag) && (newx > ((VModelPeerModel) model().modelElement().peer()).cadPane().modelwidth() - compw)) {
							((VModelPeerModel) model().modelElement().peer()).cadPane().modelwidth(((VModelPeerModel) model().modelElement().peer()).cadPane().modelwidth() + VModelConstants.PAGE_WIDTH);
							reshapeflag = true;
						} // end if
						if ((!reshapeflag) && (newy > ((VModelPeerModel) model().modelElement().peer()).cadPane().modelheight() - comph)) {
							((VModelPeerModel) model().modelElement().peer()).cadPane().modelheight(((VModelPeerModel) model().modelElement().peer()).cadPane().modelheight() + VModelConstants.PAGE_HEIGHT);
							reshapeflag = true;
						} // end if

						if ((newx < scrollx) ||
								(newx > scrollx + vieww - compw - vw)) {
							scrolldx = dx;
						} // end if
						if ((newy < scrolly) ||
								(newy > scrolly + viewh - comph - hh)) {
							//reshapeflag = true;
							scrolldy = dy;
						} // end if

						c.setLocation(newx,newy);
/*
						if (c.peer() != null) {
							c.peer().set("XOFFSET", newx);
							c.peer().set("YOFFSET", newy);
						} // end if peer
*/
					} // end if selected()
				} // for j

				if (reshapeflag) {
					//modelCAD().reshapeshifter();
					((VModelPeerModel) model().modelElement().peer()).cadPane().reshaper();
					((VModelPeerModel) model().modelElement().peer()).cadPane().validate();
					growondragflag = false;
				} // end if reshapeflag

				((VModelPeerModel) model().modelElement().peer()).cadPane().hbar().setValue(scrollx + scrolldx);
				((VModelPeerModel) model().modelElement().peer()).cadPane().vbar().setValue(scrolly + scrolldy);
				((VModelFrame) model().peer()).dirty(true);
				((VModelPeerModel) model().modelElement().peer()).cadPane().repaint();
			} // if active && selected
			repaint();
			self().getParent().repaint();
			e.consume();
		} // method mouseDragged
	} // innerclass PeerMouseMotionAdapter

	public class ComponentLineBorder extends EmptyBorder {
		public int thickness;

		public ComponentLineBorder(int thickness) {
			super(thickness, thickness, thickness, thickness);
			this.thickness = thickness;
		} // constructor

		public void paintBorder(Component c, Graphics g, int x, int y, int width, int height) {
			Color oldColor = g.getColor();
			//VModelPeerElement vgc = (VModelPeerElement) c;
			int i;
			if (/*vgc.*/selected()) {
				int[] foo = {0,13,24,13};
				int[] bar = {12,24,12,0};
				g.setColor(RED);
				if (c instanceof VModelPeerDescriber) g.drawRoundRect(x, y, width-1, height-1,40,40);
				else if (c instanceof VModelPeerMarker) g.drawPolygon(new Polygon(foo, bar, 4));
				else g.drawRect(x, y, width-1, height-1);
			} else if (isAncestorSelected()) {
				int[] foo = {0,13,24,13};
				int[] bar = {12,24,12,0};
				g.setColor(PINK);
				if (c instanceof VModelPeerDescriber) g.drawRoundRect(x, y, width-1, height-1,40,40);
				else if (c instanceof VModelPeerMarker) g.drawPolygon(new Polygon(foo, bar, 4));
				else g.drawRect(x, y, width-1, height-1);
			} // end if selected
			g.setColor(oldColor);
		} // method paintBorder
	} // innerclass ComponentLineBorder

	public class BoxPane extends VModelPanel {
		public ImageIcon boxImage = null;
		public ImageIcon openImage = null;
		public BoxPane(VModelGUI gui) {
			super(gui);
			boxImage = gui.getImage("box.gif");
			openImage = gui.getImage("box2.gif");
		} // constructor
		public BoxPane(VModelGUI gui, ImageIcon i) {
			super(gui);
			boxImage = i;
			openImage = i;
		} // constructor
		public BoxPane(VModelGUI gui, ImageIcon i, ImageIcon o) {
			super(gui);
			boxImage = i;
			openImage = o;
		} // constructor
		public void paint(Graphics g) {
			if (peer().getBoolean("HAS-EXTENDED-ELEMENTS"))
				g.drawImage(boxImage.getImage(),0,0,getSize().width,getSize().height,null);
			else
				g.drawImage(openImage.getImage(),0,0,getSize().width,getSize().height,null);
			paintChildren(g);
		} // method paint
	} // innerclass BoxPane

	public class LinkListener extends MouseAdapter {
		VModelPeerElement component = null;
		public LinkListener ( VModelPeerElement component ) {
			super();
			this.component = component;
		} // method EntityMouseListener

		public void mousePressed(MouseEvent e) {
			if (VModelPeerOntology.selectedOntology() != null) {
				if (VModelPeerOntology.selectedOntology().typep("RELATIONSHIP")) {
					if (active()) {
						linkpoint(component);
						((Component) model().peer()).repaint();
					} // end if
				} // end if
			} // end if
		} // method mouseClicked
	} // inner-class ModelCanvasMouseListener

public String annotation = "";
public void annotate ( ) {
	final JDialog d = new JDialog();
	Container c = d.getContentPane();
	JTextArea t = new JTextArea(annotation);
	JPanel buttons = new JPanel(new BorderLayout());
	JButton ok = new JButton("OK");
	buttons.add(ok, BorderLayout.EAST);
	ok.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			d.hide();
		} // method actionPerformed
	}); // actionListener
	t.setBorder(new EmptyBorder(5,5,5,5));
	t.setLineWrap(true);
	c.setLayout(new BorderLayout());
	c.add(new JLabel(gui().getImage("wizard.gif")), BorderLayout.WEST);
	c.add(new JLabel("Enter Notes:"), BorderLayout.NORTH);
	c.add(t, BorderLayout.CENTER);
	c.add(buttons, BorderLayout.SOUTH);
	d.setModal(true);
	d.setSize(300,200);
	d.setLocation(QRGutils.center(getSize()));
	d.show();
	annotation = t.getText();
	d.dispose();
} // method annotate

	protected JMenuItem menuitemAnnotate = null;
	public JMenuItem menuitemAnnotate ( ) {
		if (menuitemAnnotate == null) {
			menuitemAnnotate(new JMenuItem("Annotate"));
			menuitemAnnotate.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					final ActionEvent e = ev;
					try {
						Thread t = new Thread() {
							public void run ( ) {
								annotation = peer().getString("NOTES");
								annotate();
								if (!annotation.equals(peer().getString("NOTES"))) {
									peer().set("NOTES", annotation);
									((VModelFrame) gui().frame()).dirty(true);
								} // end if
							} // method run
						}; // Anonymous Class Thread
						t.start();
					} catch (Exception err) {}
				} // method actionPerformed
			}); // actionListener
		} // end if menuitemAnnotate
		return(menuitemAnnotate);
	} // reader menuitemAnnotate
	public JMenuItem menuitemAnnotate ( JMenuItem m ) { return( menuitemAnnotate = m ); }
	
	protected JMenuItem menuitemHelp = null;
	public JMenuItem menuitemHelp ( ) {
		if (menuitemHelp == null) {
			menuitemHelp(new JMenuItem("Help"));
			menuitemHelp.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					final ActionEvent e = ev;
					try {
						Thread t = new Thread() {
							public void run ( ) {
								gui().browser().setTitle("VModel Help: " + peer().getString("LABEL"));
								gui().browser().setSize(700,300);
								gui().browser().gotoPage(QRGutils.QRGroot() + HELPPATH, peer().getString("HELP-LINK"));
							} // method run
						}; // Anonymous Class Thread
						t.start();
					} catch (Exception err) {}
				} // method actionPerformed
			}); // actionListener
		} // end if menuitemHelp
		return(menuitemHelp);
	} // reader menuitemHelp
	public JMenuItem menuitemHelp ( JMenuItem m ) { return( menuitemHelp = m ); }
	
	protected JMenuItem menuitemDisconnectAll = null;
	public JMenuItem menuitemDisconnectAll ( ) {
		if (menuitemDisconnectAll == null) {
			menuitemDisconnectAll(new JMenuItem("Disconnect All"));
			menuitemDisconnectAll.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					final ActionEvent e = ev;
					try {
						Thread t = new Thread() {
							public void run ( ) {
								VModelElement mumble = null;
								if (self() instanceof VModelPeerRelationship) model().disconnect(peer());
								for(int i = peer().connections().size() - 1; i >= 0; i--) {
									mumble = (VModelElement) peer().connections().elementAt(i);
									model().disconnect(mumble, peer()); 
								} // end for i
								((Component) model().peer()).repaint();
							} // method run
						}; // Anonymous Class Thread
						t.start();
					} catch (Exception err) {}
				} // method actionPerformed
			}); // actionListener
		} // end if menuitemDisconnectAll
		return(menuitemDisconnectAll);
	} // reader menuitemDisconnectAll
	public JMenuItem menuitemDisconnectAll ( JMenuItem m ) { return( menuitemDisconnectAll = m ); }
	
	public class VMenuItem extends JMenuItem implements ActionListener {
		public VModelElement element = null;
		public VMenuItem ( String s ) { super(s); }
		public void actionPerformed(ActionEvent ev) {
			final ActionEvent e = ev;
			try {
				Thread t = new Thread() {
					public void run ( ) {
						model().disconnect(element, peer());
						((Component) model().peer()).repaint();
					} // method run
				}; // Anonymous Class Thread
				t.start();
			} catch (Exception err) {}
		} // method actionPerformed
	} // innerclass VMenuItem
	
	protected JMenu menuitemDisconnectConnections = null;
	public JMenu menuitemDisconnectConnections ( ) {
		VModelElement mumble = null;
		VMenuItem grumble = null;
		if (menuitemDisconnectConnections == null) menuitemDisconnectConnections(new JMenu("Disconnect"));
		menuitemDisconnectConnections.removeAll();
		for(int i = peer().connections().size() - 1; i >= 0; i--) {
			mumble = (VModelElement) peer().connections().elementAt(i);
			grumble = new VMenuItem(mumble.getString("TYPE"));
			grumble.element = mumble;
			grumble.addActionListener(grumble);
			menuitemDisconnectConnections.add(grumble);
		} // end for i
		return(menuitemDisconnectConnections);
	} // reader menuitemDisconnectConnections
	public JMenu menuitemDisconnectConnections ( JMenu m ) { return( menuitemDisconnectConnections = m ); }
	
	protected JMenuItem menuitemExtendOne = null;
	public JMenuItem menuitemExtendOne ( ) {
		if (menuitemExtendOne == null) {
			menuitemExtendOne(new JMenuItem("Expand One Level"));
			menuitemExtendOne.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				final ActionEvent e = ev;
				try {
					Thread t = new Thread() {
						public void run ( ) {
							expandOne(peer());
							validate();
							((Component) model().peer()).repaint();
						} // method run
					}; // Anonymous Class Thread
					t.start();
				} catch (Exception err) {}
			} // method actionPerformed
			}); // actionListener
		} // end if menuitemExtendOne
		return(menuitemExtendOne);
	} // reader menuitemExtendOne
	public JMenuItem menuitemExtendOne ( JMenuItem m ) { return( menuitemExtendOne = m ); }
	
	protected JMenuItem menuitemExtendAll = null;
	public JMenuItem menuitemExtendAll ( ) {
		if (menuitemExtendAll == null) {
			menuitemExtendAll(new JMenuItem("Expand All Levels"));
			menuitemExtendAll.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				final ActionEvent e = ev;
				try {
					Thread t = new Thread() {
						public void run ( ) {
							expandAll(peer());
							validate();
							((Component) model().peer()).repaint();
						} // method run
					}; // Anonymous Class Thread
					t.start();
				} catch (Exception err) {}
			} // method actionPerformed
			}); // actionListener
		} // end if menuitemExtendAll
		return(menuitemExtendAll);
	} // reader menuitemExtendAll
	public JMenuItem menuitemExtendAll ( JMenuItem m ) { return( menuitemExtendAll = m ); }
	
	protected JMenuItem menuitemContractAll = null;
	public JMenuItem menuitemContractAll ( ) {
		if (menuitemContractAll == null) {
			menuitemContractAll(new JMenuItem("Contract All Levels"));
			menuitemContractAll.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				final ActionEvent e = ev;
				try {
					Thread t = new Thread() {
						public void run ( ) {
							contractAll(peer());
							validate();
							((Component) model().peer()).repaint();
						} // method run
					}; // Anonymous Class Thread
					t.start();
				} catch (Exception err) {}
			} // method actionPerformed
			}); // actionListener
		} // end if menuitemContractAll
		return(menuitemContractAll);
	} // reader menuitemContractAll
	public JMenuItem menuitemContractAll ( JMenuItem m ) { return( menuitemContractAll = m ); }
	
	protected JMenuItem menuitemSelectGrouped = null;
	public JMenuItem menuitemSelectGrouped ( ) {
		if (menuitemSelectGrouped == null) {
			menuitemSelectGrouped(new JMenuItem("Select Grouped Elements"));
			menuitemSelectGrouped.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				final ActionEvent e = ev;
				try {
					Thread t = new Thread() {
						public void run ( ) {
							selected(true);
							VModelElement mumbleFrotz = null;
							for(int j = 0; j < peer().elements().size(); j++) {
								mumbleFrotz = (VModelElement) peer().elements().elementAt(j);
								VModelPeerElement grumbleFrotz = makePeer(mumbleFrotz);
								if (grumbleFrotz != null) grumbleFrotz.selected(true);
							} // for j
							((Component) model().peer()).repaint();
						} // method run
					}; // Anonymous Class Thread
					t.start();
				} catch (Exception err) {}
			} // method actionPerformed
			}); // actionListener
		} // end if menuitemSelectGrouped
		return(menuitemSelectGrouped);
	} // reader menuitemSelectGrouped
	public JMenuItem menuitemSelectGrouped ( JMenuItem m ) { return( menuitemSelectGrouped = m ); }
	
	protected JMenuItem menuitemUnBind = null;
	public JMenuItem menuitemUnBind ( ) {
		if (menuitemUnBind == null) {
			menuitemUnBind(new JMenuItem("UnBind"));
			menuitemUnBind.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				final ActionEvent e = ev;
				try {
					Thread t = new Thread() {
						public void run ( ) {
							model().unbind(peer());
							validate();
							((Component) model().peer()).repaint();
						} // method run
					}; // Anonymous Class Thread
					t.start();
				} catch (Exception err) {}
			} // method actionPerformed
			}); // actionListener
		} // end if menuitemUnBind
		return(menuitemUnBind);
	} // reader menuitemUnBind
	public JMenuItem menuitemUnBind ( JMenuItem m ) { return( menuitemUnBind = m ); }
	
	protected JMenuItem menuitemVariablize = null;
	public JMenuItem menuitemVariablize ( ) {
		if (menuitemVariablize == null) {
			menuitemVariablize(new JMenuItem("Variablize"));
			menuitemVariablize.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				final ActionEvent e = ev;
				try {
					Thread t = new Thread() {
						public void run ( ) {
							model().variablize(peer());
							validate();
							((Component) model().peer()).repaint();
						} // method run
					}; // Anonymous Class Thread
					t.start();
				} catch (Exception err) {}
			} // method actionPerformed
			}); // actionListener
		} // end if menuitemVariablize
		return(menuitemVariablize);
	} // reader menuitemVariablize
	public JMenuItem menuitemVariablize ( JMenuItem m ) { return( menuitemVariablize = m ); }
	
	protected JMenuItem menuitemUnVariablize = null;
	public JMenuItem menuitemUnVariablize ( ) {
		if (menuitemUnVariablize == null) {
			menuitemUnVariablize(new JMenuItem("UnVariablize"));
			menuitemUnVariablize.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				final ActionEvent e = ev;
				try {
					Thread t = new Thread() {
						public void run ( ) {
							model().unvariablize(peer());
							validate();
							((Component) model().peer()).repaint();
						} // method run
					}; // Anonymous Class Thread
					t.start();
				} catch (Exception err) {}
			} // method actionPerformed
			}); // actionListener
		} // end if menuitemUnVariablize
		return(menuitemUnVariablize);
	} // reader menuitemUnVariablize
	public JMenuItem menuitemUnVariablize ( JMenuItem m ) { return( menuitemUnVariablize = m ); }
	
	protected JMenuItem menuitemSkin = null;
	public JMenuItem menuitemSkin ( ) {
		if (menuitemSkin == null) {
			menuitemSkin(new JMenuItem("Skin"));
			menuitemSkin.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				final ActionEvent e = ev;
				try {
					Thread t = new Thread() {
						public void run ( ) {
							peer().set("SKINNED", true);
							((Component) model().peer()).repaint();
						} // method run
					}; // Anonymous Class Thread
					t.start();
				} catch (Exception err) {}
			} // method actionPerformed
			}); // actionListener
		} // end if menuitemSkin
		return(menuitemSkin);
	} // reader menuitemSkin
	public JMenuItem menuitemSkin ( JMenuItem m ) { return( menuitemSkin = m ); }
	
	protected JMenuItem menuitemUnSkin = null;
	public JMenuItem menuitemUnSkin ( ) {
		if (menuitemUnSkin == null) {
			menuitemUnSkin(new JMenuItem("UnSkin"));
			menuitemUnSkin.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				final ActionEvent e = ev;
				try {
					Thread t = new Thread() {
						public void run ( ) {
							peer().set("SKINNED", false);
							((Component) model().peer()).repaint();
						} // method run
					}; // Anonymous Class Thread
					t.start();
				} catch (Exception err) {}
			} // method actionPerformed
			}); // actionListener
		} // end if menuitemUnSkin
		return(menuitemUnSkin);
	} // reader menuitemUnSkin
	public JMenuItem menuitemUnSkin ( JMenuItem m ) { return( menuitemUnSkin = m ); }
	
	protected JMenuItem menuitemSkinColor = null;
	public JMenuItem menuitemSkinColor ( ) {
		if (menuitemSkinColor == null) {
			menuitemSkinColor(new JMenuItem("Change Skin Color"));
			menuitemSkinColor.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				final ActionEvent e = ev;
				try {
					Thread t = new Thread() {
						public void run ( ) {
							Color c =  peer().getColor("GROUPCOLOR");
							if (c == BLACK) c = GREEN;
							c = JColorChooser.showDialog(null, "Select Skin Color", c);
							if (c != null) {
								peer().set("GROUPCOLOR", c.getRGB());
								self().getParent().repaint();
							} // end if c
						} // method run
					}; // Anonymous Class Thread
					t.start();
				} catch (Exception err) {}
			} // method actionPerformed
			}); // actionListener
		} // end if menuitemSkinColor
		return(menuitemSkinColor);
	} // reader menuitemSkinColor
	public JMenuItem menuitemSkinColor ( JMenuItem m ) { return( menuitemSkinColor = m ); }
	
	public void makePopupMenu ( ) {
		menu().removeAll();
		menu().add(menuitemAnnotate());
		menu().add(menuitemHelp());
	} // method makePopupMenu

	public void showPopupMenu (int x, int y ) {
		makePopupMenu();
		//menu.show(gui().frame(), x, y); 
		menu.show(this, x, y); 
	} // method showPopupMenu
		
	// Constructors
	// --------------------------------------------------------------------------
	public VModelPeerElement (VModelGUI gui, VModelModel model, VModelElement peer ) {
		super(gui);
		peer(peer);
		peer().peer(this);
		self(this);
		model(model);
		
		setBackground(peer().getColor("COLOR"));
		setOpaque(false);
		addMouseListener(new PeerMouseMotionAdapter());
		addMouseMotionListener(new PeerMouseMotionAdapter());
		
		displayMode(displayMode());
		setLocation(peer().getInt("XOFFSET"),peer().getInt("YOFFSET"));
		setSize(getSize().width+6, getSize().height+6);
		
		//setPreferredSize(getSize());
		setBorder(new CompoundBorder(new EmptyBorder(2,2,2,2),new ComponentLineBorder(2)));
		setVisible(true);
	   setSize(getPreferredSize().width, getHeight());
	   setAutoscrolls(true);
	} // constructor
	
} // class VModelGUIComponent

// -----------------------------------------------------------------------------
// End of File: VModelGUIComponent.java        (c) 2001 Northwestern University
