package edu.qrg.vmodel.controllers.classic;
// Sun Java 1.3.1 -Sourcepath QRG/VModel/v4/code/src
// -----------------------------------------------------------------------------
// File name: VModelGUIModelFrame.java
//    System: VModel
//   Version: 4.0
//  Abstract: VModel is a visual representation system developed to provide 
//            computer support to students learning how to create, articulate,
//            and reason with scientific models.  There is a special emphasis on
//            supporting the reuse and integration of previously modeled
//            material in the course of making new models. 
//    Author: Leo C. Ureel II
//   Created: March 7, 1999 18:52:54
//   Purpose: The VModelGUIModelFrame represents the top-level VModel window.  
//            The event driven nature of this system gives this class a great 
//            deal of control over the flow and interpretation of data.
// -----------------------------------------------------------------------------
// (c)2001 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
// -----------------------------------------------------------------------------
// Modified: Friday, June 22, 2001 at 12:10:52 by ureel
// -----------------------------------------------------------------------------
import java.awt.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*; //EtchedBorder
import java.util.*;
import javax.swing.tree.*;
import java.awt.image.*;
import java.beans.*;
import java.awt.event.*;

// -----------------------------------------------------------------------------
// Class: BuildPane
// -----------------------------------------------------------------------------
public class VModelPeerModel extends VModelPanel implements VModelConstants, VModelPeer, ComponentListener {
	// Properties & Accessors
	// --------------------------------------------------------------------------
	protected VModelElement peer = null;
	public VModelElement peer ( ) { return(peer); }
	public VModelElement peer ( VModelElement p ) { return(peer = p); }
	
	protected VModelModel model = null;
	public VModelModel model ( ) { return(model); }
	public VModelModel model ( VModelModel m ) { return(model = m); }
	
	protected ProblemPane problemPane = null;
	public ProblemPane problemPane ( ) { 
		if (problemPane == null) problemPane(new ProblemPane(gui()));
		return(problemPane); 
	} // reader problemPane
	public ProblemPane problemPane ( ProblemPane p ) { return(problemPane = p); }
	
	protected VModelCAD cadPane = null;
	public VModelCAD cadPane ( ) { 
		if (cadPane == null) {
			cadPane(new VModelCAD(gui(), model(), peer()));
		} // end if cadPane
		return(cadPane); 
	} // reader cadPane
	public VModelCAD cadPane ( VModelCAD p ) { return(cadPane = p); }
	
	// Methods
	// --------------------------------------------------------------------------
	public int displayMode ( int m ) {
		peer().set("DISPLAY_MODE", m);
		//removeAll();
		switch(m) {
			case MODEL_MODE: {
				problemPane().setVisible(false);
				cadPane().setVisible(true);
				cadPane().setSize(getSize().width, getSize().height);
				break;
			} // case BOX_MODE
			case PROBLEM_MODE: {
				cadPane().setVisible(false);
				problemPane().setText(model().problemStatement());
				problemPane().setVisible(true);
				break;
			} // case OBJECT_MODE
			default: {
				displayMode(MODEL_MODE);
			} // case OBJECT_MODE
		} // end switch
		validate();
		return(peer().getInt("DISPLAY_MODE"));
	} // method displayMode

	public void groupSelected ( ) {
		VModelElement newGroup = model().makeGroup();
		int x = cadPane().hbar().getValue();
		int y = cadPane().vbar().getValue();
		int w = cadPane().view().getExtentSize().width;
		int h = cadPane().view().getExtentSize().height;
		String label = JOptionPane.showInputDialog("What would you like to call th new group?");
		if ((label == null) || (label.equals(""))) return;
		newGroup.set("LABEL", label);
		newGroup.set("XOFFSET", ((int) ((x + w)/2)));
		newGroup.set("YOFFSET", (int) ((y + h)/2));
		newGroup.set("HAS-EXTENDED-ELEMENTS", false);
		newGroup.set("SKINNED", false);
		newGroup.set("DISPLAY_MODE", BOX_MODE);
		model().addGroup(newGroup);
		VModelPeerGroup grumble = new VModelPeerGroup(gui(), model(), newGroup);
		grumble.displayMode(BOX_MODE);
		cadPane().componentpane().add(grumble);
		grumble.groupSelected(grumble);
		grumble.setSize(grumble.getPreferredSize().width, grumble.getPreferredSize().height);
		validate();
		repaint();
	} // method groupSelected

	// ComponentListener Methods
	// -----------------------------------------------------------------------------
	public void componentHidden(ComponentEvent e) {}
	public void componentMoved(ComponentEvent e) {
    	cadPane().setSize(getSize().width, getSize().height);
	} // method componentMoved
	public void componentResized(ComponentEvent e) {
		cadPane().setSize(getSize().width, getSize().height);
	} // method componentResized
	public void componentShown(ComponentEvent e) { }

	// InnerClass: ProblemPane
	// --------------------------------------------------------------------------
	public boolean problemPaneUpdateFlag = false;

/*	
	public class ProblemPane extends VModelPanel implements VModelConstants, DocumentListener {
		public JLabel problemicon = null;
		public JTextArea problemlabel = null;
		public JTextArea problemexample = null;
		public JTextArea problemtext = null;
		public Color color = null;
		public JScrollPane scrollproblemtext = null;
		public JScrollPane scrollproblemlabel = null;
		public JScrollPane scrollproblemexample = null;
		public String originalProblemStatement = "";
		
		public ProblemPane (VModelGUI gui) {
			super(gui);
			
			originalProblemStatement = model().problemStatement();
			
			setOpaque(true);
			color = gui().preferences().getColor("COLOR");
			setBackground(Color.white);
			
			problemicon = new JLabel(gui.getImage("wizard.gif"));
			problemicon.setBorder(new EmptyBorder(5,5,5,5));
			add(problemicon, BorderLayout.WEST);
			problemicon.hide();
			
			problemexample = new JTextArea("An example of a problem statement is: \"This model will explain why the level of the ocean is rising.\"");
			problemexample.setBackground(Color.white);
			problemexample.setBorder(new EmptyBorder(5,5,5,5));
			problemexample.setLineWrap(true);
			problemexample.setWrapStyleWord(true);
			problemexample.setRows(4);
			problemexample.setEditable(false);
			scrollproblemexample = new JScrollPane(problemexample, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
			scrollproblemexample.setBackground(color);
			add(scrollproblemexample, BorderLayout.SOUTH);
			//problemexample.hide();
			
			problemtext = new JTextArea(model.getString("PROBLEM_STATEMENT"));
			problemtext.setBackground(color);
			problemtext.setLineWrap(true);
			problemtext.setWrapStyleWord(true);
			problemtext.setRows(4);
			problemtext.getDocument().addDocumentListener(this);
			scrollproblemtext = new JScrollPane(problemtext, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
			scrollproblemtext.setBackground(color);
			scrollproblemtext.setBorder(new TitledBorder(new LineBorder(Color.black, 2), "Problem Statement"));
			add(scrollproblemtext, BorderLayout.CENTER);
			
			problemlabel = new JTextArea("Every model explains something that is happening in the world. Models that you make with Vmodel explain how something changes.\n\nThe thing that changes is usually something that can be measured with a number, such as amount, rate, level, etc.\n\nIn the space below, please write what this model will be explaining.\nYou should specify the thing that is changing (e.g. rate, level, amount, etc.) and how it is changing (increasing or decreasing.)");//,4,0,TextArea.SCROLLBARS_VERTICAL_ONLY);
			problemlabel.setBackground(Color.white);
			problemlabel.setLineWrap(true);
			problemlabel.setWrapStyleWord(true);
			problemlabel.setRows(4);
			problemlabel.setEditable(false);
			scrollproblemlabel = new JScrollPane(problemlabel, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
			scrollproblemlabel.setBackground(color);
			add(scrollproblemlabel, BorderLayout.NORTH);
			//validate();
			//problemlabel.hide();
		} // constructor ProblemPane


	
		public void insertUpdate(DocumentEvent e) { textUpdate( ); }
		public void removeUpdate(DocumentEvent e) { textUpdate( ); }
		public void changedUpdate(DocumentEvent e) { textUpdate( ); }
	
		public void textUpdate( ) { 
			problemPaneUpdateFlag = true; 
			if (model().peer() != null) ((VModelFrame) model().peer()).dirty(true);
		} // method textUpdate
		
		public String getText ( ) {
			return(problemtext.getText());
		} // method getText
		
		public void setText ( String s ) {
			problemtext.setText(s);
		} // method getText
		
		public void updateProblemStatement ( ) {
	QRGutils.debug("VModelPeerModel","updateProblemStatement");
			if (problemPaneUpdateFlag) {
				model().problemStatement(getText());
				originalProblemStatement = getText();
				problemPaneUpdateFlag = false;
			} // end if problemPaneUpdateFlag
		} // method changeMode
	} // class ProblemPane
*/

	public class ProblemPane extends VModelPanel implements VModelConstants /*, DocumentListener */ {
		 
		public JLabel problemicon = null;
		public JTextArea explainarea = null;
		public Color color = null;
		public String originalProblemStatement = "";
		public String problemtext = "";
      ButtonGroup pschoice = new ButtonGroup(); 
		public JRadioButton brain = null;
		public JRadioButton describe = null;
		public JRadioButton modelit = null;

		public ProblemPane (VModelGUI gui) {
			super(gui); //??
			setBackground (color = gui().preferences().getColor("COLOR"));
                  setLayout(new GridLayout(4, 1));
			JPanel choicearea = new JPanel(new GridLayout(4, 1));
		brain = new JRadioButton("brainstorm.", false);
		describe = new JRadioButton("describe and relate basic objects.", false);
		modelit = new JRadioButton("make a model about a change in amounts.", false);
		brain.setOpaque(false);
		describe.setOpaque(false);
		modelit.setOpaque(false);
		pschoice.add(brain);
		pschoice.add(describe);
		pschoice.add(modelit);
			choicearea.setLayout(new GridLayout(4, 1));
			choicearea.setBackground(Color.white);
				JTextArea choicelabel = new JTextArea("I am making a model to...");
					choicelabel.setBackground(Color.white);
					choicelabel.setLineWrap(true);
					choicelabel.setWrapStyleWord(true);
					choicelabel.setRows(4);
					choicelabel.setEditable(false);
					choicearea.add(choicelabel);
					choicearea.add(brain); 
					choicearea.add(describe);
					choicearea.add(modelit);
			add(choicearea);
			JTextArea problemexplanation = new JTextArea("If you are making a model to explain a change, please write what that change is in the space below.\n\nThe thing that changes is usually something that can be measured with a number, such as amount, rate, level, etc. You should say if it is increasing or decreasing.");//,");
		      problemexplanation.setBackground (WHITE);
			problemexplanation.setLineWrap(true);
			problemexplanation.setWrapStyleWord(true);
			add(problemexplanation);
                  
			explainarea = new JTextArea();
			explainarea.setBackground(color = gui().preferences().getColor("COLOR"));	
			explainarea.setBorder(new TitledBorder(new LineBorder(Color.black, 2), "Problem Statement"));		
			add(explainarea);

			JTextArea problemstatement = new JTextArea("For example: This model will explain why the level of the ocean is rising.");
		      problemstatement.setBackground (WHITE);
			add(problemstatement, BorderLayout.SOUTH);
}//ends problempane

		public void updateProblemStatement ( ) {
	QRGutils.debug("VModelPeerModel","updateProblemStatement");
			//if (problemPaneUpdateFlag) {
				model().problemStatement(getText());
				originalProblemStatement = getText();
				problemPaneUpdateFlag = false;
			//} // end if problemPaneUpdateFlag
		} // method changeMode

		public String getText ( ) {
			String foo = explainarea.getText();
			if ((foo == null) || (foo.equals("") || (foo.equalsIgnoreCase("???")))) {
				if (brain.isSelected()) {
					foo = brain.getLabel();
					explainarea.setText(foo);
				} else if (describe.isSelected()) {
					foo = describe.getLabel();
					explainarea.setText(foo);
				}
			} // end if foo
			return(foo);
		} // method getText
		
		public void setText ( String s ) {
			explainarea.setText(s);
		} // method getText
		


}//ends xxx



	public void updateProblemStatement ( ) {
		problemPane().updateProblemStatement();
	} // method changeMode

	// Constructors
	// --------------------------------------------------------------------------
	public VModelPeerModel (VModelGUI gui, VModelModel model, VModelElement peer ) {
		super(gui);
		model(model);
		peer(peer);
		peer.peer(this);
		add(cadPane(), BorderLayout.CENTER);
		cadPane().setVisible(false);
		add(problemPane(), BorderLayout.CENTER);
		problemPane().setVisible(true);
		addComponentListener(this);
		setBackground(RED);
problemPane().setText(model().problemStatement());
	} // constructor
	
} // class VModelGUIModelScrollPane


// -----------------------------------------------------------------------------
// End of File: VModelGUIModelFrame.java        (c) 2001 Northwestern University
