package edu.qrg.vmodel.controllers.classic;
// Sun Java 1.3.1 -Sourcepath QRG/VModel/v5/code/source
// -----------------------------------------------------------------------------
// File name: VModelGUIModelFrame.java
//    System: VModel
//   Version: 5.0
//  Abstract: VModel is a visual representation system developed to provide 
//            computer support to students learning how to create, articulate,
//            and reason with scientific models.  There is a special emphasis on
//            supporting the reuse and integration of previously modeled
//            material in the course of making new models. 
//    Author: Leo C. Ureel II
//   Created: March 7, 1999 18:52:54
//   Purpose: The VModelGUIModelFrame represents the top-level VModel window.  
//            The event driven nature of this system gives this class a great 
//            deal of control over the flow and interpretation of data.
// -----------------------------------------------------------------------------
// (c)2001 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
// -----------------------------------------------------------------------------
// Modified: Friday, June 22, 2001 at 12:10:52 by ureel
// -----------------------------------------------------------------------------
import java.awt.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*; //EtchedBorder
import java.util.*;
import javax.swing.tree.*;
import java.awt.image.*;
import java.beans.*;
import java.awt.event.*;

// -----------------------------------------------------------------------------
// Class: VModelPeerElement
// -----------------------------------------------------------------------------
public class VModelPeerRelationship extends VModelPeerElement implements VModelConstants {
	// Properties & Accessors
	// --------------------------------------------------------------------------
	protected JLabel north = null;
	public JLabel north ( ) { return( north ); }
	public JLabel north ( JLabel l ) { return( north = l ); }

	protected JLabel south = null;
	public JLabel south ( ) { return( south ); }
	public JLabel south ( JLabel l ) { return( south = l ); }

	protected JLabel east = null;
	public JLabel east ( ) { return( east ); }
	public JLabel east ( JLabel l ) { return( east = l ); }

	protected JLabel west = null;
	public JLabel west ( ) { return( west ); }
	public JLabel west ( JLabel l ) { return( west = l ); }

	protected ImageIcon up = null;
	public ImageIcon up ( ) { return( up ); }
	public ImageIcon up ( ImageIcon i ) { return( up = i ); }

	protected ImageIcon down = null;
	public ImageIcon down ( ) { return( down ); }
	public ImageIcon down ( ImageIcon i ) { return( down = i ); }

	protected ImageIcon left = null;
	public ImageIcon left ( ) { return( left ); }
	public ImageIcon left ( ImageIcon i ) { return( left = i ); }

	protected ImageIcon right = null;
	public ImageIcon right ( ) { return( right ); }
	public ImageIcon right ( ImageIcon i ) { return( right = i ); }

	protected Color color = null;
	public Color color ( ) { return( color ); }
	public Color color ( Color c ) { return( color = c ); }

	protected VModelPeerMarker inputMarker = null;
	public VModelPeerMarker inputMarker ( ) { return( inputMarker ); }
	public VModelPeerMarker inputMarker ( VModelPeerMarker m ) { return( inputMarker = m ); }

	protected VModelPeerMarker outputMarker = null;
	public VModelPeerMarker outputMarker ( ) { return( outputMarker ); }
	public VModelPeerMarker outputMarker ( VModelPeerMarker m ) { return( outputMarker = m ); }

	// Methods
	// --------------------------------------------------------------------------
	public void twist ( ) {
		if (peer().getString("ORIENTATION").equalsIgnoreCase("NORTH"))
			setOrientation("EAST");
		else if (peer().getString("ORIENTATION").equalsIgnoreCase("EAST"))
			setOrientation("SOUTH");
		else if (peer().getString("ORIENTATION").equalsIgnoreCase("SOUTH"))
			setOrientation("WEST");
		else setOrientation("NORTH");
	} // method setOrientation

	public void flip ( ) {
		if (peer().getString("ORIENTATION").equalsIgnoreCase("NORTH"))
			setOrientation("SOUTH");
		else if (peer().getString("ORIENTATION").equalsIgnoreCase("EAST"))
			setOrientation("WEST");
		else if (peer().getString("ORIENTATION").equalsIgnoreCase("SOUTH"))
			setOrientation("NORTH");
		else setOrientation("EAST");
	} // method setOrientation

	public void setOrientation ( ) {
		setOrientation(peer().getString("ORIENTATION"));
	} // method setOrientation

	public void setOrientation ( String orientation ) {
		north().setVisible(false);
		south().setVisible(false);
		east().setVisible(false);
		west().setVisible(false);
		if (orientation.equalsIgnoreCase("NORTH")) {
			north().setIcon(up);
			if (peer().getBoolean("DIRECTIONAL")) south().setIcon(up);
			else south().setIcon(down);
			north().setVisible(true);
			south().setVisible(true);
		} else if (orientation.equalsIgnoreCase("SOUTH")) {
			south().setIcon(down);
			if (peer().getBoolean("DIRECTIONAL")) north().setIcon(down);
			else north().setIcon(up);
			north().setVisible(true);
			south().setVisible(true);
		} else if (orientation.equalsIgnoreCase("EAST")) {
			east().setIcon(right);
			if (peer().getBoolean("DIRECTIONAL")) west().setIcon(right);
			else west().setIcon(left);
			east().setVisible(true);
			west().setVisible(true);
		} else {
			orientation = "WEST";
			west().setIcon(left);
			if (peer().getBoolean("DIRECTIONAL")) east().setIcon(left);
			else east().setIcon(right);
			east().setVisible(true);
			west().setVisible(true);
		} // end if orientation
		peer().set("ORIENTATION", orientation);
		setSize(getPreferredSize().width, getPreferredSize().height);
		if (getParent() != null) getParent().repaint();		
		validate();
	} // method setOrientation

	// VModelPeerElement Methods
	// --------------------------------------------------------------------------
	protected JMenuItem menuitemReverse = null;
	public JMenuItem menuitemReverse ( ) {
		if (menuitemReverse == null) {
			menuitemReverse(new JMenuItem("Reverse"));
			menuitemReverse.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					final ActionEvent e = ev;
					try {
						Thread t = new Thread() {
							public void run ( ) {
								model().reverse(peer());
								flip();
							} // method run
						}; // Anonymous Class Thread
						t.start();
					} catch (Exception err) {}
				} // method actionPerformed
			}); // actionListener
		} // end if menuitemReverse
		return(menuitemReverse);
	} // reader menuitemReverse
	public JMenuItem menuitemReverse ( JMenuItem m ) { return( menuitemReverse = m ); }
	
	protected JMenuItem menuitemTwist = null;
	public JMenuItem menuitemTwist ( ) {
		if (menuitemTwist == null) {
			menuitemTwist(new JMenuItem("Twist"));
			menuitemTwist.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					final ActionEvent e = ev;
					try {
						Thread t = new Thread() {
							public void run ( ) {
								twist();
							} // method run
						}; // Anonymous Class Thread
						t.start();
					} catch (Exception err) {}
				} // method actionPerformed
			}); // actionListener
		} // end if menuitemTwist
		return(menuitemTwist);
	} // reader menuitemTwist
	public JMenuItem menuitemTwist ( JMenuItem m ) { return( menuitemTwist = m ); }
	
	protected JMenuItem menuitemFlip = null;
	public JMenuItem menuitemFlip ( ) {
		if (menuitemFlip == null) {
			menuitemFlip(new JMenuItem("Flip"));
			menuitemFlip.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					final ActionEvent e = ev;
					try {
						Thread t = new Thread() {
							public void run ( ) {
								flip();
							} // method run
						}; // Anonymous Class Thread
						t.start();
					} catch (Exception err) {}
				} // method actionPerformed
			}); // actionListener
		} // end if menuitemFlip
		return(menuitemFlip);
	} // reader menuitemFlip
	public JMenuItem menuitemFlip ( JMenuItem m ) { return( menuitemFlip = m ); }
	
	protected JMenuItem menuitemDisconnectInput = null;
	public JMenuItem menuitemDisconnectInput ( ) {
		if (menuitemDisconnectInput == null) {
			menuitemDisconnectInput(new JMenuItem("Disconnect Input"));
			menuitemDisconnectInput.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					final ActionEvent e = ev;
					try {
						Thread t = new Thread() {
							public void run ( ) {
								model().disconnect(peer(), peer().input());
								((JComponent) model().peer()).repaint();
							} // method run
						}; // Anonymous Class Thread
						t.start();
					} catch (Exception err) {}
				} // method actionPerformed
			}); // actionListener
		} // end if menuitemDisconnectInput
		return(menuitemDisconnectInput);
	} // reader menuitemDisconnectInput
	public JMenuItem menuitemDisconnectInput ( JMenuItem m ) { return( menuitemDisconnectInput = m ); }
	
	protected JMenuItem menuitemDisconnectOutput = null;
	public JMenuItem menuitemDisconnectOutput ( ) {
		if (menuitemDisconnectOutput == null) {
			menuitemDisconnectOutput(new JMenuItem("Disconnect Output"));
			menuitemDisconnectOutput.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ev) {
					final ActionEvent e = ev;
					try {
						Thread t = new Thread() {
							public void run ( ) {
								model().disconnect(peer(), peer().output());
								((JComponent) model().peer()).repaint();
							} // method run
						}; // Anonymous Class Thread
						t.start();
					} catch (Exception err) {}
				} // method actionPerformed
			}); // actionListener
		} // end if menuitemDisconnectOutput
		return(menuitemDisconnectOutput);
	} // reader menuitemDisconnectOutput
	public JMenuItem menuitemDisconnectOutput ( JMenuItem m ) { return( menuitemDisconnectOutput = m ); }
	
	public void makePopupMenu ( ) {
		super.makePopupMenu();
		//menu().addSeparator();

					JMenuItem coachMenuItem = gui().makeCoachMenuItem("Grammar Coach",peer());
					if (model().checkConnectable(peer())) coachMenuItem.setIcon(gui().getImage("smiley-coach-small.gif"));
					else coachMenuItem.setIcon(gui().getImage("puzzle-coach-small.gif"));
					menu.add(coachMenuItem);

		if (!peer().typep("DESCRIBER")) {
			menu().addSeparator();
			menu().add(menuitemReverse());
		} // end if reversable
		menu().addSeparator();
		menu().add(menuitemTwist());
		menu().add(menuitemFlip());
		if ((peer().input() != null) || (peer().output() != null) || (!peer().connections().isEmpty())) {
			menu().addSeparator();
			menu().add(menuitemDisconnectAll());
		} // end if connections
		if (peer().input() != null) {
			menuitemDisconnectInput().setText("Disconnect " + peer().input().getString("TYPE") + ": " + model.label(peer().input()));
			menu().add(menuitemDisconnectInput());
		} // end if input
		if (peer().output() != null) {
			menuitemDisconnectOutput().setText("Disconnect " + peer().output().getString("TYPE") + ": " + peer().output().getString("LABEL"));
			menu().add(menuitemDisconnectOutput());
		} // end if input
		if (!peer().connections().isEmpty()) {
			menu().add(menuitemDisconnectConnections());
		} // end if input
	} // method makePopupMenu

	public void showPopupMenu (int x, int y ) {
		super.showPopupMenu(x, y); 
	} // method showPopupMenu
		
	public int displayMode ( int m ) {
		peer().set("DISPLAY_MODE", RELATIONSHIP_MODE);
		return(RELATIONSHIP_MODE);
	} // method displayMode
	
	// Constructors
	// --------------------------------------------------------------------------
	public VModelPeerRelationship (VModelGUI gui, VModelModel model, VModelElement peer ) {
		super(gui, model, peer);
		VModelPanel relationshipPane = new VModelPanel(gui);
		JLabel typelabel = new JLabel(peer().getString("TYPE"));
		VModelPanel lpanel = makeLinksPanel(gui);
		if (peer().getBoolean("EDITABLE")) {
			label(new JTextField(peer().getString("LABEL")));
			label().setBorder(new LineBorder(peer().getColor("COLOR"), 1));
			label().addFocusListener(this);
			label().addKeyListener(new TextKeyAdapter());
		} // end if editable
		typelabel.setForeground(WHITE);
		//typelabel.setBackground(peer().getColor("COLOR"));
		lpanel.setBackground(peer().getColor("COLOR"));
		if (peer().getBoolean("EDITABLE")) lpanel.setSize(label().getPreferredSize().width+lpanel.getPreferredSize().width, 18);
		else lpanel.setSize(typelabel.getPreferredSize().width+lpanel.getPreferredSize().width, 18);
		//lpanel.setPreferredSize(lpanel.getSize());
		lpanel.setBorder(new EmptyBorder(1,1,1,1));
		if (peer().getBoolean("EDITABLE")) lpanel.add(label(), 0);
		else lpanel.add(typelabel, 0);
		lpanel.revalidate();
		//lpanel.setOpaque(false);
		relationshipPane.setOpaque(false);
		relationshipPane.add(lpanel, BorderLayout.CENTER);
		relationshipPane.revalidate();
		relationshipPane.setLocation(peer().getInt("XOFFSET"), peer().getInt("YOFFSET"));
		relationshipPane.setSize(Math.max(lpanel.getPreferredSize().width, relationshipPane.getPreferredSize().width), relationshipPane.getPreferredSize().height-3);
		add(relationshipPane, BorderLayout.CENTER);
		color(peer().getColor("COLOR"));
		if (color() == BLUE) {
			up(gui().getImage("blue-up-arrow.gif"));
			down(gui().getImage("blue-down-arrow.gif"));
			left(gui().getImage("blue-left-arrow.gif"));
			right(gui().getImage("blue-right-arrow.gif"));
		} else if (color() == PINK) {
			up(gui().getImage("pink-up-arrow.gif"));
			down(gui().getImage("pink-down-arrow.gif"));
			left(gui().getImage("pink-left-arrow.gif"));
			right(gui().getImage("pink-right-arrow.gif"));
		} else if (color() == DARKRED) {
			up(gui().getImage("red-up-arrow.gif"));
			down(gui().getImage("red-down-arrow.gif"));
			left(gui().getImage("red-left-arrow.gif"));
			right(gui().getImage("red-right-arrow.gif"));
		} else {
			up(gui().getImage("black-up-arrow.gif"));
			down(gui().getImage("black-down-arrow.gif"));
			left(gui().getImage("black-left-arrow.gif"));
			right(gui().getImage("black-right-arrow.gif"));
		} // end if color
		add(north(new JLabel()), BorderLayout.NORTH);
		add(south(new JLabel()), BorderLayout.SOUTH);
		add(east(new JLabel()), BorderLayout.EAST);
		add(west(new JLabel()), BorderLayout.WEST);
		north().setHorizontalAlignment(SwingConstants.CENTER);
		south().setHorizontalAlignment(SwingConstants.CENTER);
		setOrientation();
	} // constructor
} // class VModelGUIComponent

// -----------------------------------------------------------------------------
// End of File: VModelGUIComponent.java        (c) 2001 Northwestern University
