package edu.qrg.vmodel.controllers.classic;
// Sun Java 1.3.1 -Sourcepath QRG/VModel/v4/code/source
// -----------------------------------------------------------------------------
// File name: VModelPreferences.java
//    System: VModel
//   Version: 4.0
//    Author: Leo C. Ureel II
//   Created: March 7, 1999 18:52:54
//   Purpose: VModel preferences facility.
// -----------------------------------------------------------------------------
// (c)2001 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
// -----------------------------------------------------------------------------
//  Modified: Thursday, June 21, 2001 at 12:10:52 by ureel
// -----------------------------------------------------------------------------
import java.util.*;
import java.io.*;
import java.lang.*;
import java.awt.*;
import javax.swing.*;

// -----------------------------------------------------------------------------
// Notes:
// * Does simplifying a life mean giving up values?  Implications to me?
// * Do we need to save globalPrefs anymore? I ask because all values are
//   now set only in localPrefs.  See the set function as oppossed to get.
// -----------------------------------------------------------------------------

// -----------------------------------------------------------------------------
// Class: VModelPreferences 
// Description: VModel preferences facility.
// -----------------------------------------------------------------------------
public class VModelPreferences implements VModelConstants, SwingConstants {
	// Properties & Accessors
	// --------------------------------------------------------------------------
	protected static VModelElement globalPreferences = null;
	public static VModelElement globalPreferences ( ) { 
		if (globalPreferences == null) globalPreferences(new VModelElement("PREFERENCES"));
		return(globalPreferences); 
	} // reader globalPreferences
	public static VModelElement globalPreferences ( VModelElement prefs ) { 
		return(globalPreferences = prefs); 
	} // writer globalPreferences

	protected static VModelElement localPreferences = null;
	public static VModelElement localPreferences ( ) { 
		if (localPreferences == null) localPreferences(new VModelElement("PREFERENCES"));
		return(localPreferences); 
	} // reader localPreferences
	public static VModelElement localPreferences ( VModelElement prefs ) { 
		return(localPreferences = prefs); 
	} // writer localPreferences

	protected static VModelElement dataPreferences = null;
	public static VModelElement dataPreferences ( ) { 
		if (dataPreferences == null) dataPreferences(new VModelElement("PREFERENCES"));
		return(dataPreferences); 
	} // reader dataPreferences
	public static VModelElement dataPreferences ( VModelElement prefs ) { 
		return(dataPreferences = prefs); 
	} // writer dataPreferences

	protected static VModelSystem system = null;
	public static VModelSystem system ( ) { return(system); }
	public static VModelSystem system ( VModelSystem s ) { return(system = s); }
	
	// File IO
	// --------------------------------------------------------------------------
	public static VModelElement loadGlobalPrefs() {
		VModelElement result = null;
		if (QRGutils.isApplet()) result = load(system().getCodeBase() + INIFILE);
		else result = load(QRGutils.QRGroot() + INIFILE);
		if (result != null) globalPreferences(result);
		return(result);
	} // method loadGlobalPrefs

	public VModelElement loadDataPrefs() {
		String inifile = null;
		VModelElement result = null;
		if (QRGutils.isApplet()) inifile = QRGutils.makeQRGpath("resources2") + DATFILE;
		else inifile = QRGutils.makeQRGpath("resources2") + DATFILE;
		result = load(inifile);
		if (result != null) dataPreferences(result);
		return(result);
	} // method loadDataPrefs

	public static VModelElement loadLocalPrefs( String inifile) {
		VModelElement result = load(inifile);
		if (result != null) localPreferences(result);
		return(result);
	} // method loadLocalPrefs

	public static VModelElement load( String inifile) {
		VModelElement result = VModelElement.load(inifile);
		if (result != null) result.set("FULLFILEPATH", inifile);
		else {
			QRGutils.warn("VModel","Unable to load preference file " + inifile + ".");
		} // end if result else
		return(result);
	} // method load

	public static void saveGlobalPrefs ( ) {
		if (globalPreferences() != null) {
			String filename = (String) globalPreferences().get("FULLFILEPATH");
			if (filename != null) globalPreferences().save(filename, PREFERENCES_DOCTYPE);
		} // end if globalPreferences
	} // method saveGlobalPrefs
	
	public static void saveLocalPrefs ( ) {
		if (localPreferences() != null) {
			String filename = localPreferences().getString("FULLFILEPATH");
			if (filename != null) localPreferences().save(filename, PREFERENCES_DOCTYPE);
			QRGutils.debug("VModelPreferences","saveLocalPrefs filename=" + filename);
		} // end if localPreferences
	} // method savelocalPrefs
	
	public void save ( ) {
		system.updatePreferences();
		saveLocalPrefs();
		saveGlobalPrefs();
	} // method save


	// Get attribute values.
	// -----------------------------------------------------------------------
    public Object get ( String tag ) {
		Object result = localPreferences().get(tag);
		if (result == null) result = globalPreferences().get(tag);
		if (result == null) result = dataPreferences().get(tag);
		if (result == null) 
			QRGutils.debug("PREFERENCES", "All preferences = null for get("+tag+")");
		return(result);
    } // method get

    public Color getColor ( String tag ) {
		Color result = Color.black;
		Object value = null;
		value = localPreferences.get(tag);
		if (value == null) value = globalPreferences().get(tag);
		if (value == null) value = dataPreferences().get(tag);
		if ( value != null ) result = QRGutils.matchColor((String) value);
		else QRGutils.debug("PREFERENCES", "getColor("+tag+") returned null.");
		return result;
    } // method getColor

    public int getRGBColor ( String tag ) {
		int result = 0;
		Color color = getColor(tag);
		if (color != null) result = color.getRGB();
		else QRGutils.debug("PREFERENCES", "getRGBColor("+tag+") returned null.");
		return(result);
    } // method getRGBColor

    public int getInt ( String tag ) {
		int result = 0;
		String value = (String) get(tag);
		if (value != null) {
			value = value.trim();
			try {result = Integer.parseInt(value);}
			catch(NumberFormatException e) {}
		} else QRGutils.debug("PREFERENCES", "Tag not found for getInt("+tag+")");
		return result;
    } // method getInt

    public double getDouble ( String tag ) {
		double result = 0.0;
		String value = (String) get(tag);
		if (value != null) {
			value = value.trim();
			try {result = Double.valueOf(value).doubleValue();}
			catch(NumberFormatException e) {}
		} else QRGutils.debug("PREFERENCES", "Tag not found for getDouble("+tag+")");
		return(result);
    } // method getDouble

    public String getString ( String tag ) {
		String result = "";
		String value = (String) get(tag);
		if (value != null) result = value;
		else QRGutils.debug("PREFERENCES", "Tag not found for getString("+tag+")");
		return(result);
    } // method getString

    public boolean getBoolean ( String tag ) {
		boolean result = false;
		String value = (String) get(tag);
		if (value != null) {
			value = value.trim();
			result = value.equalsIgnoreCase("TRUE");
		} else QRGutils.debug("PREFERENCES", "Tag not found for getBoolean("+tag+")");
		return(result);
    } // method getBoolean

	// Get attribute values from a child element.
	// -----------------------------------------------------------------------
    public Object get ( String etag, String atag ) {
		Object result = null;
		Object value = null;
		VModelElement element = null;
		element = localPreferences().findElement(etag);
		if (element != null) value = element.get(atag);
		if (value == null) {
			element = globalPreferences().findElement(etag);
			if (element != null) value = element.get(atag);
		} // if value
		if (value == null) {
			element = dataPreferences().findElement(etag);
			if (element != null) value = element.get(atag);
		} // if value
		if ( value != null ) result = value;
		else QRGutils.debug("PREFERENCES", "get("+etag+", "+atag+") returned null.");
		return(result);
    } // method get

    public Color getColor ( String etag, String atag ) {
		Color result = null;
		Object value = null;
		VModelElement element = null;
		element = localPreferences().findElement(etag);
		if (element != null) value = element.get(atag);
		if (value == null) {
			element = globalPreferences().findElement(etag);
			if (element != null) value = element.get(atag);
		} // if value
		if (value == null) {
			element = dataPreferences().findElement(etag);
			if (element != null) value = element.get(atag);
		} // if value
		if ( value != null ) result = QRGutils.matchColor((String) value);
		else QRGutils.debug("PREFERENCES", "getColor("+etag+", "+atag+") returned null.");
		return(result);
    } // method getColor

    public int getRGBColor ( String etag, String atag ) {
		int result = 0;
		Color color = getColor(etag, atag);
		if (color != null) result = color.getRGB();
		else QRGutils.debug("PREFERENCES", "getRGBColor("+etag+", "+atag+") returned null.");
		return(result);
    } // method getRGBColor

    public int getInt ( String etag, String atag ) {
		int result = 0;
		String value = (String) get(etag, atag);
		if (value != null) {
			value = value.trim();
			try {result = Integer.parseInt(value);}
			catch(NumberFormatException e) {}
		} else QRGutils.debug("PREFERENCES", "Tag not found for getInt("+etag+", "+atag+")");
		return(result);
    } // method getInt

    public double getDouble ( String etag, String atag ) {
		double result = 0;
		String value = (String) get(etag, atag);
		if (value != null) {
			value = value.trim();
			try {result = Double.valueOf(value).doubleValue();}
			catch(NumberFormatException e) {}
		} else QRGutils.debug("PREFERENCES", "Tag not found for getDouble("+etag+", "+atag+")");
		return(result);
    } // method getDouble

    public String getString ( String etag, String atag ) {
		String result = (String) get(etag, atag);
		if (result == null) {
			result = "";
			QRGutils.debug("PREFERENCES", "Tag not found for getString("+etag+", "+atag+")");
		} // if result
		return(result);
    } // method getString

    public boolean getBoolean ( String etag, String atag ) {
		boolean result = false;
		String value = (String) get(etag, atag);
		if (value != null) {
			value = value.trim();
			result = value.equalsIgnoreCase("TRUE");
		} else QRGutils.debug("PREFERENCES", "Tag not found for getBoolean("+etag+", "+atag+")");
		return(result);
    } // method getBoolean

	// Set attribute values.
	// -----------------------------------------------------------------------
    public Object set ( String tag, Object value ) {
		Object result = null;
		result = localPreferences().set(tag, value);
		return(result);
    } // method set

    public Object set ( String tag, int value ) {
		return(set(tag, "" + value));
    } // method set

    public Object set ( String tag, boolean value ) {
		return(set(tag, "" + value));
    } // method set

    public Object set ( String tag, double value ) {
		return(set(tag, "" + value));
    } // method set

    public Object set ( String tag, Color value ) {
		return(set(tag, "" + value.getRGB()));
    } // method set

	// Set attribute values of a child element.
	// -----------------------------------------------------------------------
    public Object set ( String etag, String atag, Object value ) {
		Object result = null;
		result = localPreferences().set(etag, atag, value);
		return(result);
    } // method set

    public Object set ( String etag, String atag, int value ) {
		return(set(etag, atag, "" + value));
    } // method set

    public Object set ( String etag, String atag, double value ) {
		return(set(etag, atag, "" + value));
    } // method set

    public Object set ( String etag, String atag, boolean value ) {
		return(set(etag, atag, "" + value));
    } // method set

    public Object set ( String etag, String atag, Color value ) {
		return(set(etag, atag, "" + value.getRGB()));
    } // method set

	// Constructors
	// --------------------------------------------------------------------------
	public VModelPreferences( VModelSystem system ) {
		system(system);
		loadGlobalPrefs();
	} // method VModelPreferences

} // class VModelPreferences

// -----------------------------------------------------------------------------
// End of File: VModelPreferences.java          (c) 2001 Northwestern University
