package edu.qrg.vmodel.controllers.classic;
/**
 * Title:        VModel<p>
 * Description:  Using this tool, students express their knowledge in an
 * ever-expanding collection of principles and processes.
 * <p>
 * Copyright:    Copyright (c) Robert T. Harris<p>
 * Company:      Qualitative Reasoning Group<p>
 * @author Robert T. Harris
 * @version 1.0
 */
//package vmodel;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.print.*;
import javax.swing.*;
import javax.swing.border.*;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;

import java.awt.print.Book;
import java.awt.print.Pageable;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.EtchedBorder;

/**
 * Generic print preview panel.  Renders single pages of a Printable object to the
 * screen.  Supports flipping between the pages of a Printable object.
 */
public class VModelGUIPrintPreviewDialog extends JDialog implements ActionListener {
    VModelGUI gui = null;
    VModel system = null;

    /**
     * Creates a Book using the parameters of a Printable object and initializes
     * a this component using the Book.  A Book is used because printable
     * objects do not calculate page breaks or track the page count.
     * @param p       The object to be previewed
     * @param pf      The PageFormat used to render the Printable object.  The
     *                PageFormat is provided by the class implementing the
     *                Printable Interface
     * @param pages   The total number of pages in the printable object
     */
    public VModelGUIPrintPreviewDialog(Printable p, PageFormat pf, int pages) {
        setModal(true);
        Book book = new Book();
        book.append(p, pf, pages);
        layoutUI(book);
	this.book = book;
    }

   /**
    * Initializes this component using a Book object supplied by the user.
    * @param b      Book used to rending the pages of the Printable object
    */
   public VModelGUIPrintPreviewDialog(Book b, VModelGUI gui) {
       this.gui = gui;
       system = gui.system;
       layoutUI(b);
       book = b;
   }

    public void paint(Graphics g) {
	super.paint(g);
        Graphics2D g2 = (Graphics2D)g;
        PageFormat pageFormat = book.getPageFormat(currentPage);

        double xoff; // x offset of page start in window
        double yoff; // y offset of page start in window
        double scale; // scale factor to fit page in window
        double px = pageFormat.getWidth();
        double py = pageFormat.getHeight();
        double ix = pageFormat.getImageableWidth();
        double iy = pageFormat.getImageableHeight();
        double sx = canvas.getWidth() - 1;
        double sy = canvas.getHeight() - 1;

//	QRGutils.debug("DEBUG","px="+px+" ix="+ix+" PW="+system.PAGE_WIDTH+" mw="+gui.selectedFrame.modelwidth+" p="+book.getNumberOfPages());
 //       system.debug("DEBUG","py="+py+" iy="+iy+" PH="+system.PAGE_HEIGHT+" mh="+gui.selectedFrame.modelheight);

        if (px / py < sx / sy) { // center horizontally
            scale = sy / py;
            xoff = 0.5 * (sx - scale * px);
            yoff = 0;
        } else {// center vertically
            scale = sx / px;
            xoff = 0;
            yoff = 0.5 * (sy - scale * py);
        }
        g2.translate((float)xoff, (float)yoff);
        g2.scale((float)scale, (float)scale);
	//  	g2.shear((float)scale/2, (float)scale/2);

        // draw page outline (ignoring margins)
        Rectangle2D page = new Rectangle2D.Double(0, 0, px, py);
	//((Graphics)g2).setClip(0,0,(int)px,(int)py);
  	g2.setClip(page);
//          g2.setPaint(Color.black);
//  	g2.fillRect((int)-xoff,(int)-yoff,(int)px,(int)py);
        g2.setPaint(Color.white);
        g2.fill(page);
        g2.setPaint(Color.black);
        g2.draw(page);


//  	Image pic = gui.selectedFrame.makeModelImage();
//  	g2.drawImage(pic,0,0,null);

        Printable printable = book.getPrintable(currentPage);
        try {
            printable.print(g2, pageFormat, currentPage);
         } catch (PrinterException exception) {
            exception.printStackTrace();
        }
    } // method paint

   /**
    * Initializes the graphical properties of this component.
    * @param book   The Pageable representation of the object being rendered.
    */
   public void layoutUI(Book book) {
      setSize(600, 400);

      Container contentPane = getContentPane();
      //JPanel backingPane = new JPanel();
      canvas = new PrintPreviewCanvas(book,gui);
      canvas.setOpaque(false);
      //backingPane.add(canvas, "Center");
      contentPane.add(canvas, "Center");
      //contentPane.add(backingPane, "Center");

      JPanel buttonPanel = new JPanel();

      nextButton = new JButton("Next");
      buttonPanel.add(nextButton);
      nextButton.addActionListener(this);

      previousButton = new JButton("Previous");
      buttonPanel.add(previousButton);
      previousButton.addActionListener(this);

      closeButton = new JButton("Close");
      buttonPanel.add(closeButton);
      closeButton.addActionListener(this);

      contentPane.add(buttonPanel, "South");
   }

   /**
    * Responds to user interaction with the panel.
    * @param event    The event generated by user interaction
    */
   public void actionPerformed(ActionEvent event) {
      Object source = event.getSource();
      if (source == nextButton) {
          canvas.flipPage(1);
          flipPage(1);
      } else if (source == previousButton) {
          canvas.flipPage(-1);
          flipPage(-1);
      } else if (source == closeButton) {
          setVisible(false);
	  dispose();
      }
   }

   private JButton nextButton;

   private JButton previousButton;

   private JButton closeButton;

   private PrintPreviewCanvas canvas;

   public void flipPage(int offset) {
      int newPage = currentPage + offset;
      if (0 <= newPage && newPage < book.getNumberOfPages()) {
          currentPage = newPage;
          repaint();
      }
   }

   private Book book;
   private int currentPage;
}

/**
 * Canvas used to display the rendered object.
 */
class PrintPreviewCanvas extends JPanel {
    VModelGUI gui = null;
    VModel system = null;
    /**
     * Initializes this component using a Book object supplied by the outter
     * class.  Using a Book facilitates page management.
     * @param b   A Pageable representation of the object being rendered
     */
    public PrintPreviewCanvas(Book b, VModelGUI gui) {
	this.gui = gui;
	system = gui.system;
        book = b;
        currentPage = 0;
    }

    /**
     * Renders this Component and its contents to the screen.  The contents this
     * component are scaled to fit within its boundaries.
     * @param g   The graphics context used to render this component
     */
    public void xpaintComponent(Graphics g) {
        //super.paintComponent(g);
        Graphics2D g2 = (Graphics2D)g;
        PageFormat pageFormat = book.getPageFormat(currentPage);

        double xoff; // x offset of page start in window
        double yoff; // y offset of page start in window
        double scale; // scale factor to fit page in window
        double px = pageFormat.getWidth();
        double py = pageFormat.getHeight();
        double sx = getWidth() - 1;
        double sy = getHeight() - 1;


        if (px / py < sx / sy) { // center horizontally
            scale = sy / py;
            xoff = 0.5 * (sx - scale * px);
            yoff = 0;
        } else {// center vertically
            scale = sx / px;
            xoff = 0;
            yoff = 0.5 * (sy - scale * py);
        }
        g2.translate((float)xoff, (float)yoff);
        g2.scale((float)scale, (float)scale);
//  	g2.shear((float)scale, (float)scale);

        // draw page outline (ignoring margins)
        Rectangle2D page = new Rectangle2D.Double(0, 0, px, py);
	//((Graphics)g2).setClip(0,0,(int)px,(int)py);
  	g2.setClip(page);
//          g2.setPaint(Color.black);
//  	g2.fillRect((int)-xoff,(int)-yoff,(int)px,(int)py);
        g2.setPaint(Color.green);
        g2.fill(page);
        g2.setPaint(Color.black);
        g2.draw(page);


//	Image pic = gui.selectedFrame().makeModelImage();
	//g2.drawImage(pic,0,0,null);

//          Printable printable = book.getPrintable(currentPage);
//          try {
//              printable.print(g2, pageFormat, currentPage);
//           } catch (PrinterException exception) {
//              exception.printStackTrace();
//          }

   }

   /**
    * Changes the visible page by adding the offset to the current page.
    * @param offset   The distance between the current page and the desired page
    */
   public void flipPage(int offset) {
      int newPage = currentPage + offset;
      if (0 <= newPage && newPage < book.getNumberOfPages()) {
          currentPage = newPage;
          repaint();
      }
   }

   private Book book;
   private int currentPage;
}
