package edu.qrg.vmodel.controllers.classic;
// Sun Java 1.3.1 -Sourcepath QRG/VModel/v4/code/src
// -----------------------------------------------------------------------------
// File name: VModelStudent.java
//    System: VModel
//   Version: 4.0
//    Author: Leo C. Ureel II
//   Created: March 7, 1999 18:52:54
//   Purpose: VModel student modeling facility.  This aggregate class 
//            encapsulates a variety of student information.  StudentModeler data 
//            includes: history log, userid, and home path.
// -----------------------------------------------------------------------------
// (c)2001 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
// -----------------------------------------------------------------------------
//  Modified: Thursday, June 21, 2001 at 12:10:52 by ureel
// -----------------------------------------------------------------------------
import java.util.*;
import javax.swing.*;
import java.io.*;
import java.lang.*;
import java.awt.*;

// -----------------------------------------------------------------------------
// Notes:
// * Should the file IO routines be included in this class?
// -----------------------------------------------------------------------------

// -----------------------------------------------------------------------------
// Class: VModelStudent
// Description: VModel student modeling facility.  This aggregate class 
//              encapsulates a variety of student information.  StudentModeler data 
//              includes: history log, userid, and home path.
// -----------------------------------------------------------------------------
public class VModelStudent implements VModelConstants {
	// InnerClasses and Interfaces
	// --------------------------------------------------------------------------

	// Properties and Accessors
	// --------------------------------------------------------------------------
	protected VModel system = null;
	public VModel system ( ) { return(system); }
	public VModel system ( VModel s ) { return(system = s); }

	protected VModelElement student = null;
	public VModelElement student( ) { return( student ); }
	public VModelElement student( VModelElement s ) { return( student = s ); }
	
	// --------------------------------------------------------------------------
	// Method: login
	// Description: Students must login to the system.
	// --------------------------------------------------------------------------
	public VModelElement login ( ) {
		VModelElement result = null;
		String id = null;
		String psswd = null;
		boolean loop = true;
		while(loop == true) {
		    id = JOptionPane.showInputDialog("Enter UserID:");
		    if (id == null) System.exit(0);
		    id = id.trim();
		    id = id.replace(' ', '-');
		    if (id.equalsIgnoreCase("DEVELOPER")) {
				psswd = JOptionPane.showInputDialog("Enter Password:");
				if ((psswd == null) || (!psswd.equalsIgnoreCase("XYZZY")))
			    	System.exit(-1);
		    } // end if
		    if (!id.equals("")) result = load(id);
		    if (result != null) loop = false;
		} // end while
		return(result);
    } // method login

	// File IO.
	// --------------------------------------------------------------------------
    public VModelElement load (String id) {
		VModelElement result = student();
		String path = null;
		String fullpath = null;
		String fullfilepath = null;
		String filename = null;
		File file = null;
		String name = id;
		String email = "";
		String age = "";
		int color = 0;
		Color clr = null;
		id = id.trim();
		id = id.replace(' ', '-');
		if (!id.equals("")) {
		    filename = id + ".ini";
		    path = USERPATH + id + "/";
		    fullpath = QRGutils.QRGroot() + path;
		    fullfilepath = fullpath + filename;
		    if (fullfilepath == null) fullfilepath = path + filename;
		    file = new File(fullfilepath);
		    if ((file != null) && (!file.exists())) {
				// Should ask before creating new user (L8r.) ureel.
				if (JOptionPane.showConfirmDialog(null,"The userid, " + id + " was not found. Would you like to create it?", "UserID not found.", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
				    name = JOptionPane.showInputDialog("What is your name?");
				    age = JOptionPane.showInputDialog("How old are you?");
				    email = JOptionPane.showInputDialog("What is your email address?");
				    clr = JColorChooser.showDialog(null, "What is your favorite color?", Color.white);
				    if (clr != null) color = clr.getRGB();
				    try {
	                    new File(file.getParent()).mkdirs();
	                    new File(file.getParent()+"/models/").mkdirs();
					    new File(file.getParent()+"/groups/").mkdirs();
					    new File(file.getParent()+"/objects/").mkdirs();
					    FileWriter fw = new FileWriter(file);
					    PrintWriter pw = new PrintWriter(fw);
					    pw.println(" ");
					    pw.close();
					    fw.close();
		            } catch (Exception e) {
						JOptionPane.showMessageDialog(null, "Unable To Create New User", "alert", JOptionPane.ERROR_MESSAGE);
						return(null);
				    } // end try - catch
				    result = new VModelElement("STUDENT");
				    result.set("NAME", name);
				    result.set("USERID", id);
				    result.set("EMAIL", email);
				    result.set("AGE", ""+age);
				    result.set("COLOR", ""+color);
				    result.set("FULLPATH", fullpath);
				    result.set("PATH", path);
				    result.set("FILEPATH", path + filename);
				    result.set("FULLFILEPATH", fullfilepath);
				    result.set("FILENAME", filename);
				    result.set("GROUPSPATH", path + "groups/");
				    result.set("MODELSPATH", path + "models/");
				    result.save(fullfilepath);
				} else return(login()); 
		    } // end if
			try {
			    result = VModelElement.load(fullfilepath);
			    if (result == null) {
				    QRGutils.debug("DEBUG","null load of " + fullfilepath);
			    	result = new VModelElement("STUDENT");
				    result.set("NAME", name);
				    result.set("USERID", id);
				    result.set("EMAIL", email);
				    result.set("AGE", ""+age);
				    result.set("COLOR", ""+color);
				    result.set("FULLPATH", fullpath);
				    result.set("FULLFILEPATH", fullfilepath);
				    result.set("PATH", path);
				    result.set("FILEPATH", path + filename);
				    result.set("FILENAME", filename);
				    result.set("GROUPSPATH", path + "groups/");
				    result.set("MODELSPATH", path + "models/");
				    result.save(fullfilepath);
			    } // end if result
			} catch (Exception e) {
			    QRGutils.debug("DEBUG","VModelStudent.load(system,"+fullfilepath+") Error:"+e);
			    result = new VModelElement("STUDENT");
			    result.set("NAME", "STUDENT");
			} // try-catch
	      result.set("FULLPATH", fullpath);
	      result.set("FULLFILEPATH", fullfilepath);
		} // end if
		return(result);
    } // method login

    public void savePrep ( ) {
		student().savePrep();
    } // method savePrep

    public void save ( ) {
		save(getString("FULLFILEPATH"));
    } // method save

    public void save (String filename) {
		savePrep();
		if (!QRGutils.isApplet()) student().save(filename);
		else QRGutils.debug("VModelStudent", "save - Attempt to save from Applet.");
    } // method save

    // Preferences
    // -----------------------------------------------------------------------
    public void updatePreferences ( ) {
		student(system.preferences.localPreferences());
    } // method updatePreferences

	// Get attribute values.
	// -----------------------------------------------------------------------
    public Object get ( String tag ) {
		Object result = null;
		if (student() != null) result = student().get(tag);
		else QRGutils.debug("STUDENT", "Variable student = null.");
		return(result);
    } // method get

    public Color getColor ( String tag ) {
		Color result = Color.black;
		if (student() != null) result = student().getColor(tag);
		else QRGutils.debug("student", "Variable student = null.");
		return result;
    } // method getColor

    public int getRGBColor ( String tag ) {
		int result = 0;
		Color color = getColor(tag);
		if (color != null) result = color.getRGB();
		else QRGutils.debug("student", "getColor returned null.");
		return(result);
    } // method getRGBColor

    public int getInt ( String tag ) {
		int result = 0;
		if (student() != null) result = student().getInt(tag);
		else QRGutils.debug("student", "Variable student = null.");
		return result;
    } // method getInt

    public double getDouble ( String tag ) {
		double result = 0.0;
		if (student() != null) result = student().getDouble(tag);
		else QRGutils.debug("student", "Variable student = null.");
		return(result);
    } // method getDouble

    public String getString ( String tag ) {
		String result = "";
		if (student() != null) result = student().getString(tag);
		else QRGutils.debug("student", "Variable student = null.");
		return(result);
    } // method getDouble

    public boolean getBoolean ( String tag ) {
		boolean result = false;
		if (student() != null) result = student().getBoolean(tag);
		else QRGutils.debug("student", "Variable student = null.");
		return(result);
    } // method getDouble

	// Get attribute values from a child element.
	// -----------------------------------------------------------------------
    public Object get ( String etag, String atag ) {
		Object result = null;
		if (student() != null) result = student().get(etag, atag);
		else QRGutils.debug("student", "Variable student = null.");
		return(result);
    } // method get

    public Color getColor ( String etag, String atag ) {
		Color result = null;
		if (student() != null) result = student().getColor(etag, atag);
		else QRGutils.debug("student", "Variable student = null.");
		return(result);
    } // method getColor

    public int getRGBColor ( String etag, String atag ) {
		int result = 0;
		Color color = getColor(etag, atag);
		if (color != null) result = color.getRGB();
		else QRGutils.debug("student", "getColor returned null.");
		return(result);
    } // method getRGBColor

    public int getInt ( String etag, String atag ) {
		int result = 0;
		if (student() != null) result = student().getInt(etag, atag);
		else QRGutils.debug("student", "Variable student = null.");
		return(result);
    } // method get

    public double getDouble ( String etag, String atag ) {
		double result = 0;
		if (student() != null) result = student().getDouble(etag, atag);
		else QRGutils.debug("student", "Variable student = null.");
		return(result);
    } // method getDouble

    public String getString ( String etag, String atag ) {
		String result = "";
		if (student() != null) result = student().getString(etag, atag);
		else QRGutils.debug("student", "Variable student = null.");
		return(result);
    } // method getString

    public boolean getBoolean ( String etag, String atag ) {
		boolean result = false;
		if (student() != null) result = student().getBoolean(etag, atag);
		else QRGutils.debug("student", "Variable student = null.");
		return(result);
    } // method getString

	// Set attribute values.
	// -----------------------------------------------------------------------
    public Object set ( String tag, Object value ) {
		Object result = null;
		if (student() != null) result = student().set(tag, value);
		else QRGutils.debug("student", "Variable student = null.");
		return(result);
    } // method set

    public Object set ( String tag, int value ) {
		return(set(tag, "" + value));
    } // method set

    public Object set ( String tag, boolean value ) {
		return(set(tag, "" + value));
    } // method set

    public Object set ( String tag, double value ) {
		return(set(tag, "" + value));
    } // method set

    public Object set ( String tag, Color value ) {
		return(set(tag, "" + value.getRGB()));
    } // method set

	// Set attribute values of a child element.
	// -----------------------------------------------------------------------
    public Object set ( String etag, String atag, Object value ) {
		Object result = null;
		if (student() != null) result = student().set(etag, atag, value);
		else QRGutils.debug("student", "Variable student = null.");
		return(result);
    } // method set

    public Object set ( String etag, String atag, int value ) {
		return(set(etag, atag, "" + value));
    } // method set

    public Object set ( String etag, String atag, double value ) {
		return(set(etag, atag, "" + value));
    } // method set

    public Object set ( String etag, String atag, boolean value ) {
		return(set(etag, atag, "" + value));
    } // method set

    public Object set ( String etag, String atag, Color value ) {
		return(set(etag, atag, "" + value.getRGB()));
    } // method set

	// Constructor
	// --------------------------------------------------------------------------
	public VModelStudent( VModel system ) {
		boolean forceLogin = true;
		this.system = system;

		student(new VModelElement("STUDENT"));
		
//		student().set("NAME", (QRGutils.isApplet() ? "Web" : PLATFORM_NAMES[PLATFORM+1]) + " VModel User");
		student().set("USERID", "student");
		student().set("EMAIL", "vmodel@cs.northwestern.edu");
		student().set("AGE", "12");
		student().set("COLOR", "-8084279");
		student().set("FULLPATH", QRGutils.QRGroot()+USERPATH+"student/");
		student().set("FULLFILEPATH", QRGutils.QRGroot()+USERPATH+"student/student.ini");
		student().set("PATH", USERPATH+"student/");
		student().set("FILEPATH", USERPATH+"student/student.ini");
		student().set("FILENAME", "student.ini");
		student().set("GROUPSPATH", USERPATH+"student/groups/");
		student().set("MODELSPATH", USERPATH+"student/models/");
		
		forceLogin = system().preferences().getBoolean("STUDENT","FORCE-LOGIN");
		
		if (DEVELOPER) student(load("developer"));
		else if (forceLogin && !QRGutils.isApplet()) student(login());

		system().preferences().localPreferences(student);
		
		QRGutils.QRGuserpath(student().getString("PATH"));
	
	} // constructor VModelStudent

} // class VModelStudent

// -----------------------------------------------------------------------------
// End of File: VModelStudent.java              (c) 2001 Northwestern University
