package edu.qrg.vmodel.controllers.macclassroom;

import edu.qrg.utils.*;
import edu.qrg.vmodel.*;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.lang.Cloneable;
import java.lang.Object;
import java.lang.String;
import java.lang.System;
import java.util.EventListener;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.LineBorder;
import javax.swing.colorchooser.AbstractColorChooserPanel;
import java.beans.*;
import javax.swing.event.*;

/** <PRE> Sun Java 1.3.1 -Sourcepath QRG/VModel/development/code/gui
* ----------------------------------------------------------------------------
*  2003 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
* ----------------------------------------------------------------------------
* File name: LoginWizard.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* ----------------------------------------------------------------------------
*   Class: LoginWizard
* Purpose: Student Login Facility
* ----------------------------------------------------------------------------
* </PRE> */
public class LoginWizard extends JDialog 
									implements VModelConstants, ActionListener, ChangeListener, DocumentListener {
	// Constructors
	// ------------------------------------------------------------------------
	public LoginWizard () {
		super((Frame) null, "VModel Login", true);
		getContentPane().setLayout(new BorderLayout());
		setSize(QRGutils.calcWindowSize());
		setLocation(QRGutils.center(getSize()));
    } // default constructor
		
	public LoginWizard (JFrame frame) {
		super(frame, "VModel Login", true);
		getContentPane().setLayout(new BorderLayout());
		setSize(QRGutils.calcWindowSize());
		setLocation(QRGutils.center(getSize()));
    	//setModal(true);
    } // default constructor
		
	public LoginWizard (JFrame frame, VModelPreferences preferences, boolean doLogin) {
		this(frame);
		preferences(preferences);
		if (doLogin) doLogin();
	} // constructor

	public LoginWizard (VModelPreferences preferences, boolean doLogin) {
		this();
		preferences(preferences);
		if (doLogin) doLogin();
	} // constructor

	public LoginWizard (VModelPreferences preferences) {
		this(preferences, false);
	} // constructor

	public LoginWizard (JFrame frame, VModelPreferences preferences) {
		this(frame, preferences, false);
	} // constructor

	// Properties & Accessors
	// ------------------------------------------------------------------------
	protected VModelPreferences preferences = null;
	public VModelPreferences preferences () { return(preferences); }
	public VModelPreferences preferences (VModelPreferences p) {
		return(preferences = p);
	} // end writer preferences
	
	protected JPanel loginPane = null;
	public JPanel loginPane () { return(loginPane); }
	public JPanel loginPane (JPanel p) { return (loginPane = p); }
	
	protected JPanel newUserPane = null;
	public JPanel newUserPane () { return(newUserPane); }
	public JPanel newUserPane (JPanel p) { return (newUserPane = p); }
	
	protected JTextField useridWidget = null;
	public JTextField useridWidget () { return(useridWidget); }
	public JTextField useridWidget (JTextField t) {
		return(useridWidget = t);
	} // end writer useridWidget
	
	protected JLabel useridLabelWidget = null;
	public JLabel useridLabelWidget () { return(useridLabelWidget); }
	public JLabel useridLabelWidget (JLabel l) {
		return(useridLabelWidget = l);
	} // end writer 
	
	protected JTextArea userNameWidget = null;
	public JTextArea userNameWidget () { return(userNameWidget); }
	public JTextArea userNameWidget (JTextArea t) {
		return(userNameWidget = t);
	} // end writer userNameWidget
	
	protected JTextField userEmailWidget = null;
	public JTextField userEmailWidget () { return(userEmailWidget); }
	public JTextField userEmailWidget (JTextField t) {
		return(userEmailWidget = t);
	} // end writer userEmailWidget
	
	protected JTextField userAgeWidget = null;
	public JTextField userAgeWidget () { return(userAgeWidget); }
	public JTextField userAgeWidget (JTextField t) {
		return(userAgeWidget = t);
	} // end writer userEmailWidget
	
	protected JColorChooser userColorWidget = null;
	public JColorChooser userColorWidget () { return(userColorWidget); }
	public JColorChooser userColorWidget (JColorChooser c) {
		return(userColorWidget = c);
	} // end writer 
	
	// Required Methods
	// ------------------------------------------------------------------------
	public void actionPerformed(ActionEvent e) {
		Object o = e.getSource();
		String path = null;
		if (o instanceof JButton) {
			if (((JButton) o).getText().equalsIgnoreCase("QUIT")) {
				setVisible(false);
				if (!QRGutils.isApplet()) System.exit(-1);
			} else if (((JButton) o).getText().equalsIgnoreCase("CANCEL")) {
				preferences().set("userId", "");
				preferences().set("userName", "");
				preferences().set("userEmail", "");
				preferences().set("userAge", "");
				Color c = userColorWidget().getColor();
				while (((c.getRed() + c.getGreen() + c.getBlue())/3) < 127 ) {
					c = c.brighter();
				} // end while
				preferences().set("userColor", c);
				switch2LoginPane(true);
			} else if (((JButton) o).getText().equalsIgnoreCase("BACK")) {
				preferences().set("userId", useridWidget().getText());
				preferences().set("userName", userNameWidget().getText());
				preferences().set("userEmail", userEmailWidget().getText());
				preferences().set("userAge", userAgeWidget().getText());
				Color c = userColorWidget().getColor();
				while (((c.getRed() + c.getGreen() + c.getBlue())/3) < 127 ) {
					c = c.brighter();
				} // end while
				preferences().set("userColor", c);
				switch2LoginPane(false);
			} else if (((JButton) o).getText().equalsIgnoreCase("OK")) {
				path = QRGutils.makeUserPath(QRGutils.replaceIllegalFileSystemChars(useridWidget().getText()));
				if (!userPathExists(path)) {
					userNameWidget().setText( preferences().getString( "userName" ) );
					userEmailWidget().setText( preferences().getString( "userEmail" ) );
					userAgeWidget().setText( preferences().getString( "userAge" ) );
					Color c = preferences().getColor( "userColor" );
					if ( c == BLACK ) c = WHITE;
					userColorWidget().setColor(c);
					switch2NewUserPane(true);
				} else {
					QRGutils.QRGuserpath(path);
					preferences().loadLocalPrefs("VModel.ini");
					preferences().set("newLogin",false );
					if (preferences().getString("USERNAME").equalsIgnoreCase("KCARNEY")) preferences.set("SOUND", true);
				Color c = preferences().getColor("userColor");
				while (((c.getRed() + c.getGreen() + c.getBlue())/3) < 127 ) {
					c = c.brighter();
				} // end while
				preferences().set("userColor", c);
					setVisible(false);
				} // end if
			} else if (((JButton) o).getText().equalsIgnoreCase("FINISH")) {
				path = QRGutils.makeUserPath(QRGutils.replaceIllegalFileSystemChars(useridWidget().getText()));
				if (!userPathExists(path)) createNewUserPath(path);
				QRGutils.QRGuserpath(path);
				preferences().set("userId", useridWidget().getText());
				preferences().set("userName", userNameWidget().getText());
				preferences().set("userEmail", userEmailWidget().getText());
				preferences().set("userAge", userAgeWidget().getText());
				Color c = userColorWidget().getColor();
				while (((c.getRed() + c.getGreen() + c.getBlue())/3) < 127 ) {
					c = c.brighter();
				} // end while
				preferences().set("userColor", c);
				preferences().set("newLogin", true );
				setVisible(false);
			}
		} else {
			finishButton().setEnabled((!userNameWidget().getText().equals("")) &&
				(!userEmailWidget().getText().equals("")) &&
				(!userAgeWidget().getText().equals("")) &&
				(userColorWidget().getColor() != null));
		} // end if
	} // method actionPerformed
	
	protected JButton okButton = null;
	public JButton okButton () { return(okButton); }
	public JButton okButton (JButton b) { return(okButton = b); }

	protected JButton finishButton = null;
	public JButton finishButton () { return(finishButton); }
	public JButton finishButton (JButton b) { return(finishButton = b); }

	// Methods
	// ------------------------------------------------------------------------
	public void switch2LoginPane (boolean init) {
		getContentPane().removeAll();
		if (init) initLoginPane();
		getContentPane().add(loginPane(), BorderLayout.CENTER);
		getRootPane().setDefaultButton(okButton());
		setVisible(true); 
		invalidate();
		repaint();
	} // method switch2LoginPane
	
	public void switch2NewUserPane (boolean init) {
		getContentPane().removeAll();
		if (init) initNewUserPane();
		getContentPane().add(newUserPane(), BorderLayout.CENTER);
		getRootPane().setDefaultButton(finishButton());
		finishButton().setEnabled((!useridWidget().getText().equals("")) && 
			(!userNameWidget().getText().equals("")) &&
			(!userEmailWidget().getText().equals("")) &&
			(!userAgeWidget().getText().equals("")) &&
			(userColorWidget().getColor() != null));
		setVisible(true); 
		invalidate();
		repaint();
	} // method switch2NewUserPane
	
	public String resetUserPath () {
		QRGutils.initUserPath();
		QRGutils.QRGuserpath(QRGutils.makeUserPath("VModel"));
		return(QRGutils.QRGuserpath());
	} // end method resetUserPath
	
	public boolean createNewUserPath (String userpath) {
		boolean result = false;
		File path = new File(userpath);
		result = path.mkdirs();
		return(result);
	} // method createNewUserPane
	
	public boolean userPathExists (String userpath) {
		boolean result = false;
		File path = new File(userpath);
		result = path.exists();
		return(result);
	} // method userPathExists
	
	public JPanel initLoginPane () {
		JPanel result = loginPane();
		JPanel widgets = null;
		JPanel buttonPanel = null;
		JButton okButton = null;
		JButton cancelButton = null;
		String userid = null;
		if (result == null) {
			loginPane(new JPanel(new BorderLayout()));
			loginPane().add(
				new JLabel(ImageCache.getImage("VModelLogin.gif")),
				BorderLayout.CENTER);
    		useridWidget(new JTextField("", 26));
//KeyStroke enter = KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0);
//Keymap map = useridWidget().getKeymap();
//map.removeKeyStrokeBinding(enter);
    		buttonPanel = new JPanel(new GridLayout(1, 2));
    		okButton = new JButton("Ok");
    		okButton.setDefaultCapable(true);
    		okButton.addActionListener(this);
    		okButton(okButton);
    		getRootPane().setDefaultButton(okButton());
    		buttonPanel.add(okButton);
    		cancelButton = new JButton("Quit");
    		cancelButton.addActionListener(this);
    		if (!QRGutils.isApplet()) buttonPanel.add(cancelButton);
			widgets = new JPanel(new GridLayout(4,1));    		
    		widgets.add(new JLabel("Enter UserID:"));
    		widgets.add(useridWidget());
    		widgets.add(new JLabel(" "));
    		widgets.add(buttonPanel);
    		widgets.setVisible(true);
    		loginPane().add(widgets, BorderLayout.SOUTH);
    		result = loginPane();
		} // end if
		userid = System.getProperty("user.name");
		if (userid == null) userid = "";
		if (useridWidget() != null) useridWidget().setText(userid);
		return(result);
	} // end method initLoginPane
	
	public JPanel initNewUserPane () {
		JPanel result = newUserPane();
		JPanel gadgets = null;
		JPanel widgets = null;
		JPanel gromets = null;
		JColorChooser clrChooser = null;
		AbstractColorChooserPanel[] panels = null;
		JPanel buttonPanel = null;
		JButton backButton = null;
		JButton okButton = null;
		JButton cancelButton = null;
		String userid = null;
		JScrollPane scrollpane = null;
		if (result == null) {
			newUserPane(new JPanel(new BorderLayout()));
    		gadgets = new JPanel(new BorderLayout());
    		widgets = new JPanel(new GridLayout(3, 1));
    		widgets.add(useridLabelWidget(new JLabel("UserID not found.  Please enter user profile information.")));
    		widgets.add(new JLabel(""));
    		widgets.add(new JLabel("Enter your name(s):"));
    		gadgets.add(widgets, BorderLayout.NORTH);
    		
    		//userNameWidget(new JTextField("", 32));
    		userNameWidget(new JTextArea(3, 32));
    		userNameWidget().setLineWrap(true);
    		userNameWidget().setWrapStyleWord(true);
    		scrollpane = new JScrollPane(userNameWidget());
    		//widgets.add(userNameWidget());
    		gadgets.add(scrollpane, BorderLayout.CENTER);
    		
    		widgets = new JPanel(new GridLayout(4, 1));
    		widgets.add(new JLabel(""));
    		gromets = new JPanel(new GridLayout(1,2));
			gromets.add(new JLabel("Enter your eMail:"));
    		gromets.add(new JLabel("Enter your Age:"));
    		widgets.add(gromets);
    		
    		gromets = new JPanel(new GridLayout(1,2));
			gromets.add(userEmailWidget(new JTextField("")));
    		gromets.add(userAgeWidget(new JTextField("")));
    		widgets.add(gromets);
    		widgets.add(new JLabel(""));
    		gadgets.add(widgets, BorderLayout.SOUTH);
    		newUserPane().add(gadgets, BorderLayout.NORTH);
    		
			clrChooser = new JColorChooser();
			panels = clrChooser.getChooserPanels();
			clrChooser.removeChooserPanel(panels[ 2 ]);
			clrChooser.removeChooserPanel(panels[ 1 ]);
			clrChooser.setBorder(new LineBorder(BLACK));
			userColorWidget(clrChooser);
			
			widgets = new JPanel(new BorderLayout());
    		widgets.add(new JLabel("Select your color:"), BorderLayout.NORTH);
    		widgets.add(userColorWidget(), BorderLayout.CENTER);
    		newUserPane().add(widgets, BorderLayout.CENTER);
    		

    		buttonPanel = new JPanel(new GridLayout(1, 2));
    		
    		backButton = new JButton("Back");
    		backButton.addActionListener(this);
    		buttonPanel.add(backButton);
    		okButton = new JButton("Finish");
    		okButton.setDefaultCapable(true);
    		okButton.addActionListener(this);
    		finishButton(okButton);
    		buttonPanel.add (okButton);
    		cancelButton = new JButton("Cancel");
    		cancelButton.addActionListener(this);
    		if (!QRGutils.isApplet()) buttonPanel.add(cancelButton);

//    		useridWidget().addActionListener( this );
     		userNameWidget().getDocument().addDocumentListener( this );
    		userEmailWidget().getDocument().addDocumentListener( this );
    		userAgeWidget().getDocument().addDocumentListener( this );
 //   		userColorWidget().addActionListener( this );
    		userColorWidget().getSelectionModel().addChangeListener( this );

    		newUserPane().add(buttonPanel, BorderLayout.SOUTH);
    		
    		result = newUserPane();
		} // end if
		
		if ((useridLabelWidget() != null) && (useridWidget() != null)) 
			useridLabelWidget().setText("Userid " + useridWidget().getText() + " not found. Please enter profile information below:");
		if (userNameWidget() != null) userNameWidget().setText(preferences().getString("userName"));
		if (userEmailWidget() != null) userEmailWidget().setText(preferences().getString("userEmail"));
		if (userAgeWidget() != null) userAgeWidget().setText(preferences().getString("userAge"));
		if (userColorWidget() != null) userColorWidget().setColor(preferences().getColor("userColor"));
		return(result);
	} // end method initNewUserPane
	
	public void stateChanged(ChangeEvent e) {
		finishButton().setEnabled((!userNameWidget().getText().equals("")) &&
			(!userEmailWidget().getText().equals("")) &&
			(!userAgeWidget().getText().equals("")) &&
			(userColorWidget().getColor() != null));
	} // method
	
	public void changedUpdate(DocumentEvent e) {
		finishButton().setEnabled((!userNameWidget().getText().equals("")) &&
			(!userEmailWidget().getText().equals("")) &&
			(!userAgeWidget().getText().equals("")) &&
			(userColorWidget().getColor() != null));
	} // method
	
	public void insertUpdate(DocumentEvent e) {
		finishButton().setEnabled((!userNameWidget().getText().equals("")) &&
			(!userEmailWidget().getText().equals("")) &&
			(!userAgeWidget().getText().equals("")) &&
			(userColorWidget().getColor() != null));
	} // method
	
	public void removeUpdate(DocumentEvent e) {
		finishButton().setEnabled((!userNameWidget().getText().equals("")) &&
			(!userEmailWidget().getText().equals("")) &&
			(!userAgeWidget().getText().equals("")) &&
			(userColorWidget().getColor() != null));
	} // method
	

	public boolean doLogin () {
		boolean result = false;
		resetUserPath();
		getContentPane().removeAll();
		initLoginPane();
		initNewUserPane();
		getContentPane().add(newUserPane(), BorderLayout.CENTER);
		setSize(getMinimumSize().width + 50, getMinimumSize().height + 50);
		setLocation(QRGutils.center(getSize()));
		getContentPane().remove(newUserPane());
		getContentPane().add(loginPane(), BorderLayout.CENTER);
		setVisible(true);
		return(result);
	} // end method doLogin
	
} // class LoginWizard

// ---------------------------------------------------------------------------
// End of File: LoginWizard.java              (c) 2003 Northwestern University
