package edu.qrg.vmodel.controllers.macclassroom;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import javax.swing.*;
import java.net.*;

public class PNM extends JFrame {
	public static final String P1 = "P1";
	public static final String P2 = "P2";
	public static final String P3 = "P3";
	public static final String P4 = "P4";
	public static final String P5 = "P5";
	public static final String P6 = "P6";
	public static final int BILEVEL_COLOR = 1;
	public static final int GRAYSCALE_COLOR = 2;
	public static final int RGB_COLOR = 3;
	public static final int PBM = 1;
	public static final int PGM = 2;
	public static final int PPM = 3;
	public static final int ASCII = 0;
	public static final int BINARY = 3;

	public int width = -1;
	public int height = -1;
	public double scale = 1.0;
	public int translatex = 0, translatey = 0;
	
	public PNM() {
		super();
		setLocation(0,0);
		setSize(1,1);
	} // end constructor

	public boolean isWhiteSpace(int i) {
		boolean result = false;
		result = ((i == 0x20) || (i == 0xA0) || (i == 0x0D) || 
					 (i == 0x8D) || (i == 0x09) || (i == 0x0A));
		return(result);
	} // end method isWhiteSpace	
	
	public boolean isWhiteSpace(char c) {
		boolean result = false;
		result = ((c == 0x20) || (c == 0xA0) || (c == 0x0D) || 
					 (c == 0x8D) || (c == 0x09) || (c == 0x0A));
		return(result);
	} // end method isWhiteSpace	
	
	public boolean isWhiteSpace(byte b) {
		boolean result = false;
		result = ((b == 0x20) || (b == 0xA0) || (b == 0x0D) || 
					 (b == 0x8D) || (b == 0x09) || (b == 0x0A));
		return(result);
	} // end method isWhiteSpace	
	
	public int gobbleWhiteSpace (InputStream inp) {
		int c = 0;
		try {c = inp.read();}
		catch (Exception e) {c = -1;}
		while((c != -1) && 
				(isWhiteSpace(c))) {
			try {c = inp.read();}
			catch (Exception e) {c = -1;}
		} // end while
		return(c);
	} // method gobbleWhitepSpace
	
	public String readASCIITokenString (DataInputStream inp) {
		String result = "";
		int c = gobbleWhiteSpace(inp);
		while ((c != -1) && 
				 (!isWhiteSpace(c))) {
			result += (char) c; // expensive.
			try {c = inp.read();}
			catch (Exception e) {c = -1;}
		} // end while c
		return(result);
	} // end method readASCIITokenString

	public int readASCIITokenInt (DataInputStream inp) {
		int result = -1;
		String mumble = readASCIITokenString(inp);
		try {result = Integer.parseInt(mumble);}
		catch (Exception e) {result = -1;}
		return(result);
	} // end method readASCIITokenInt

	public Image decode (String filename) {
		return(decode(new File(filename)));
	} // end method decode

	public Image decode (File file) {
		Image result = null;
		try {
			FileInputStream inp = new FileInputStream(file);
			result = decode(inp);
			inp.close();
		} catch (Exception e) { }
		return(result);
	} // end method decode

	public Image decode (URL url) {
		Image result = null;
		try {
			InputStream inp = url.openStream();
			result = decode(inp);
			inp.close();
		} catch (Exception e) { }
		return(result);
	} // end method decode

	public Image decode (InputStream inp) {
		Image result = null;
		try {
			DataInputStream din = new DataInputStream(inp);
			String type = "";
			byte typeBytes[] = new byte[2];
			din.read(typeBytes);
			type = new String(typeBytes);
			if (type.equalsIgnoreCase("P1")) result = decodeP1(din);
			else if (type.equalsIgnoreCase("P2")) result = decodeP2(din);
			else if (type.equalsIgnoreCase("P3")) result = decodeP3(din);
			else if (type.equalsIgnoreCase("P4")) result = decodeP4(din);
			else if (type.equalsIgnoreCase("P5")) result = decodeP5(din);
			else if (type.equalsIgnoreCase("P6")) result = decodeP6(din);
			din.close();
		} catch (Exception e) { }
		return(result);
	} // end method decode

	public Image decodeP1 (DataInputStream inp) {
		Image result = null;
		int pos = 0, count = 0;;
		byte rb=0,gb=0,bb=0;
		int r=0,g=0,b=0;
		int width = readASCIITokenInt(inp);
		int height = readASCIITokenInt(inp);
		//int colorMax = readASCIITokenInt(inp);
		Graphics ig = null;
		if ((width != -1) && (height != -1)) {
			try {
				setVisible(true);
				result = createImage(width, height);
				setVisible(false);
				ig = result.getGraphics();
				for(int y=0; y<height; y++)
					for(int x=0; x<width; x++) {
						r = readASCIITokenInt(inp);
						g = r * 255;
						//b = readASCIITokenInt(inp);
						ig.setColor(new Color(g,g,g));
						ig.fillRect((int)(x*scale)-translatex,(int)(y*scale)-translatey,(scale<1)?1:(int)Math.ceil(scale),(scale<1)?1:(int)Math.ceil(scale));
					} // end for x
			} catch (Exception e) {result = null;}
		} // end if !eof
		return(result);
	} // end method decode

	public Image decodeP2 (DataInputStream inp) {
		Image result = null;
		int pos = 0, count = 0;;
		byte rb=0,gb=0,bb=0;
		int r=0,g=0,b=0;
		int width = readASCIITokenInt(inp);
		int height = readASCIITokenInt(inp);
		int colorMax = readASCIITokenInt(inp);
		Graphics ig = null;
		if ((width != -1) && (height != -1) && (colorMax != -1) && (colorMax <= 255)) {
			try {
				setVisible(true);
				result = createImage(width, height);
				setVisible(false);
				ig = result.getGraphics();
				for(int y=0; y<height; y++)
					for(int x=0; x<width; x++) {
						r = readASCIITokenInt(inp);
						//g = readASCIITokenInt(inp);
						//b = readASCIITokenInt(inp);
						ig.setColor(new Color(r,r,r));
						ig.fillRect((int)(x*scale)-translatex,(int)(y*scale)-translatey,(scale<1)?1:(int)Math.ceil(scale),(scale<1)?1:(int)Math.ceil(scale));
					} // end for x
			} catch (Exception e) {result = null;}
		} // end if !eof
		return(result);
	} // end method decode

	public Image decodeP3 (DataInputStream inp) {
		Image result = null;
		int pos = 0, count = 0;;
		byte rb=0,gb=0,bb=0;
		int r=0,g=0,b=0;
		int width = readASCIITokenInt(inp);
		int height = readASCIITokenInt(inp);
		int colorMax = readASCIITokenInt(inp);
		Graphics ig = null;
		if ((width != -1) && (height != -1) && (colorMax != -1) && (colorMax <= 255)) {
			try {
				setVisible(true);
				result = createImage(width, height);
				setVisible(false);
				ig = result.getGraphics();
				for(int y=0; y<height; y++)
					for(int x=0; x<width; x++) {
						//r = readASCIITokenInt(inp);
						//g = readASCIITokenInt(inp);
						//b = readASCIITokenInt(inp);
						ig.setColor(new Color(readASCIITokenInt(inp),readASCIITokenInt(inp),readASCIITokenInt(inp)));
						ig.fillRect((int)(x*scale)-translatex,(int)(y*scale)-translatey,(scale<1)?1:(int)Math.ceil(scale),(scale<1)?1:(int)Math.ceil(scale));
					} // end for x
			} catch (Exception e) {result = null;}
		} // end if !eof
		return(result);
	} // end method decode

	public Image decodeP4 (DataInputStream inp) {
		Image result = null;
		int pos = 0, count = 0;;
		byte b=0;
		int p1=0,p2=0,p3=0,p4=0,p5=0,p6=0,p7=0,p8=0;
		int width = readASCIITokenInt(inp);
		int height = readASCIITokenInt(inp);
		int x=0,y=0;
		byte pixels[] = null;
		Graphics ig = null;
		if ((width != -1) && (height != -1)) {
			pixels = new byte[(int)Math.ceil((width * height)/8)];
			try {
				count = inp.read(pixels);
				setVisible(true);
				result = createImage(width, height);
				setVisible(false);
				ig = result.getGraphics();
				for(int k=0; k<(width * height); k=k+8) {
					b = pixels[pos++];
					p1 = (b & 0x01);
					p2 = ((b >> 1) & 0x01)*255;
					p3 = ((b >> 2) & 0x01)*255;
					p4 = ((b >> 3) & 0x01)*255;
					p5 = ((b >> 4) & 0x01)*255;
					p6 = ((b >> 5) & 0x01)*255;
					p7 = ((b >> 6) & 0x01)*255;
					p8 = ((b >> 7) & 0x01)*255;
					ig.setColor(new Color(p1,p1,p1));
					x=k%width;
					y=k/width;
					ig.fillRect((int)(x*scale)-translatex,(int)(y*scale)-translatey,(scale<1)?1:(int)Math.ceil(scale),(scale<1)?1:(int)Math.ceil(scale));
					ig.setColor(new Color(p2,p2,p2));
					x=(k+1)%width;
					y=(k+1)/width;
					ig.fillRect((int)(x*scale)-translatex,(int)(y*scale)-translatey,(scale<1)?1:(int)Math.ceil(scale),(scale<1)?1:(int)Math.ceil(scale));
					ig.setColor(new Color(p3,p3,p3));
					x=(k+2)%width;
					y=(k+2)/width;
					ig.fillRect((int)(x*scale)-translatex,(int)(y*scale)-translatey,(scale<1)?1:(int)Math.ceil(scale),(scale<1)?1:(int)Math.ceil(scale));
					ig.setColor(new Color(p4,p4,p4));
					x=(k+3)%width;
					y=(k+3)/width;
					ig.fillRect((int)(x*scale)-translatex,(int)(y*scale)-translatey,(scale<1)?1:(int)Math.ceil(scale),(scale<1)?1:(int)Math.ceil(scale));
					ig.setColor(new Color(p5,p5,p5));
					x=(k+4)%width;
					y=(k+4)/width;
					ig.fillRect((int)(x*scale)-translatex,(int)(y*scale)-translatey,(scale<1)?1:(int)Math.ceil(scale),(scale<1)?1:(int)Math.ceil(scale));
					ig.setColor(new Color(p6,p6,p6));
					x=(k+5)%width;
					y=(k+5)/width;
					ig.fillRect((int)(x*scale)-translatex,(int)(y*scale)-translatey,(scale<1)?1:(int)Math.ceil(scale),(scale<1)?1:(int)Math.ceil(scale));
					ig.setColor(new Color(p7,p7,p7));
					x=(k+6)%width;
					y=(k+6)/width;
					ig.fillRect((int)(x*scale)-translatex,(int)(y*scale)-translatey,(scale<1)?1:(int)Math.ceil(scale),(scale<1)?1:(int)Math.ceil(scale));
					ig.setColor(new Color(p8,p8,p8));
					x=(k+7)%width;
					y=(k+7)/width;
					ig.fillRect((int)(x*scale)-translatex,(int)(y*scale)-translatey,(scale<1)?1:(int)Math.ceil(scale),(scale<1)?1:(int)Math.ceil(scale));
				} // end for k
			} catch (Exception e) {result = null;}
		} // end if !eof
		return(result);
	} // end method decode

	public Image decodeP5 (DataInputStream inp) {
		Image result = null;
		int pos = 0, count = 0;;
		byte b=0;
		int g=0;
		int width = readASCIITokenInt(inp);
		int height = readASCIITokenInt(inp);
		int colorMax = readASCIITokenInt(inp);
		byte pixels[] = null;
		Graphics ig = null;
		if ((width != -1) && (height != -1) && (colorMax != -1) && (colorMax <= 255)) {
			pixels = new byte[width * height];
			try {
				count = inp.read(pixels);
				setVisible(true);
				result = createImage(width, height);
				setVisible(false);
				ig = result.getGraphics();
				for(int y=0; y<height; y++)
					for(int x=0; x<width; x++) {
						b = pixels[pos++];
						g = (b >= 0) ? b : 255 + b;
						ig.setColor(new Color(g,g,g));
						ig.fillRect((int)(x*scale)-translatex,(int)(y*scale)-translatey,(scale<1)?1:(int)Math.ceil(scale),(scale<1)?1:(int)Math.ceil(scale));
					} // end for x
			} catch (Exception e) {result = null;}
		} // end if !eof
		return(result);
	} // end method decode

	public Image decodeP6 (DataInputStream inp) {
		Image result = null;
		int pos = 0, count = 0;;
		byte rb=0,gb=0,bb=0;
		int r=0,g=0,b=0;
		int width = readASCIITokenInt(inp);
		int height = readASCIITokenInt(inp);
		int colorMax = readASCIITokenInt(inp);
		byte pixels[] = null;
		Graphics ig = null;
		if ((width != -1) && (height != -1) && (colorMax != -1) && (colorMax <= 255)) {
			pixels = new byte[width * height * 3];
			try {
				count = inp.read(pixels);
				setVisible(true);
				result = createImage(width, height);
				setVisible(false);
				ig = result.getGraphics();
				for(int y=0; y<height; y++)
					for(int x=0; x<width; x++) {
						rb = pixels[pos++];
						gb = pixels[pos++];
						bb = pixels[pos++];
						r = (rb >= 0) ? rb : 255 + rb;
						g = (gb >= 0) ? gb : 255 + gb;
						b = (bb >= 0) ? bb : 255 + bb;
						ig.setColor(new Color(r,g,b));
						ig.fillRect((int)(x*scale)-translatex,(int)(y*scale)-translatey,(scale<1)?1:(int)Math.ceil(scale),(scale<1)?1:(int)Math.ceil(scale));
					} // end for x
			} catch (Exception e) {result = null;}
		} // end if !eof
		return(result);
	} // end method decode

	public int encode (Image image, String filename, int format, int type) {
		return(encode(image, new File(filename), format, type));
	} // end method encode

	public int encode (Image image, String filename, String format_type) {
		return(encode(image, new File(filename), format_type));
	} // end method encode

	public int encode (Image image, File file, String format_type) {
		int result = -1;
		if (format_type == P1) result = encode(image, file, PBM, ASCII);
		else if (format_type == P2) result = encode(image, file, PGM, ASCII);
		else if (format_type == P3) result = encode(image, file, PPM, ASCII);
		else if (format_type == P4) result = encode(image, file, PBM, BINARY);
		else if (format_type == P5) result = encode(image, file, PGM, BINARY);
		else if (format_type == P6) result = encode(image, file, PPM, BINARY);
		return(result);
	} // end method encode

	public int encode (Image image, File file, int format, int type) {
		int result = -1;
		try {
			FileOutputStream fout = new FileOutputStream(file);
			result = encode(image, fout, format + type);
			fout.close();
		} catch (Exception e) { }
		return(result);
	} // end method encode

	public int encode (Image image, OutputStream fout, String format_type) {
		int result = -1;
		if (format_type == P1) result = encode(image, fout, PBM, ASCII);
		else if (format_type == P2) result = encode(image, fout, PGM, ASCII);
		else if (format_type == P3) result = encode(image, fout, PPM, ASCII);
		else if (format_type == P4) result = encode(image, fout, PBM, BINARY);
		else if (format_type == P5) result = encode(image, fout, PGM, BINARY);
		else if (format_type == P6) result = encode(image, fout, PPM, BINARY);
		return(result);
	} // end method encode

	public int encode (Image image, OutputStream fout, int format, int type) {
		return(encode(image, fout, format+type));
	} // end method encode

	public int encode (Image image, OutputStream fout, int format_type) {
		int result = -1;
		try {
			DataOutputStream dout = new DataOutputStream(fout);
			result = encode(image, dout, format_type);
			dout.close();
		} catch (Exception e) { }
		return(result);
	} // end method encode

	public int encode (Image image, DataOutputStream dout, int format_type) {
		int result = -1;
		switch(format_type) {
			case 1: {encodeP1(image,dout); break;}
			case 2: {encodeP2(image,dout); break;}
			case 3: {encodeP3(image,dout); break;}
			case 4: {encodeP4(image,dout); break;}
			case 5: {encodeP5(image,dout); break;}
			case 6: {encodeP6(image,dout); break;}
		} // end switch format_type
		return(result);
	} // end method encode

	public int encodeP1 (Image image, DataOutputStream dout) {
		int result = -1;
		int width = image.getWidth(null);
		int height = image.getHeight(null);
		int data[] = new int[width*height];
		int pixels[] = new int[width*height*3];
		int pos = 0;
		int r=0,g=0,b=0;
		boolean grabOK = false;
		String buf = "";
	   try { 
			PixelGrabber pg = new PixelGrabber(image,0,0,width,height,data,0,width);
			PrintWriter pw = new PrintWriter(dout);
	   	grabOK= pg.grabPixels();
	    	if (grabOK) {
	    		pw.print("P1\n"+width+" "+height+" ");
	    		for(int i=0; i<(width*height); i++) {
	    			if ((i%32)==0) pw.println();
	    			r = ((((data[i] & 0xff0000) >> 16) + ((data[i] & 0x00ff00) >> 8) + (data[i] & 0x0000ff))/3);
					g = ((r >= 0) && (r <= 127)) ? 0 : 1;
	    			pw.print(g);
	    			pw.print(((char)0x09));
	    		} // end for
				pw.close();
	    		result = 0;
	    	} // if grabOK
    	} catch (Exception e) {};
		return(result);
	} // end method encode

	public int encodeP2 (Image image, DataOutputStream dout) {
		int result = -1;
		int width = image.getWidth(null);
		int height = image.getHeight(null);
		int data[] = new int[width*height];
		int pixels[] = new int[width*height*3];
		int pos = 0;
		int r=0,g=0,b=0;
		boolean grabOK = false;
		String buf = "";
	   try { 
			PixelGrabber pg = new PixelGrabber(image,0,0,width,height,data,0,width);
			PrintWriter pw = new PrintWriter(dout);
	   	grabOK= pg.grabPixels();
	    	if (grabOK) {
	    		pw.print("P2\n"+width+" "+height+"\n255 ");
	    		for(int i=0; i<(width*height); i++) {
	    			if ((i%17)==0) pw.println();
	    			pw.print(((int)((((data[i] & 0xff0000) >> 16) + ((data[i] & 0x00ff00) >> 8) + (data[i] & 0x0000ff))/3)));
	    			pw.print(((char)0x09));
	    		} // end for
				pw.close();
	    		result = 0;
	    	} // if grabOK
    	} catch (Exception e) {};
		return(result);
	} // end method encode

	public int encodeP3 (Image image, DataOutputStream dout) {
		int result = -1;
		int width = image.getWidth(null);
		int height = image.getHeight(null);
		int data[] = new int[width*height];
		int pixels[] = new int[width*height*3];
		int pos = 0;
		int r=0,g=0,b=0;
		boolean grabOK = false;
		String buf = "";
	   try { 
			PixelGrabber pg = new PixelGrabber(image,0,0,width,height,data,0,width);
			PrintWriter pw = new PrintWriter(dout);
	   	grabOK= pg.grabPixels();
	    	if (grabOK) {
	    		pw.print("P3\n"+width+" "+height+"\n255 ");
	    		for(int i=0; i<(width*height); i++) {
	    			if ((i%5)==0) pw.println();
	    			pw.print((data[i] & 0xff0000) >> 16);
	    			pw.print(((char)0x09));
	    			pw.print((data[i] & 0x00ff00) >> 8);
	    			pw.print(((char)0x09));
	    			pw.print(data[i] & 0x0000ff);
	    			pw.print(((char)0x09));
	    		} // end for
				pw.close();
	    		result = 0;
	    	} // if grabOK
    	} catch (Exception e) {};
		return(result);
	} // end method encode

	public int encodeP4 (Image image, DataOutputStream dout) {
		int result = -1;
		int width = image.getWidth(null);
		int height = image.getHeight(null);
		int data[] = new int[width*height];
		byte pixels[] = new byte[(int)Math.ceil((width*height)/8.0)];
		int pos = 0, shift = 0;
		byte r=0,g=0,b=0;
		boolean grabOK = false;
		int mumble = 0;
		byte grumble = 0;
		byte bumble = 0;
		int foo=0, bar=0, baz=0;
		PixelGrabber pg = new PixelGrabber(image,0,0,width,height,data,0,width);
	   try { 
	   	grabOK= pg.grabPixels();
	    	if (grabOK) {
	    		dout.writeBytes("P4\n"+width+" "+height+"\n");
	    		for(int i=0; i<((int)Math.ceil((width*height)/8.0)); i++) pixels[i] = 0x00;
	    		for(int i=0; i<(width*height); i++) {
	    			foo = ((data[i] & 0xff0000) >> 16);
	    			bar = ((data[i] & 0x00ff00) >> 8);
	    			baz = (data[i] & 0x0000ff);
	    			foo = (foo >= 0) ? foo : 255 + foo;
	    			bar = (bar >= 0) ? bar : 255 + bar;
	    			baz = (baz >= 0) ? baz : 255 + baz;
	    			mumble = (foo + bar + baz)/3;
	    			if ((mumble >= 0) && (mumble <= 127)) grumble = (byte) 0x00;
	    			else grumble = (byte) 0x01;
	    			bumble = (byte) (grumble << (shift++));
	    			pixels[pos] |= (byte) bumble;
	    			if (shift == 8) {
	    				shift = 0;
	    				pos++;
	    			} // end if shift
	    		} // end for
	    		dout.write(pixels,0,(int)Math.ceil((width*height)/8.0));
	    		result = 0;
	    	} // if grabOK
    	} catch (Exception e) {};
		return(result);
	} // end method encode

	public int encodeP5 (Image image, DataOutputStream dout) {
		int result = -1;
		int width = image.getWidth(null);
		int height = image.getHeight(null);
		int data[] = new int[width*height];
		byte pixels[] = new byte[width*height];
		int pos = 0;
		byte r=0,g=0,b=0;
		boolean grabOK = false;
		PixelGrabber pg = new PixelGrabber(image,0,0,width,height,data,0,width);
	   try { 
	   	grabOK= pg.grabPixels();
	    	if (grabOK) {
	    		dout.writeBytes("P5\n"+width+" "+height+"\n255\n");
	    		for(int i=0; i<(width*height); i++) {
	    			pixels[pos++] = (byte) ((((data[i] & 0xff0000) >> 16) +
	    											((data[i] & 0x00ff00) >> 8) +
	    											(data[i] & 0x0000ff))/3);
	    		} // end for
	    		dout.write(pixels,0,width*height);
	    		result = 0;
	    	} // if grabOK
    	} catch (Exception e) {};
		return(result);
	} // end method encode

	public int encodeP6 (Image image, DataOutputStream dout) {
		int result = -1;
		int width = image.getWidth(null);
		int height = image.getHeight(null);
		int data[] = new int[width*height];
		byte pixels[] = new byte[width*height*3];
		int pos = 0;
		byte r=0,g=0,b=0;
		boolean grabOK = false;
		PixelGrabber pg = new PixelGrabber(image,0,0,width,height,data,0,width);
	   try { 
	   	grabOK= pg.grabPixels();
	    	if (grabOK) {
	    		dout.writeBytes("P6\n"+width+" "+height+"\n255\n");
	    		for(int i=0; i<(width*height); i++) {
	    			pixels[pos++] = (byte) ((data[i] & 0xff0000) >> 16);
	    			pixels[pos++] = (byte) ((data[i] & 0x00ff00) >> 8);
	    			pixels[pos++] = (byte) (data[i] & 0x0000ff);
	    		} // end for
	    		dout.write(pixels,0,width*height*3);
	    		result = 0;
	    	} // if grabOK
    	} catch (Exception e) {};
		return(result);
	} // end method encode
} // end class PPM

// its too much to ask that preachers do more than lay people - ghallager
