package edu.qrg.vmodel.controllers.macclassroom;

// Sun Java 1.3.1 -Sourcepath QRG/VModel/v4/code/src
// --------------------------------------------------------------------------
// File name: VModelGUIModelFrame.java
//    System: VModel
//   Version: 4.0
//  Abstract: VModel is a visual representation system developed to provide 
//            computer support to students learning how to create, articulate,
//            and reason with scientific models.  There is a special emphasis on
//            supporting the reuse and integration of previously modeled
//            material in the course of making new models. 
//    Author: Leo C. Ureel II
//   Created: March 7, 1999 18:52:54
//   Purpose: The VModelGUIModelFrame represents the top-level VModel window.  
//            The event driven nature of this system gives this class a great 
//            deal of control over the flow and interpretation of data.
// --------------------------------------------------------------------------
// (c)2001 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
// --------------------------------------------------------------------------
// Modified: Friday, June 22, 2001 at 12:10:52 by ureel
// --------------------------------------------------------------------------
// Notes: Look up papers by John Clement
// --------------------------------------------------------------------------
import edu.qrg.utils.*;
import edu.qrg.vmodel.*;
import edu.qrg.vmodel.controllers.common.*;
import java.awt.*;
import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*; //EtchedBorder
import java.util.*;
import javax.swing.tree.*;
import java.awt.image.*;
import java.beans.*;
import java.awt.event.*;

// --------------------------------------------------------------------------
// Class: VModelGUIiModel
// --------------------------------------------------------------------------
public class ModelFrame extends JInternalFrame 
		implements VModelConstants, InternalFrameListener,
					  ComponentListener, DocumentListener {
	// Constructors
	// -----------------------------------------------------------------------
	public ModelFrame () {
		super();
		setFrameIcon(ImageCache.getImage("vmodel.gif"));
		setClosable(true);
		setResizable(true);
		setMaximizable(true);
		setIconifiable(true);
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		addInternalFrameListener(this);
		addComponentListener(this);
		registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				((MacClassroom) controller()).help();
			} // end if
		}, KeyStroke.getKeyStroke((char) KeyEvent.VK_F12), WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				((MacClassroom) controller()).delete(true);
			} // end if
		}, KeyStroke.getKeyStroke((char) KeyEvent.VK_BACK_SPACE), WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				((MacClassroom) controller()).delete(true);
			} // end if
		}, KeyStroke.getKeyStroke((char) KeyEvent.VK_DELETE), WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				((MacClassroom) controller()).copy();
			} // end if
		}, KeyStroke.getKeyStroke((char) KeyEvent.VK_C, java.awt.Event.CTRL_MASK), WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				((MacClassroom) controller()).copy();
			} // end if
		}, KeyStroke.getKeyStroke((char) KeyEvent.VK_C, java.awt.Event.META_MASK), WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				((MacClassroom) controller()).paste();
			} // end if
		}, KeyStroke.getKeyStroke((char) KeyEvent.VK_V, java.awt.Event.CTRL_MASK), WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				((MacClassroom) controller()).paste();
			} // end if
		}, KeyStroke.getKeyStroke((char) KeyEvent.VK_V, java.awt.Event.META_MASK), WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				((MacClassroom) controller()).cut();
			} // end if
		}, KeyStroke.getKeyStroke((char) KeyEvent.VK_X, java.awt.Event.CTRL_MASK), WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				((MacClassroom) controller()).cut();
			} // end if
		}, KeyStroke.getKeyStroke((char) KeyEvent.VK_X, java.awt.Event.META_MASK), WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				controller().closeModel(self());
			} // end if
		}, KeyStroke.getKeyStroke((char) KeyEvent.VK_W, java.awt.Event.CTRL_MASK), WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				controller().closeModel(self());
			} // end if
		}, KeyStroke.getKeyStroke((char) KeyEvent.VK_W, java.awt.Event.META_MASK), WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				modelPane().deselectAll();
				modelPane().selectAll();
			} // end if
		}, KeyStroke.getKeyStroke((char) KeyEvent.VK_A, java.awt.Event.CTRL_MASK), WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				modelPane().deselectAll();
				modelPane().selectAll();
			} // end if
		}, KeyStroke.getKeyStroke((char) KeyEvent.VK_A, java.awt.Event.META_MASK), WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				modelPane().deselectAll();
			} // end if
		}, KeyStroke.getKeyStroke((char) KeyEvent.VK_D, java.awt.Event.CTRL_MASK), WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				modelPane().deselectAll();
			} // end if
		}, KeyStroke.getKeyStroke((char) KeyEvent.VK_D, java.awt.Event.META_MASK), WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		
		registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					Thread t = new Thread() {
						public void run () {
							((MacClassroom)controller()).printModel();
						} // method run
					}; // Anonymous Class Thread
					t.start();
				} catch (Exception err) {}
			} // end if
		}, KeyStroke.getKeyStroke((char) KeyEvent.VK_P, java.awt.Event.CTRL_MASK), WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				try {
					Thread t = new Thread() {
						public void run () {
							((MacClassroom)controller()).printModel();
						} // method run
					}; // Anonymous Class Thread
					t.start();
				} catch (Exception err) {}
			} // end if
		}, KeyStroke.getKeyStroke((char) KeyEvent.VK_P, java.awt.Event.META_MASK), WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				((MacClassroom)controller()).printPreviewButton().doClick();
			} // end if
		}, KeyStroke.getKeyStroke((char) KeyEvent.VK_P, java.awt.Event.CTRL_MASK | java.awt.Event.SHIFT_MASK), WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
		registerKeyboardAction(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				((MacClassroom)controller()).printPreviewButton().doClick();
			} // end if
		}, KeyStroke.getKeyStroke((char) KeyEvent.VK_P, java.awt.Event.META_MASK | java.awt.Event.SHIFT_MASK), WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
	} // method
	
	public ModelFrame (Model element) {
		this();
		reference(element.id());
		model(element);
		Runnable doStart = new Runnable() {
			public void run () {
				getContentPane().setLayout(new BorderLayout());
				getContentPane().add(toptoolbar(), BorderLayout.NORTH);
				getContentPane().add(mainPanel(), BorderLayout.CENTER);
				getContentPane().add(bottomtoolbar(), BorderLayout.SOUTH);
		//		modelPane().peer(element);
				setFrameIcon(ImageCache.getImage(model().icon()));
				setTitle(model().name());
				scaleComboBox().setSelectedIndex(4);
		//	   menuitem(new ModelFrameMenuItem(gui, this, getTitle(), image));
		//	   menuitem(new JRadioButtonMenuItem(getTitle(), model().icon(), true));
				menuItem();
				modechange(model().getInt(MODE));
			} };
		SwingUtilities.invokeLater(doStart);

	} // method
	
/*	
	public ModelFrame(VModelGUI gui, VModelModel model, int x, int y, int width, int height) {
		
	   menuitem(new ModelFrameMenuItem(gui, this, getTitle(), image));

		addInternalFrameListener(this);
		addKeyListener(gui());
		addComponentListener(this);

		modelInstance(new VModelInstanceModel(gui(), model(), model().modelElement()));
		modelInstance().displayMode(MODEL_MODE);
		getContentPane().add(modelInstance(), BorderLayout.CENTER);

		problemtext = new JTextArea(model().problemStatement());
		problemtext.setLineWrap(true);
		problemtext.setWrapStyleWord(true);
		problemtext.setRows(4);
		problemtext.getDocument().addDocumentListener(this);
		problemtext.setBackground(gui().preferences().getColor("COLOR"));
		scrollproblemtext = new JScrollPane(problemtext, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scrollproblemtext.setBorder(new TitledBorder(new LineBorder(Color.black, 2), "Problem Statement"));
		scrollproblemtext.setOpaque(true);
		scrollproblemtext.setBackground(gui().preferences().getColor("COLOR"));
		getContentPane().add(scrollproblemtext, BorderLayout.NORTH);

		
		show();
    	
	} // constructor
*/	

	// Inner Classes
	// -------------------------------------------------------------------------
	class VListRenderer extends JLabel implements ListCellRenderer {
	  public VListRenderer() {
	      setOpaque(true);
	  }
	  public Component getListCellRendererComponent(
	      JList list,
	      Object value,
	      int index,
	      boolean isSelected,
	      boolean cellHasFocus)
	  {
	  		if (value != null) {
		      setText(((JLabel) value).getText());
		      setIcon(((JLabel) value).getIcon());
		   } // end if
	      setBackground(isSelected ? Color.blue : Color.white);
	      setForeground(isSelected ? Color.white : Color.black);
	      return this;
	  }
	}
	// Properties & Accessors
	// -------------------------------------------------------------------------
	// VModelFrame Properties & Accessors
	protected String reference = null;
	public String reference () { return(reference); }
	public String reference (String m) { return(reference = m); }
	
	protected Model model = null;
	public Model model () { return(model); }
	public Model model (Model m) { return(model = m); }
	
//	protected JRadioButtonMenuItem menuitem = null;
//	public JRadioButtonMenuItem menuitem () { return(menuitem); }
//	public JRadioButtonMenuItem menuitem (JRadioButtonMenuItem m) { return(menuitem = m); }

/*	
	protected JToolBar toptoolbar = null;
	public JToolBar toptoolbar () { 
		if (toptoolbar == null) {
			toptoolbar = new JToolBar();
			toptoolbar.setLayout(new GridLayout(1, 2));
			toptoolbar.setFloatable(false);
			toptoolbar.add(new JButton("Model Target", ImageCache.getImage("target.gif")));
			//toptoolbar().addSeparator();
			toptoolbar.add(new JButton("Conditions for Model", ImageCache.getImage("vmodel.gif")));
		} // end if
		return(toptoolbar); 
	} // reader
*/
	
//	public static final String BRAINSTORMMODE = "Brainstorm Mode";
	public static final String TARGETMODE = "Target Mode";
	public static final String BUILDMODE = "Build Mode";
	public static final String ANALYSISMODE = "Analysis Mode";

//	protected JComboBox modeComboBox = null;
//	public JComboBox modeComboBox () {
//		if (modeComboBox == null) {
//			JLabel[] data = new JLabel[3];
////			data[0] =  new JLabel(BRAINSTORMMODE, ImageCache.getImage("coach-lb-h-s.gif"), JLabel.CENTER);
//			data[0] =  new JLabel(TARGETMODE, ImageCache.getImage("coach-t-h-s.gif"), JLabel.CENTER);
//			data[1] =  new JLabel(BUILDMODE, ImageCache.getImage("coach-b-h-s.gif"), JLabel.CENTER);
//			data[2] =  new JLabel(ANALYSISMODE, ImageCache.getImage("coach-e-h-s.gif"), JLabel.CENTER);
//			modeComboBox = new JComboBox(data);
//			modeComboBox.setRenderer(new VListRenderer());
//			modeComboBox.addActionListener(new ActionListener() {
//				public void actionPerformed(ActionEvent e) {
//					int index = modeComboBox().getSelectedIndex();
//					modechange(index);
//				} // method
//			});
//		} // end if
//		return(modeComboBox);
//	} // reader

	protected JComboBox scaleComboBox = null;
	public JComboBox scaleComboBox () {
		if (scaleComboBox == null) {
			JLabel[] data = new JLabel[12];
			data[0] =  new JLabel("15%", ImageCache.getImage("magnify.gif"), JLabel.CENTER);
			data[1] =  new JLabel("25%", ImageCache.getImage("magnify.gif"), JLabel.CENTER);
			data[2] =  new JLabel("50%", ImageCache.getImage("magnify.gif"), JLabel.CENTER);
			data[3] =  new JLabel("75%", ImageCache.getImage("magnify.gif"), JLabel.CENTER);
			data[4] =  new JLabel("100%", ImageCache.getImage("magnify.gif"), JLabel.CENTER);
			data[5] =  new JLabel("125%", ImageCache.getImage("magnify.gif"), JLabel.CENTER);
			data[6] =  new JLabel("150%", ImageCache.getImage("magnify.gif"), JLabel.CENTER);
			data[7] =  new JLabel("175%", ImageCache.getImage("magnify.gif"), JLabel.CENTER);
			data[8] =  new JLabel("200%", ImageCache.getImage("magnify.gif"), JLabel.CENTER);
			data[9] =  new JLabel("300%", ImageCache.getImage("magnify.gif"), JLabel.CENTER);
			data[10] =  new JLabel("400%", ImageCache.getImage("magnify.gif"), JLabel.CENTER);
			data[11] =  new JLabel("800%", ImageCache.getImage("magnify.gif"), JLabel.CENTER);
			scaleComboBox = new JComboBox(data);
			scaleComboBox.setRenderer(new VListRenderer());
			scaleComboBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					int index = scaleComboBox().getSelectedIndex();
					switch (index) {
						case 0 : modelPane().scale(0.15); break;
						case 1 : modelPane().scale(0.25); break;
						case 2 : modelPane().scale(0.50); break;
						case 3 : modelPane().scale(0.75); break;
						case 4 : modelPane().scale(1.00); break;
						case 5 : modelPane().scale(1.25); break;
						case 6 : modelPane().scale(1.50); break;
						case 7 : modelPane().scale(1.75); break;
						case 8 : modelPane().scale(2.00); break;
						case 9 : modelPane().scale(3.00); break;
						case 10 : modelPane().scale(4.00); break;
						case 11 : modelPane().scale(8.00); break;
						default : modelPane().scale(1.00);
					} // end switch
					self().repaint();
				} // method
			});
		} // end if
		return(scaleComboBox);
	} // reader
	
	protected JLabel problemstatementLabel = null;
	public JLabel problemstatementLabel () {
		if (problemstatementLabel == null) {
			problemstatementLabel = new JLabel();
			problemstatementLabel.setBackground(VModel.preferences().getColor("USERCOLOR"));
			problemstatementLabel().setOpaque(true);
			if (model() != null) problemstatementLabel().setText(model().getString("PROBLEM_STATEMENT"));
		} // end if
		return(problemstatementLabel);
	} // method

	protected JToolBar toptoolbar = null;
	public JToolBar toptoolbar () { 
		if (toptoolbar == null) {
			toptoolbar = new JToolBar();
			toptoolbar.setFloatable(false);
			toptoolbar().add(targetModeButton());
			toptoolbar().add(buildModeButton());
			toptoolbar().add(analysisModeButton());
			toptoolbar().addSeparator();
//			toptoolbar().addSeparator();
//			toptoolbar().add(viewList( ));
//			toptoolbar().addSeparator();
//			toptoolbar().add(scaleComboBox());
			toptoolbar().add(problemstatementLabel());
//			toptoolbar().add(modeComboBox());
		} // end if
		return(toptoolbar); 
	} // reader
	
	protected JToolBar bottomtoolbar = null;
	public JToolBar bottomtoolbar () { 
		if (bottomtoolbar == null) {
			bottomtoolbar = new JToolBar();
			bottomtoolbar.setFloatable(false);
//			bottomtoolbar.setLayout(new GridLayout(1,1));
//		bottomtoolbar().add(modeComboBox());
		bottomtoolbar().addSeparator();
		bottomtoolbar().add(viewList( ));
		bottomtoolbar().addSeparator();
		bottomtoolbar().add(scaleComboBox());
//		bottomtoolbar().add(problemstatementLabel());
		} // end if
		return(bottomtoolbar); 
	} // reader
	
	public void modechange (int mode) {
		final int m = mode;
		Runnable doStart = new Runnable() {
			public void run () {
						if ( !foobar ) {
						foobar = true;
				userClick = false;
				CardLayout layout = (CardLayout) mainPanel().getLayout();
				switch (m) {
					case 0 : {
						layout.show(mainPanel(), TARGETMODE); 
						if (! targetModeButton.isSelected()) targetModeButton().doClick(); 
//						if (modeComboBox().getSelectedIndex() != m) modeComboBox().setSelectedIndex(m);
						model().set(MODE, m);
						((MacClassroom)controller()).sidepanel().projectButton().doClick();
						controller().checkModel(model(), Model.CHECKTARGET, true);
						break;
					} // end case
					case 1 : {
						if (controller().checkModel(model(), Model.CHECKTARGET, true)) {
			for( Enumeration instances = model().instances().elements(); instances.hasMoreElements(); ) {
				((Instance) instances.nextElement()).set("HIGHLIGHTED", false);
			} // end for
							((MacClassroom)controller()).groupButton().setEnabled(true);
							((MacClassroom)controller()).groupButton().doClick();
							((MacClassroom)controller()).groupButton().doClick();
							modelPane().viewpane().setBackground(LIGHTBLUE); 
							modelPane().pageGuideColor(GRASSGREEN); 
							layout.show(mainPanel(), BUILDMODE); 
							if (! buildModeButton.isSelected()) buildModeButton().doClick(); 
//							if (modeComboBox().getSelectedIndex() != m) modeComboBox().setSelectedIndex(m);
							model().set(MODE, m);
							modelPane().deselectAll();
							((MacClassroom)controller()).sidepanel().librarian().libraryButton().doClick();
							long buildcheck = Model.CHECKSYNTAX;
							if (model().getBoolean("CHECKQLOOPS")) buildcheck = buildcheck | Model.CHECKQLOOP;
							controller().checkModel(model(), buildcheck, false);
						} else {
							JOptionPane.showMessageDialog(null, "Your model must have a problem statement.", "Target Coach", JOptionPane.ERROR_MESSAGE,ImageCache.getImage("coach-t-c-l.gif"));
							targetModeButton().doClick();
						} // end if
						break;
					} // end case
					case 2 : {
						if (!controller().checkModel(model(), Model.CHECKPROCESS, true)) {
							JOptionPane.showMessageDialog(null, "This model can't be analyzed because it isn't complete.  Your model must have a Process.", "Build Coach", JOptionPane.ERROR_MESSAGE,ImageCache.getImage("coach-b-c-l.gif"));
							buildModeButton().doClick();
						} else if (!controller().checkModel(model(), Model.CHECKINFLUENCE, false)) {
							JOptionPane.showMessageDialog(null, "This model can't be analyzed because it isn't complete.  Your model must have a Increases or a Decreases.", "Build Coach", JOptionPane.ERROR_MESSAGE,ImageCache.getImage("coach-b-c-l.gif"));
							buildModeButton().doClick();
						} else if (!controller().checkModel(model(), Model.CHECKQLOOP, false)) {
							JOptionPane.showMessageDialog(null, "This model can't be analyzed.  There is a loop of influences in your model.", "Build Coach", JOptionPane.ERROR_MESSAGE,ImageCache.getImage("coach-b-c-l.gif"));
							model().set("CHECKQLOOPS",true);
							buildModeButton().doClick();
						} else if (!controller().checkModel(model(), Model.CHECKPREDICTIONEXISTS, false)) {
							JOptionPane.showMessageDialog(null, "This model can't be analyzed.  There are no predictions.  Right click on a parameter and make a prediction.", "Build Coach", JOptionPane.ERROR_MESSAGE,ImageCache.getImage("coach-b-c-l.gif"));
							buildModeButton().doClick();
						} else if (!controller().checkModel(model(), Model.CHECKABIGUITY, false)) {
							modelPane().viewpane().setBackground(GRASSGREEN); 
							modelPane().pageGuideColor(LIGHTBLUE); 
							layout.show(mainPanel(), BUILDMODE); 
							if (! analysisModeButton.isSelected()) analysisModeButton().doClick(); 
							model().set(MODE, m);
							((MacClassroom)controller()).sidepanel().coachButton().doClick();
							modelPane().viewpane().repaint();
							JOptionPane.showMessageDialog(null, "This model can't be analyzed.  There are multiple arrows going into a parameter that would change it in opposite directions. Fix this by making an arrow stronger or by making an assumption (right click).", "Professor Coach", JOptionPane.ERROR_MESSAGE,ImageCache.getImage("coach-e-c-l.gif"));
/*						} else {
							modelPane().viewpane().setBackground(GRASSGREEN); 
							modelPane().pageGuideColor(LIGHTBLUE); 
							layout.show(mainPanel(), BUILDMODE); 
//							if (modeComboBox().getSelectedIndex() != m) modeComboBox().setSelectedIndex(m);
							if (! analysisModeButton.isSelected()) analysisModeButton().doClick(); 
							model().set(MODE, m);
							((MacClassroom)controller()).sidepanel().coachButton().doClick();
							modelPane().viewpane().repaint();

							if (!controller().checkModel(model(), Model.CHECKABIGUITY, false)) {
								JOptionPane.showMessageDialog(null, "This model can't be analyzed.  There are multiple arrows going into a parameter that would change it in opposite directions. Fix this by making an arrow stronger or by making an assumption (right click).", "Professor Coach", JOptionPane.ERROR_MESSAGE,ImageCache.getImage("coach-e-c-l.gif"));
	//							buildModeButton().doClick();
							}
							model().set("CHECKQLOOPS",false);
							modelPane().viewpane().setBackground(GRASSGREEN); 
							modelPane().pageGuideColor(LIGHTBLUE); 
//							layout.show(mainPanel(), BUILDMODE); 
////							if (modeComboBox().getSelectedIndex() != m) modeComboBox().setSelectedIndex(m);
//							if (! analysisModeButton.isSelected()) analysisModeButton().doClick(); 
//							model().set(MODE, m);
							controller().checkModel(model(), Model.CHECKGRAMMAR, false);
							controller().simulate(model());
//							((MacClassroom)controller()).sidepanel().coachButton().doClick();
//							modelPane().viewpane().repaint();
*/
						} else {
							model().set("UNSKINNED", true);
							modelPane().viewpane().setBackground(GRASSGREEN); 
							modelPane().pageGuideColor(LIGHTBLUE); 
							layout.show(mainPanel(), BUILDMODE); 
							if (! analysisModeButton.isSelected()) analysisModeButton().doClick(); 
							model().set(MODE, m);
							((MacClassroom)controller()).sidepanel().coachButton().doClick();
							modelPane().viewpane().repaint();
							controller().checkModel(model(), Model.CHECKGRAMMAR, false);
							controller().runSimulation(model);

						} // end if
						break;
					} // end case
					default : {
						layout.show(mainPanel(), BUILDMODE); 
//						if (modeComboBox().getSelectedIndex() != m) modeComboBox().setSelectedIndex(1);
						if (! buildModeButton.isSelected()) buildModeButton().doClick();
						((MacClassroom)controller()).sidepanel().librarian().libraryButton().doClick();
						model().set(MODE, 1);
							long buildcheck = Model.CHECKSYNTAX;
							if (model().getBoolean("CHECKQLOOPS")) buildcheck = buildcheck | Model.CHECKQLOOP;
							controller().checkModel(model(), buildcheck, true);
					} // end default case 
				} // end switch
				modelPane().deselectAll();
//				controller().checkModel(model(), Model.CHECKALL);
				modelPane().updateContextHelp();
				controller().updatePeers(model());
				controller().updateWidgets();
				userClick = true;
						foobar = false;
						} // end if
			} };
		SwingUtilities.invokeLater(doStart);
	} // method
	
	protected boolean foobar = false;

	protected ButtonGroup modeButtonGroup = null;
	public ButtonGroup modeButtonGroup () {
		if (modeButtonGroup == null) modeButtonGroup = new ButtonGroup();
		return(modeButtonGroup);
	} // reader

	public static boolean userClick = true;	
	protected JToggleButton targetModeButton = null;
	public JToggleButton targetModeButton () {
		if (targetModeButton == null) {
			targetModeButton = new JToggleButton(TARGETMODE, ImageCache.getImage("nobackground_16_target.GIF"));
			modeButtonGroup().add(targetModeButton);
			targetModeButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (userClick && (model().getInt(MODE) != 0)) {
						modechange(0);
						if ((((int) Math.floor(Math.random() * (100))) < 5) &&
								(VModel.preferences().getString("USERID").equalsIgnoreCase("developer") ||
								VModel.preferences().getString("USERID").equalsIgnoreCase("ureel") ||
								VModel.preferences().getString("USERID").equalsIgnoreCase("kcarney")))
							SoundCache.playSound("cow-2.au");
						else SoundCache.playSound("click1.au");
					} // end if
				} // method
			});
		} // end if
		return(targetModeButton);
	} // reader

	protected JToggleButton buildModeButton = null;
	public JToggleButton buildModeButton () {
		if (buildModeButton == null) {
			buildModeButton = new JToggleButton(BUILDMODE, ImageCache.getImage("nobackground_16_build.GIF"));
			modeButtonGroup().add(buildModeButton);
			buildModeButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (userClick && (model().getInt(MODE) != 1)) {
						modechange(1);
						if ((((int) Math.floor(Math.random() * (100))) < 5) &&
								(VModel.preferences().getString("USERID").equalsIgnoreCase("developer") ||
								VModel.preferences().getString("USERID").equalsIgnoreCase("ureel") ||
								VModel.preferences().getString("USERID").equalsIgnoreCase("kcarney")))
							SoundCache.playSound("cow-2.au");
						else SoundCache.playSound("click1.au");
					} // end if
				} // method
			});
		} // end if
		return(buildModeButton);
	} // reader

	protected JToggleButton analysisModeButton = null;
	public JToggleButton analysisModeButton () {
		if (analysisModeButton == null) {
			analysisModeButton = new JToggleButton(ANALYSISMODE, ImageCache.getImage("nobackground16-einstein.GIF"));
			modeButtonGroup().add(analysisModeButton);
			analysisModeButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (userClick && (model().getInt(MODE) != 2)) {
						modechange(2);
						if ((((int) Math.floor(Math.random() * (100))) < 5) &&
								(VModel.preferences().getString("USERID").equalsIgnoreCase("developer") ||
								VModel.preferences().getString("USERID").equalsIgnoreCase("ureel") ||
								VModel.preferences().getString("USERID").equalsIgnoreCase("kcarney")))
							SoundCache.playSound("cow-2.au");
						else SoundCache.playSound("click1.au");
					} // end if
				} // method
			});
		} // end if
		return(analysisModeButton);
	} // reader
	
	protected JComboBox viewList = null;
	public JComboBox viewList ( ) {
		if ( viewList == null ) {
			JLabel[] data2 = new JLabel[7];
			data2[0] =  new JLabel("Normal View", ImageCache.getImage("glasses.gif"), JLabel.CENTER);
			data2[1] =  new JLabel("Entity View", ImageCache.getImage("glasses-entity.gif"), JLabel.CENTER);
			data2[2] =  new JLabel("Process View", ImageCache.getImage("glasses-process.gif"), JLabel.CENTER);
			//data2[3] =  new JLabel("Relationship View", ImageCache.getImage("glasses-relationship.gif"), JLabel.CENTER);
			//data2[4] =  new JLabel("Situation View", ImageCache.getImage("glasses-situation.gif"), JLabel.CENTER);
			data2[3] =  new JLabel("Causal View", ImageCache.getImage("glasses-causal.gif"), JLabel.CENTER);
			data2[4] =  new JLabel("Justification View", ImageCache.getImage("glasses-justification.gif"), JLabel.CENTER);
			data2[5] =  new JLabel("Coach View", ImageCache.getImage("glasses-coach.gif"), JLabel.CENTER);
			//data2[8] =  new JLabel("Annotation View", ImageCache.getImage("glasses-annotation.gif"), JLabel.CENTER);
			//data2[9] =  new JLabel("Role View", ImageCache.getImage("glasses-role.gif"), JLabel.CENTER);
			data2[6] =  new JLabel("Q-Physics View", ImageCache.getImage("glasses-qp.gif"), JLabel.CENTER);
			//data2[11] =  new JLabel("Target View", ImageCache.getImage("glasses-target.gif"), JLabel.CENTER);
			viewList = new JComboBox(data2);
			viewList.setRenderer(new VListRenderer());
			viewList.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					modelPane().view( viewList().getSelectedIndex() );
					self().repaint();
				} // method
			});
		} // end if 
		return( viewList ); 
	} // method

	protected JScrollPane scrollpane = null;
	public JScrollPane scrollpane () {
		if (scrollpane == null) scrollpane = new JScrollPane();
		return(scrollpane);
	} // reader

	protected JPanel brainstormModePane = null;
	public JPanel brainstormModePane () {
		if (brainstormModePane == null) {
			brainstormModePane = new JPanel(new BorderLayout());
			brainstormModePane.setBackground(YELLOW);
		} // end if
		return(brainstormModePane);
	} // reader

// goals
// undeclared = 0
// brainstorm = 1
// describe & relate = 2
// model change = 3

	protected JTextArea explainarea = null;
	public JTextArea explainarea() {
		if (explainarea == null) {
			explainarea = new JTextArea(model().getString("PROBLEM_STATEMENT"));
			explainarea.setBackground(VModel.preferences().getColor("USERCOLOR"));	
			explainarea.setBorder(new TitledBorder(new LineBorder(Color.black, 2), "Problem Statement"));		
			explainarea.getDocument().addDocumentListener(this); // DocumentListener
		} // end if
		return(explainarea);
	} // method

	
	protected JPanel targetModePane = null;
	public JPanel targetModePane () {
		if (targetModePane == null) {
			targetModePane = new JPanel(new BorderLayout());
			targetModePane.setBackground (VModel.preferences().getColor("USERCOLOR"));
//			JTextArea problemexplanation = new JTextArea();
			JTextPane problemexplanation = new JTextPane();
			problemexplanation.setContentType("text/html");
//			problemexplanation.setText("\n\nVmodel models help to explain why something is changing, or they help you predict how something might change. The thing that changes is something that can be measured with a number, such as amount, rate, level, etc. \n\n In the box below you need to include a model target. In your model target, specify the thing that is changing and say if it is increasing or decreasing.  \n\nFor example: This model will explain why the level of the ocean is rising.\n\n");
			problemexplanation.setText("<P>Vmodel models help to explain why something is changing, or they help you predict how or why something might change. The thing that changes is something that can be described with a number, such as amount, rate, level, etc. In the box below you need to include a model target. </P><P><B>The model target is a statement of the thing the model is trying to explain or predict. </B></P><UL><LI>If you are making a model to explain a change you have observed, the model target should specify the thing that is changing and say if it is increasing or decreasing. <I>For example: This model will explain why the level of the ocean is rising.</I> </LI><LI>If you are making a model to try to predict how something will change you can specify what you care about and what might influence it. <I>For example: If we keep chopping down the rainforests, what will happen to the amount of oxygen in the atmosphere?</I></LI></UL>");
			problemexplanation.setBackground (WHITE);
			explainarea().setLineWrap(true);
			explainarea().setWrapStyleWord(true);
			problemexplanation.setEditable(false);
			targetModePane.add(problemexplanation, BorderLayout.NORTH);
			targetModePane.add(explainarea(), BorderLayout.CENTER);
			targetModePane.setBackground(SKYBLUE);
		} // end if
		return(targetModePane);
	} // reader

	protected JPanel buildModePane = null;
	public JPanel buildModePane () {
		if (buildModePane == null) {
			buildModePane = new JPanel(new BorderLayout());
			buildModePane.setBackground(LIGHTBLUE);
			//buildModePane.setBackground(GREEN);
			buildModePane.add(modelPane(), BorderLayout.CENTER);
			buildModePane.setOpaque(true);
		} // end if
		return(buildModePane);
	} // reader

	protected ModelPane modelPane = null;
	public ModelPane modelPane () {
		if (modelPane == null) {
			modelPane = new ModelPane(model());
			modelPane.setOpaque(false);
		} // end if
		return(modelPane);
	} // reader

	protected JPanel mainPanel = null;
	public JPanel mainPanel () {
		if (mainPanel == null) {
			CardLayout layout = new CardLayout();
			mainPanel = new JPanel(layout);
//			mainPanel.add(brainstormModePane(), BRAINSTORMMODE);
			mainPanel.add(targetModePane(), TARGETMODE);
			mainPanel.add(buildModePane(), BUILDMODE);
//			mainPanel.add(analysisModePane(), ANALYSISMODE);
		} // end if
		return(mainPanel);
	} // reader
	
	protected boolean dirty = false;
	public boolean dirty () { return(dirty); }
	public boolean dirty (boolean b) {
		if (b) {
			setTitle("* " + model().name());
		} else {
			setTitle(model().name());
		} // end if
/*		
		int mode = model.getInt("MODE");
		long buildcheck = Model.CHECKSYNTAX;
		if (model().getBoolean("CHECKQLOOPS")) buildcheck = buildcheck | Model.CHECKQLOOP;
		if (mode == 0) controller().checkModel(model(), Model.CHECKTARGET);
		else if (mode == 1) controller().checkModel(model(), buildcheck);
		else if (mode == 2) controller().checkModel(model(), Model.CHECKALL);
		else controller().checkModel(model(), buildcheck);
		controller().updatePeers(model());
*/		
		controller().updateWidgets();
		return(dirty = b);
	} // method
	
	// Methods
	// --------------------------------------------------------------------------
	//public boolean isIcon () { return(false); }
	//public void setIcon (boolean s) throws PropertyVetoException { }
	
	public void select (boolean b) {
//		QRGutils.debug("ModelFrame.select", "b=" + b);
		menuItem().setSelected(b);
		if (b) {
			show();
			try { setSelected(true); }
			catch (PropertyVetoException e) { };
		} else {
			((JDesktopPane) getParent()).getDesktopManager().deactivateFrame(this);
		} // if b
	} // method select

	public int close () {
		int result = -1;
// Need to remove elements from allDefinitions and allRelationships and allOntologies etc.
			dispose();
			result = 0;
		return(result);
	} // method close
	
	public void dispose () { super.dispose(); }

	public void print (Graphics g) {
		super.print(g);
/*
		VModelElement modelelement = model().modelElement();
		VModelInstanceModel modelpeer = ((VModelInstanceModel) (modelelement.peer()));
		VModelPeerElement foo = null;
		VModelCAD cad = modelpeer.cadPane();
		JPanel pane = cad.componentpane;
		JComponent panel = null;
		boolean opaque = false;
		Color color = null;
		int x,y,w,h;
		g.setColor(WHITE);
		g.fillRect(0,0, pane.getSize().width, pane.getSize().height);
		pane.print(g);


		for(int i = 0; i < pane.getComponentCount(); i++) {
			foo = (VModelPeerElement) pane.getComponent(i);

			x = foo.getX();
			y = foo.getY();
			w = foo.getSize().width;
			h = foo.getSize().height;
			g.setClip(x, y, w, h);
			pane.print(g);
			g.translate(x,y);
			g.setClip(0, 0, w, h);
			color = foo.getBackground();
			opaque = foo.isOpaque();
			foo.printAll(g);	
			g.translate(-x,-y);
	} // for i
*/	
	} // method print

	public void doDefaultCloseAction() {
		//QRGutils.debug("ModelFrame", "doDefaultCloseAction");
		if (QRGutils.detectPlatform() == MACINTOSH) {
			switch(getDefaultCloseOperation()) {
				case HIDE_ON_CLOSE:
					try {
					setClosed(true);
					} catch (PropertyVetoException pve) {}
					break;
				case DISPOSE_ON_CLOSE:
					try {
					setClosed(true);
					dispose();  // only executes if close wasn't vetoed.
					} catch (PropertyVetoException pve) {}
					break;
				case DO_NOTHING_ON_CLOSE:
					default: 
					break;
			}
		} else {
			fireInternalFrameEvent(InternalFrameEvent.INTERNAL_FRAME_CLOSING);
			switch(getDefaultCloseOperation()) {
				case DO_NOTHING_ON_CLOSE:
					break;
				case HIDE_ON_CLOSE:
					setVisible(false);
					if (isSelected())
						try {
							setSelected(false);
						} catch (PropertyVetoException pve) {}

					/* should this activate the next frame? that's really
					desktopmanager's policy... */
					break;
				case DISPOSE_ON_CLOSE:
					try {
						fireVetoableChange(IS_CLOSED_PROPERTY, Boolean.FALSE,
						Boolean.TRUE);
						isClosed = true;
						firePropertyChange(IS_CLOSED_PROPERTY, Boolean.FALSE,
						Boolean.TRUE);
						dispose();
					} catch (PropertyVetoException pve) {}
					break;
				default: 
					break;
			}
		}
	}

//	public void setClosed(boolean b) throws PropertyVetoException {
//		if (dirty() && 
//			 (JOptionPane.YES_OPTION != JOptionPane.showConfirmDialog(null, "The model, " + model().getString("LABEL") + ", has not been saved.\n Do you want to close it anyway?", "Alert! Model Not Saved.", JOptionPane.YES_NO_OPTION))) 
//			throw(new PropertyVetoException("sorry", new PropertyChangeEvent(this, "sorry", this, this)));
//		else super.setClosed(b);
//	} // method setClosed
	
	public Controller controller () {
		Controller result = null;
		result = (Controller) VModel.controller();
		return(result);
	} // method

	// InternalFrameListener
	// --------------------------------------------------------------------------
	public void internalFrameOpened(InternalFrameEvent e) {
	} // method internalFrameOpened
	public void internalFrameClosing(InternalFrameEvent e) {
//		close();
			if (QRGutils.detectPlatform() == WINDOWS) {
				if (controller().closeModel(self()) == 0) controller().closeModel(self(), true);
			} else controller().closeModel(self(), true);
	} // method internalFrameClosing
	public void internalFrameClosed(InternalFrameEvent e) {
	} // method internalFrameClosed
	public void internalFrameIconified(InternalFrameEvent e) {
	} // method internalFrameIconified
	public void internalFrameDeiconified(InternalFrameEvent e) {
	} // method internalFrameDeiconified
	public void internalFrameActivated(InternalFrameEvent e) {
		menuItem().setSelected(true);
//		invalidate();
		controller().openWindows().repaint();
		controller().currentModel((Model) model());
		controller().selectedWindow(self());
//		Librarian.currentWindowFragment((Model) model());
// 		if (gui().selectedModel() != ((ModelFrame) e.getSource()).model()) gui().select(this);
	} // method internalFrameActivated
	public void internalFrameDeactivated(InternalFrameEvent e) {
//		Modeler.currentModel(null);
//		Modeler.selectedWindow(null);
	} // method internalFrameDeactivated
		
	// ComponentListener Methods
	// --------------------------------------------------------------------------
	public void componentHidden(ComponentEvent e) {
	} // method componentHidden
	public void componentMoved(ComponentEvent e) {
	} // method componentMoved
	public void componentResized(ComponentEvent e) {
	} // method componentResized
	public void componentShown(ComponentEvent e) {
	} // method componentShown

	// MenuItem
	protected JRadioButtonMenuItem menuItem = null;
	public JRadioButtonMenuItem menuItem () {
		if (menuItem == null) {
			menuItem = new JRadioButtonMenuItem(getTitle(), getFrameIcon());
			menuItem.addActionListener(new ActionListener() {
  				public void actionPerformed(ActionEvent e) {
					try {
						self().show();
						self().toFront();
						self().setVisible(true);
						self().setSelected(true);
					} catch (PropertyVetoException pve) {}
				} // method actionPerformed
  			});
		} // end if
		return(menuItem);
	} // reader
	
	public ModelFrame self () { return(this); }
	
	public void setTitle (String title) {
		super.setTitle(title);
		menuItem().setText(title);
	} // method
	
	public void insertUpdate(DocumentEvent e) { textUpdate(); }
	public void removeUpdate(DocumentEvent e) { textUpdate(); }
	public void changedUpdate(DocumentEvent e) { textUpdate(); }

	public void textUpdate() { 
		model().set("PROBLEM_STATEMENT", explainarea().getText());
		problemstatementLabel().setText(model().getString("PROBLEM_STATEMENT"));
		self().dirty(true);
	} // method textUpdate
	
	public void updateWidgets () {
		targetModeButton().setEnabled(true);
		buildModeButton().setEnabled(true);
//		long buildcheck = Model.CHECKSYNTAX;
//		if (model().getBoolean("CHECKQLOOPS")) buildcheck = buildcheck | Model.CHECKQLOOP;
		analysisModeButton().setEnabled( (model().getInt("MODE") == 2) ||
										//controller().checkModel(model(), Model.CHECKTARGET) && 
//										controller().checkModel(model(), buildcheck) && 
											((model().getInt(MODE) > 0) &&
											(!model().instances().isEmpty()) && 
											(model().bugs().isEmpty())));
//		problemstatementLabel().setText(model().getString("PROBLEM_STATEMENT"));
//		viewList().setEnabled(model().getInt(MODE) == 1);
		viewList().setEnabled(false);
		scaleComboBox().setEnabled(model().getInt(MODE) >= 1);

	} // method
	
	public String annotation = "";
	public void annotate () {
		final JDialog d = new JDialog();
		Container c = d.getContentPane();
		JTextArea t = new JTextArea(model.getString(NOTES));
		JScrollPane s = new JScrollPane(t);
		QRGpanel buttons = new QRGpanel(new BorderLayout());
		JButton ok = new JButton("OK");
		buttons.add(ok, BorderLayout.EAST);
		ok.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				d.setVisible(false);
			} // method actionPerformed
		}); // actionListener
		t.setBorder(new EmptyBorder(5, 5, 5, 5));
		t.setLineWrap(true);
		c.setLayout(new BorderLayout());
		c.add(new JLabel(ImageCache.getImage("wizard.gif")), BorderLayout.WEST);
		c.add(new JLabel("Enter Notes:"), BorderLayout.NORTH);
		c.add(s, BorderLayout.CENTER);
		c.add(buttons, BorderLayout.SOUTH);
		d.setModal(true);
		d.setSize(300,200);
		d.setLocation(QRGutils.center(d.getSize()));
		d.show();
		annotation = t.getText();
		if (! annotation.equals(model.getString(NOTES))) {
			model.set(NOTES, annotation);
			controller().selectedWindow().dirty(true);
//			Modeler.self().currentModel().log("Annotation", "element=" + id() + " text=\"" + replaceReturns(annotation()) +"\"");
		} // end if
		d.dispose();
	} // method annotate

} // Class VModelGUIiModel

// --------------------------------------------------------------------------
// End of File: VModelGUIModelFrame.java     (c) 2001 Northwestern University