package edu.qrg.vmodel.controllers.common;

import edu.qrg.utils.*;
import edu.qrg.vmodel.*;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.lang.Cloneable;
import java.lang.Exception;
import java.lang.Object;
import java.lang.String;
import java.net.URL;
import java.util.Vector;
import sun.audio.AudioPlayer;
import sun.audio.AudioStream;

/** <PRE> Sun Java 1.3.1 -Sourcepath QRG/VModel/development/code/gui
* ----------------------------------------------------------------------------
*  2003 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
* ----------------------------------------------------------------------------
* File name: SoundCache.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* ----------------------------------------------------------------------------
*   Class: SoundCache
* Purpose: Sound manager.
* ----------------------------------------------------------------------------
* </PRE> */
public class SoundCache implements VModelConstants {
	// sounds
   protected static Vector sounds = null;
	public static Vector sounds() {
		if (sounds == null) sounds = new Vector(); 
		return(sounds); 
	} // reader sounds
	public static Vector sounds(Vector v) { return(sounds = v); }

    // SOUND CACHE
    // -----------------------------------------------------------------------
    // Description: Finds an sound in the cache.
    // Side Effects: 
    // -----------------------------------------------------------------------
    public static KeyValuePair findSound(String filename) {
		KeyValuePair attribute = null;
		for(int i = 0; i < sounds().size(); i++) {
		    attribute = (KeyValuePair) sounds().elementAt(i);
		    if (attribute.key().equalsIgnoreCase(filename)) break;
		    else attribute = null;
		} // end for i
		return(attribute);
    } // method getSound

    // Description: Loads and caches an sound file. 
    // Side Effects: Overwrites previously cached load of file.
    // -----------------------------------------------------------------------
    public static AudioStream loadSound (String filename) {
		String mumble = QRGutils.QRGroot()+SOUNDPATH+filename;
		KeyValuePair grumble = findSound(filename);
		AudioStream result = null;
		if ((!QRGutils.isApplet()) && (grumble == null) && 
			(!((new File(mumble)).exists())))
			QRGutils.debug("loadSound", "File Not Found: " + mumble);
		else {
			try {
				result = new AudioStream(new FileInputStream(QRGutils.QRGroot() + SOUNDPATH + filename));
				result.mark(result.getLength());
			} catch (IOException e) { result = null; }
			if (result != null) {
			    if (grumble == null)
					sounds().addElement(new KeyValuePair(filename, result));
			    else grumble.value(result);
			} else if (grumble != null) result = (AudioStream) grumble.value();
		} // end if exists
		return(result);
    } // method loadSound

    // Description: Gets/Loads a cached sound file. 
    // Side Effects: Loads file if not in the cache.
	// -----------------------------------------------------------------------
    public static AudioStream getSound(String filename) {
		AudioStream result = null;
		KeyValuePair attribute = findSound(filename);
		if (attribute == null) result = loadSound(filename);
		else {
			result = (AudioStream) attribute.value();
			try { result.close(); } catch (IOException e) { }
			result = loadSound(filename);
		} // end if else
		return(result);
    } // method getSound

	public static void playSound (String filename) {
		if (VModel.preferences().getBoolean("SOUND")) {
			if (QRGutils.isApplet()) {
				try { 
					QRGutils.applet().play(new URL(QRGutils.QRGroot()), 
							SOUNDPATH + filename);
				} catch (Exception e) { e.printStackTrace(); }
			} else {
				try {
					AudioStream as = getSound(filename);
					AudioPlayer.player.start(as);
				} catch (Exception e) { e.printStackTrace(); }
			} // end if else
		} // end if
	} // method playSound

} // class SoundCache
        
// -----------------------------------------------------------------------------
// End of File: SoundCache.java       (c) 2001 Northwestern University