package edu.qrg.vmodel;

import edu.qrg.utils.*;

import java.io.Serializable;

import java.lang.Cloneable;
import java.lang.Object;
import java.lang.String;

import java.util.Vector;

/** <PRE> Sun Java 1.3.1 -Sourcepath QRG/VModel/development/code/engine
* ----------------------------------------------------------------------------
*  2003 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
* ----------------------------------------------------------------------------
* File name: PREFERENCES.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* ----------------------------------------------------------------------------
*   Class: PREFERENCES
* Purpose: Individual PREFERENCES Object.  Each PREFERENCES object may
*          contain several key-value pairs and also many child preferences.
* ----------------------------------------------------------------------------
* </PRE> */
public class PREFERENCES extends PList 
	implements Cloneable, VModelConstants, Serializable, XMLio.XMLable {
	// Constructors
	// ------------------------------------------------------------------------
	public PREFERENCES() { super(); } // default constructor
	
	public PREFERENCES(String tag) {
		this();
		tag(tag);
	} // constructor
	
	public PREFERENCES (Vector keyValues, Vector children) {
		tag("PREFERENCES");
		plist(keyValues);
		// We are not processing child preferences at this time.
		children(children);
	} // constructor
	
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Initializes slots after object is created by XML loader.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public void XMLioObjectInit (Vector keyValues, Vector children) {
		tag("PREFERENCES");
		plist(keyValues);
		// We are not processing child preferences at this time.
		children(children);
	} // constructor
	
	// Properties & Accessors
	// ------------------------------------------------------------------------
	/** @serial */
	protected String tag = "";
	public String tag () { return(tag); }
	public String tag (String s) { return(tag = s); }
	
	/** @serial */
	protected Vector children = null;
	public Vector children () {
		if (children == null) children = new Vector();
		return(children);
	} // reader
	public Vector children (Vector v) { return(children = v); }
	
	/** @deprecated Use keyValuePairs(). */
	public Vector attributes () { return(keyValuePairs()); }
	/** @deprecated Use keyValuePairs(Vector). */
	public Vector attributes (Vector v) { return(keyValuePairs(v)); }
	
	/** @deprecated Use children(). */
	public Vector elements () { return(children()); }
	/** @deprecated Use children(Vector). */
	public Vector elements (Vector v) { return(children(v)); }
	
	// Stringification.
	// -----------------------------------------------------------------------
	public String toString (int indent) {
		String result = "";
		int i = indent, j = 0;
		result = QRGutils.space(i) + "<" + tag.toUpperCase();
		result += stringifyKeyValuePairs(result.length() + 1);
		result += " />\r\n";
		return(result);
	} // method toString
	
	public String toString () { 
		return(toString(0)); 
	} // method toString
	
	// Cloning.
	// ------------------------------------------------------------------------
	public Object clone() {
		Object result = new PREFERENCES(new String(tag()));
		((PREFERENCES) result).copyKeyValuePairs(this);
		return(result);
	} // method clone
	
	public void copyKeyValuePairs(PREFERENCES e) {
		for(int j = 0; j < e.keyValuePairs().size(); j++) {
			KeyValuePair keyValuePair = (KeyValuePair) e.keyValuePairs().elementAt(j);
			if (keyValuePair.value() instanceof String) 
				set(new String(keyValuePair.key()), new String((String) keyValuePair.value()));
			else set(new String(keyValuePair.key()), keyValuePair.value());
		} // for j
	} // method
	
	// XMLio.XMLable methods	
	// ------------------------------------------------------------------------
	public String xmlDef () { 
		String result = 	"<!DOCTYPE PREFERENCES [\r\n" +
								"<!ELEMENT PREFERENCES EMPTY >\r\n" +
								"<!ATTLIST PREFERENCES CONTROLLER CDATA #REQUIRED >\r\n" +
								"]>";
		return(result); 
	} // method xmlDef
	public String xmlify () { return(this.toString()); }

	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Saves the ontology to filename in XML format.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public void exportXML (String filename) {
		XMLio.save(QRGutils.QRGuserpath() + filename + "." + XML_SUFFIX , this);
	} // method
	
} // class

// ---------------------------------------------------------------------------
// END OF FILE: PREFERENCES.java                 2003 Northwestern University