package edu.qrg.vmodel;

import edu.qrg.utils.*;

import edu.qrg.utils.KeyValuePair;
import edu.qrg.utils.KeyValuePair.KVPinterface;
import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.Panel;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.Serializable;
import java.lang.Boolean;
import java.lang.Class;
import java.lang.ClassNotFoundException;
import java.lang.Cloneable;
import java.lang.Exception;
import java.lang.IllegalAccessException;
import java.lang.InstantiationException;
import java.lang.NoSuchMethodException;
import java.lang.Object;
import java.lang.String;
import java.lang.System;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.util.Enumeration;
import java.util.EventListener;
import java.util.Stack;
import java.util.Vector;
import javax.swing.ImageIcon;
import javax.swing.JApplet;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JWindow;
import javax.swing.Timer;
import javax.swing.tree.TreeNode;
import javax.swing.UIManager;

/** <PRE> Sun Java 1.3.1 -Sourcepath QRG/VModel/development/code/engine
* ----------------------------------------------------------------------------
*  2003 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
* ----------------------------------------------------------------------------
* File name: VModel.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* ----------------------------------------------------------------------------
*   Class: VModel
* Purpose: Main VModel class. This is the class that should be loaded to
*          start the system for either applet or application.
* ----------------------------------------------------------------------------
* NOTES:
* ----------------------------------------------------------------------------
* </PRE> */
public class VModel extends JApplet 
		implements VModelConstants , edu.qrg.vmodel.VModel.VMController {
	// Constructors
   // ------------------------------------------------------------------------
	public VModel() {
		super();
	} // constructor

	// Interface
   // ------------------------------------------------------------------------
   
	public interface VMController extends VModelConstants, Cloneable, Serializable {
		// Properties and Accessors
		// ------------------------------------------------------------------------

		/** <PRE> 
		* Test to determine if this is an applet.
		* Since an instance of the applet class is always created,
		* assume true and negate in application main.
		* </PRE>*/
		public boolean isApplet ();
		public boolean isApplet (boolean b);

		// The controller controls the system.
		public Class controllerClass ();
		public Class controllerClass (Class c);

		// The controller controls the system.
//		public static VModel.VMController controller ();
		public VModel.VMController controller (VModel.VMController c);
		
		// Launchpad class
//		public static VModel applet ();
//		public static VModel applet (VModel v);

		// Preferences
//		public static VModelPreferences preferences ();
//		public static VModelPreferences preferences (VModelPreferences p);

		// used?
		public void updatePreferences ();

		// Project
		public PROJECT project ();
		public PROJECT project (PROJECT p);

		// Splash Screen
		public Container VModelGUISplash();
		public Container VModelGUISplash(Container c);
		
		// BaseOntology
		public Ontology ontology ();
		public Ontology ontology (Ontology o);
		
		// Libraries
		// -----------------------------------------------------------------------
		public Ontology makeOntology (String label, String documentation,
												 String helpURL);
		
		public Ontology makeCategory (String label, String documentation,
												 String helpURL);
		
		public Ontology classModels ();
		public Ontology classModels (Ontology o);

		public Ontology vModels ();
		public Ontology vModels (Ontology o);

		public Ontology myModels ();
		public Ontology myModels (Ontology o);

		public Ontology myEntities ();
		public Ontology myEntities (Ontology o);

		public Ontology myThings ();
		public Ontology myThings (Ontology o);

		public Ontology myMultipleThings ();
		public Ontology myMultipleThings (Ontology o);

		public Ontology mySubstances ();
		public Ontology mySubstances (Ontology o);

		public Ontology myProcesses ();
		public Ontology myProcesses (Ontology o);

		public Ontology myAttributes ();
		public Ontology myAttributes (Ontology o);

		public Ontology projectOntology ();
		public Ontology projectOntology (Ontology o);

		public Ontology myLibrary ();
		public Ontology myLibrary (Ontology o);

		public Ontology library ();
		
		public boolean userIsDeveloper( );
		
		// VModel.VMController methods
		// ------------------------------------------------------------------------
		public Definition findDefinition (String id);
		
		public Definition findDefinition (Model model, String id);
		
		public void go ();
		
		public boolean quit ();
		
		public String makeHelpURL(String base);
		
		// Project File IO
		// --------------------------------------------------------------------------
		public PROJECT loadProject(String projectFileName);

		public void saveProject ();

		public void saveProject (String filename);
		
		// Methods
	   // ------------------------------------------------------------------------
	   // Attempt to automatically set the QRGroot path.
		public void setQRGroot (String inifile);
		
		// Retrieve the value of an applet parameter
		public String getParameter (String key, String def);

		// If args were passed, override preferences
		public void setInitialProperties ();

		// Load specified controller class
		public void loadController(String classname);

		// Splash	
		public void showSplash ();
			
		// Required Applet Methods
	   // ------------------------------------------------------------------------
		// Initialize the applet
		public void init ();

		// Standard method to start the applet
		public void start();

		// Standard method to stop the applet
		public void stop();

		// Standard method to destroy the applet
		public void destroy();
		
		// Get info on the applet parameters
		public String[ ][ ] getParameterInfo ();

		// Get applet information
		public String getAppletInfo ();
		
		// Required Application Methods
	   // ------------------------------------------------------------------------
		// Main entry point when running standalone application
//		public static void main(String[] args);

		// Models
		// --------------------------------------------------------------------------
		public Vector models ();

		public Vector addModel (Model model);

		public Vector removeModel (Model model);

		public Model makeModel (boolean logp);
		
		public Vector addDefinition (Model model, Definition definition);

//		protected Vector listRelatedSources (Model model, Definition definition, Instance source, Vector visited);
		
//		protected void connectRelatedSources (Model model, Vector sourcePrototypePairs);
		
		public int save (Model model);

		public int save (Model model, boolean savedB4);
		
		public int save (Model model, String filename);	

		public Model load (String dir, String filename);
		
		public Model load (String filepath);


		// Definitions
		// ----------------------------------------------------------------------
		public Definition makeDefinition (Model model, Instance source);
		
//		protected Vector listRelatedPrototypes (Model model, Definition definition, Instance source, Vector visited);
		
		public Definition makeDefinition (Model model);
		
//		protected Vector listRelatedPrototypes (Model model, Definition definition);
		
		public Definition makeDefinition (Model model, Vector sources);
		
//		protected Vector listRelatedPrototypes (Model model, Definition definition, Vector sources);
		
//		protected void connectRelatedPrototypes (Model model, Vector sourcePrototypePairs);
		
		// Ontologies
		// ----------------------------------------------------------------------
		public void addToOntology (Model model, Ontology ontology, Definition definition);

		public void removeFromOntology (Model model, Ontology ontology, Definition definition);

		public Definition addToLibrary (Model model, Instance instance);

		public Definition addToLibrary (Model model, Ontology ontology, Instance instance);

		public Definition addToLibrary (Model model, Vector stuff);

		public Definition addToLibrary (Model model, Ontology ontology, Vector stuff);

		// Instances
		// ----------------------------------------------------------------------
		public Instance makeInstance (Instance prototype);
		
		public Instance makeInstance (Model model, Instance prototype);
		
		public void addInstance (Model model, Instance instance);
		
		public void addInstance (Model model, Vector instanceVector);
		
		public void removeInstance (Model model, Instance instance);
		
		public void removeInstance (Model model, Vector instanceVector);
		
		public Instance connect (Model model, Definition relationshipDefinition, Instance source, Instance target);
		
		public Instance connect (Model model, Instance relationship, Instance source, Instance target);
		
		public Instance connect (Instance relationship, Instance source, Instance target);
		
		public void rename (Model model, Instance instance, String name, boolean logp);
		
		// Simulation
		// ----------------------------------------------------------------------
		public void resetState (Model model);
			
		public boolean simulate (Model model);
		
		public boolean resolveState (Model model, Instance attribute);
		
		// not used?
		public void finalizeState ( Instance attribute, String state );

		public Vector orderAttributes (Model model);
		
		public Vector orderAttributes (Model model, Instance instance);
		
		public String getEffect (Model model, Instance instance);
		
		public boolean checkModel( Model model, long checks, boolean clear );
		
		public void updatePeers( Model model );
		
	} // interface

// STOP HERE
// =====================================================================================================================================

	// Properties and Accessors
   // ------------------------------------------------------------------------

	/** <PRE> 
	* Test to determine if this is an applet.
	* Since an instance of the applet class is always created,
	* assume true and negate in application main.
	* </PRE>
	* @serial */
	protected boolean isApplet = true;
	public boolean isApplet () { return(isApplet); }
	public boolean isApplet (boolean b) { return(isApplet = b); }

	// The controller controls the system.
	/** @serial */
	protected Class controllerClass = null;
	public Class controllerClass () { return(controllerClass); }
	public Class controllerClass (Class c) { return(controllerClass = c); }

	// The controller controls the system.
	protected static VModel.VMController controller = null;
	public static VModel.VMController controller () { return(controller); }
	public VModel.VMController controller (VModel.VMController c) { return(controller = c); }
	
	// Launchpad class
	protected static VModel applet = null;
	public static VModel applet () { return(applet); }
	public static VModel applet (VModel v) { return(applet = v); }

	// Preferences
	protected static VModelPreferences preferences = null;
	public static VModelPreferences preferences () { return(preferences); }
	public static VModelPreferences preferences (VModelPreferences p) { 
		return(preferences = p); 
	} // writer preferences

	public void updatePreferences () { }

	// Project
	/** @serial */
	protected static PROJECT project = null;
	public PROJECT project () { return(project); } 
	public PROJECT project (PROJECT p) { return(project = p); }

	// Variables to hold applet parameters
	/** @serial */
	public String[ ] args = null;
	
	/** @serial */
	protected Container VModelGUISplash = null;
	public Container VModelGUISplash() { return(VModelGUISplash); }
	public Container VModelGUISplash(Container c) { return(VModelGUISplash = c); }
	
	/** @serial */
	protected Timer splashTimer1 = null;
	/** @serial */
	protected Timer splashTimer2 = null;
	
	// BaseOntology
	/** @serial */
	protected Ontology ontology = null;	
	public Ontology ontology () {
		if (ontology == null) ontology = new BaseOntology();
		return(ontology);
	} // reader
	public Ontology ontology (Ontology o) { return(ontology = o); }
	
	// Libraries
	// -----------------------------------------------------------------------
	public Ontology makeOntology (String label, String documentation,
											 String helpURL) {
		Ontology result = new Ontology();
		result.name(label);
		result.helpURL(helpURL);
		result.documentation(documentation);
		return(result);
	} // method
	
	public Ontology makeCategory (String label, String documentation,
											 String helpURL) {
		Ontology result = new Ontology();
		result.name(label);
		result.helpURL(helpURL);
		result.documentation(documentation);
		return(result);
	} // method
	
	/** @serial */
	protected Ontology classModels = null;
	public Ontology classModels () {
		if (classModels == null) 
			classModels = makeOntology("Class Models",
				"Class models are ...",
				makeHelpURL("class-models"));
		return(classModels);
	} // reader
	public Ontology classModels (Ontology o) { return(classModels = o); }

	/** @serial */
	protected Ontology vModels = null;
	public Ontology vModels () {
		if (vModels == null) 
			vModels = makeOntology("VModels",
				"VModels are...",
				makeHelpURL("vmodels"));
		return(vModels);
	} // reader
	public Ontology vModels (Ontology o) { return(vModels = o); }

	/** @serial */
	protected Ontology myModels = null;
	public Ontology myModels () {
		if (myModels == null) 
			myModels = makeCategory("My Models",
				"My Models are...",
				makeHelpURL("my-models"));
		return(myModels);
	} // reader
	public Ontology myModels (Ontology o) { return(myModels = o); }

	/** @serial */
	protected Ontology myEntities = null;
	public Ontology myEntities () {
		if (myEntities == null) 
			myEntities = makeCategory("My Groups",
				"My Groups are...",
				makeHelpURL("my-entities"));
		return(myEntities);
	} // reader
	public Ontology myEntities (Ontology o) { return(myEntities = o); }

	/** @serial */
	protected Ontology myThings = null;
	public Ontology myThings () {
		if (myThings == null) 
			myThings = makeCategory("My Things",
				"My Things are...",
				makeHelpURL("my-things"));
		return(myThings);
	} // reader
	public Ontology myThings (Ontology o) { return(myThings = o); }

	/** @serial */
	protected Ontology myMultipleThings = null;
	public Ontology myMultipleThings () {
		if (myMultipleThings == null) 
			myMultipleThings = makeCategory("My Multiple Things",
				"My Multiple Things are...",
				makeHelpURL("my-multiplethings"));
		return(myMultipleThings);
	} // reader
	public Ontology myMultipleThings (Ontology o) { return(myMultipleThings = o); }

	/** @serial */
	protected Ontology mySubstances = null;
	public Ontology mySubstances () {
		if (mySubstances == null) 
			mySubstances = makeCategory("My Substances",
				"My Substances are...",
				makeHelpURL("my-substances"));
		return(mySubstances);
	} // reader
	public Ontology mySubstances (Ontology o) { return(mySubstances = o); }

	/** @serial */
	protected Ontology myProcesses = null;
	public Ontology myProcesses () {
		if (myProcesses == null) 
			myProcesses = makeCategory("My Processes",
				"My Processes are...",
				makeHelpURL("my-processes"));
		return(myProcesses);
	} // reader
	public Ontology myProcesses (Ontology o) { return(myProcesses = o); }

	/** @serial */
	protected Ontology myAttributes = null;
	public Ontology myAttributes () {
		if (myAttributes == null) 
			myAttributes = makeCategory("My Attributes",
				"My Attributes are...",
				makeHelpURL("my-attributes"));
		return(myAttributes);
	} // reader
	public Ontology myAttributes (Ontology o) { return(myAttributes = o); }

	/** @serial */
	protected Ontology projectOntology = null;
	public Ontology projectOntology () {
		if (projectOntology == null) {
			projectOntology = makeOntology("Project Library",
				"Project Library is ...",
				makeHelpURL("project-library"));
		} // end if
		return(projectOntology);
	} // reader
	public Ontology projectOntology (Ontology o) { return(projectOntology = o); }

	/** @serial */
	protected Ontology myLibrary = null;
	public Ontology myLibrary () {
		if (myLibrary == null) {
			myLibrary = makeOntology("My Library",
				"My Library is ...",
				makeHelpURL("my-library"));
			myLibrary.add(myModels());
			myLibrary.add(myProcesses());
			myLibrary.add(myEntities());
			myLibrary.add(myThings());
			myLibrary.add(myMultipleThings());
			myLibrary.add(mySubstances());
			myLibrary.add(myAttributes());
		} // end if
		return(myLibrary);
	} // reader
	public Ontology myLibrary (Ontology o) { return(myLibrary = o); }

	public Ontology library () { return(myLibrary()); }
	
	public boolean userIsDeveloper( ) {
		String user = preferences().getString("userId");
		if (user.equals("")) user = System.getProperty("user.name");
		return((!user.equals("")) && 
				(user.equalsIgnoreCase("ureel") || 
				 user.equalsIgnoreCase("kcarney") || 
				 user.equalsIgnoreCase("hinrichs") || 
				 user.equalsIgnoreCase("forbus") || 
				 user.equalsIgnoreCase("developer")));
	} // method
	
	// VModel.VMController methods
	// ------------------------------------------------------------------------
	public Definition findDefinition (String id) {
		Definition result = null;
		if (result == null) result = (Definition) myLibrary().findElement(id);
		if (result == null) result = (Definition) myProcesses().findElement(id);
		if (result == null) result = (Definition) myEntities().findElement(id);
		if (result == null) result = (Definition) myThings().findElement(id);
		if (result == null) result = (Definition) myMultipleThings().findElement(id);
		if (result == null) result = (Definition) mySubstances().findElement(id);
		if (result == null) result = (Definition) myAttributes().findElement(id);
		if (result == null) result = (Definition) myModels().findElement(id);
		if (result == null) result = (Definition) classModels().findElement(id);
		if (result == null) result = (Definition) vModels().findElement(id);
		if (result == null) result = (Definition) projectOntology().findElement(id);
		if (result == null) result = (Definition) ontology().findElement(id);
		if (result == null) QRGutils.debug("VModel", "findDefinition", "result=" + result + " id=" + id);
		return(result);
	} // method
	
	public Definition findDefinition (Model model, String id) {
		Definition result = model.findLocal(id);
		if (result == null) result = findDefinition(id);
		return(result);
	} // method
	
	public void go () { 
		QRGutils.debug("VModel.go", "VModel is called as default controller.");
		ontology();
		myLibrary();
		quit();
	} // method
	
	public boolean quit () {
		QRGutils.debug("VModel.quit", "Exiting VModel.");
//		QRGutils.debug("VModel.quit", "BaseOntology=" + QRGutils.eol() + ontology().toCML());
//		QRGutils.debug("VModel.quit", "ClassroomOntology=" + QRGutils.eol() + (new ClassroomOntology()).toCML());
		QRGutils.debug("VModel.quit", "VModelPreferences=" + QRGutils.eol() + preferences());
		if(! isApplet()) QRGutils.debug(false);
		System.exit(0);
		return(true);
	} // method
	
	public String makeHelpURL(String base) {
		return(QRGutils.QRGroot() + HELPPATH + base + "." + HTML_SUFFIX);
	} // method
	
	// Project File IO
	// --------------------------------------------------------------------------
	public PROJECT loadProject(String projectFileName) {
		PROJECT result = null;
		String projectFile = projectFileName;
//		Object temp = XMLio.load(QRGutils.QRGroot() + PROJECTPATH + projectFileName);
		Object temp = XMLio.load(projectFileName);
		if (temp instanceof PROJECT) {
			result = (PROJECT) temp;
		} else if (temp instanceof PREFERENCES) {
			result = new PROJECT(((PREFERENCES) temp).keyValuePairs(), ((PREFERENCES) temp).children());
		} else if (temp instanceof PList) {
			result = new PROJECT(((PList) temp).keyValuePairs(), null);
		} // end if else if
		if (result != null) {
			result.set("FULLFILEPATH", projectFile);
			project(result);
		} else {
			QRGutils.warn("VModel","Unable to load project file " + projectFile + ".");
		} // end if result else
		return(result);
	} // method load

	public void saveProject () {
		saveProject(project().getString("FULLFILEPATH"));
	} // method save

	public void saveProject (String filename) {
		if (project() != null) {
			if (filename != null) XMLio.save(filename, "", project());
		} // end if globalPreferences
	} // method saveGlobalPrefs
	
	// Methods
   // ------------------------------------------------------------------------
   // Attempt to automatically set the QRGroot path.
	public void setQRGroot (String inifile) {
		if (QRGutils.isApplet()) {
			URL baseURL = null;
			String baseString = "";
			try { 
				baseURL = new URL(getCodeBase().toString()); 
				baseString = baseURL.toString();
			} catch (Exception e) { baseString = getCodeBase().toString(); }
			QRGutils.QRGroot(baseString);
		} else {
			String userdir =  System.getProperty("user.dir");
			URL baseURL = getClass().getResource(inifile);
			String baseString = null;
			if (baseURL != null) {
				baseString = baseURL.getFile();
				if (baseString != null) baseString = (new File(baseString)).getParent() + "/";
			} else {
				File userdirpath = new File(userdir);
				File userinifile = new File(userdirpath, inifile);
				while((userinifile != null) && (!userinifile.exists())) {
					userdir = userdirpath.getParent(); 
					if (userdir != null) {
						userdirpath = new File(userdir);
						userinifile = new File(userdirpath, inifile);
					} else userinifile = null;
				} // end while
				if (userinifile != null) baseString = userinifile.getParent() + "/";
			} // end if codeURL else
			if (baseString != null) QRGutils.QRGroot(baseString);
			else QRGutils.QRGroot(VMODELROOT);
		} // end if is applet
	} // method setQRGroot
	
	// Retrieve the value of an applet parameter
	public String getParameter (String key, String def) {
		return (!isApplet) ? System.getProperty(key, def) :
			(getParameter(key) != null ? getParameter(key) : def);
	} // method getParameter

	// If args were passed, override preferences
	public void setInitialProperties () {
		if (QRGutils.isApplet()) {
			String[ ][ ] info = getParameterInfo();
			for (int i = 0; i < info.length; i++)
				preferences().set(info[ i ][ 0 ], getParameter(info[ i ][ 0 ]));
		} else if (args != null) {
			for (int i = 0; i < args.length; i = i + 2)
				preferences().set(args[ i ], args[ i + 1 ]);
		} // end if isApplet else
//		if (preferences.get("CONTROLLER") == null) preferences.set("CONTROLLER", "Classroom");
	} // method setProperties

public static boolean loadControllerDone = false;
	
	// Load specified controller class
	public void loadController(String classname) {
		try {
        controllerClass(Class.forName(classname));
        Class[] argSpecs = new Class[1];
        argSpecs[0] = getClass();
        Constructor constructor = controllerClass().getDeclaredConstructor(argSpecs);
        Object[] argObjs = new Object[1];
        argObjs[0] = applet();
        controller((VModel.VMController) constructor.newInstance(argObjs));
      } catch (ClassNotFoundException e) {
      	QRGutils.debug("VModel.loadController ClassNotFoundException classname=" + classname);
      	controller(this);
      } catch (NoSuchMethodException e2) {
      	QRGutils.debug("VModel.loadController NoSuchMethodException classname=" + classname);
      	controller(this);
      } catch (IllegalAccessException e3) {
      	QRGutils.debug("VModel.loadController IllegalAccessException classname=" + classname);
      	controller(this);
      } catch (InvocationTargetException e4) {
      	QRGutils.debug("VModel.loadController InvocationTargetException classname=" + classname);
      	controller(this);
      } catch (InstantiationException e5) {
		QRGutils.debug("VModel.loadController InstantiationException classname=" + classname);
		controller(this);
      } // try - catch
      if ((controller() == null) || (controller() == this)) {
      	controllerClass(getClass());
      	controller(this);
      } // end if
      loadControllerDone = true;
	} // method loadController

	// Splash	
	public void showSplash () { 
		int w = 302;
		int h = 202;
		String iconPath = QRGutils.QRGroot() + IMAGEPATH;
		ImageIcon image = null;
		JPanel pane = null;
		JLabel label = null;
		if (VModelGUISplash() == null) {
			if (QRGutils.isApplet()) {
				pane = new JPanel(new BorderLayout());
				pane.setBackground(SPLASH);
				image = QRGutils.getImageIcon(iconPath + "VModelSplash.gif");
				if (image == null) QRGutils.warn("Splash Screen", "File not Found: VmodelSplash.gif");
				label = new JLabel(image);
				VModelGUISplash(pane);
				pane.add(label, BorderLayout.CENTER);
				pane.setLocation(0, 0);
				pane.setSize(((JApplet) QRGutils.applet()).getContentPane().getSize());
				((JApplet) QRGutils.applet()).getLayeredPane().add(pane, 0);
			} else {
				VModelGUISplash(new JWindow());
				image = QRGutils.getImageIcon(iconPath + "VModelSplash.gif");
				if (image == null) QRGutils.warn("Splash Screen", "File not Found: VmodelSplash.gif");
				label = new JLabel(image);
				((JWindow) VModelGUISplash()).getContentPane().setLayout(new GridLayout(1,1));
				((JWindow) VModelGUISplash()).getContentPane().add(label);
				QRGutils.calcScreenSize();
				((JWindow) VModelGUISplash()).setLocation(
					((int)QRGutils.screenDimensions().getSize().width-w)/2, 
					((int)QRGutils.screenDimensions().getSize().height/2-h)/2);
				((JWindow) VModelGUISplash()).setSize(w, h);
			} // end if !isApplet
			label.setVisible(true);
		} // end if VModelGUISplash
		VModelGUISplash().setVisible(true);
		VModelGUISplash().validate();
		VModelGUISplash().repaint();

		splashTimer1 = new Timer(1500, new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				splashTimer1.stop();
				splashTimer2.stop();
				if ((!QRGutils.isApplet()) && (controller() != null) && (controllerClass() != null)) { 
					VModelGUISplash().setVisible(false);
					((JWindow) VModelGUISplash()).dispose();
					start();
				} else {
					splashTimer1.restart();
					splashTimer2.restart();
				} } }); // set new Timer
		splashTimer1.start();

		splashTimer2 = new Timer(30, new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				VModelGUISplash().setVisible(true);
			   if (!QRGutils.isApplet()) {
					((JWindow) VModelGUISplash()).toFront();
				} } }); // set new Timer
		splashTimer2.start();
	} // default constructor VModelGUISplash
	
	// Required Applet Methods
   // ------------------------------------------------------------------------
	// Initialize the applet
	public void init () {
		// Resolve problem with VM in early browsers.
		JRootPane rp = getRootPane();
 		rp.putClientProperty("defeatSystemEventQueueCheck", Boolean.TRUE);
 		
 		// identify self
 		applet(this);
 		controller(this);

		// initialize QRGutils		
		QRGutils.debug(TESTRUN, "VModel.dbg", 500000);
		QRGutils.debug("VModel.init", "SYSTEM INFO", getAppletInfo());
		setQRGroot(INIFILE);
		QRGutils.initUserPath();
		QRGutils.QRGuserpath(QRGutils.makeUserPath("VModel")); 
		
		// Splash Screen
		showSplash();
		
		// Preferences
		preferences(new VModelPreferences(this));
		preferences().loadGlobalPrefs(INIFILE);
		
		// assign initial preferences
		setInitialProperties();
		
		// load default project
		loadProject(QRGutils.QRGroot() + PROJECTPATH + "default.vmp");

		// load and instantiate controller class
		loadController(preferences.getString("CONTROLLER"));
		
		System.gc();
	} // method init

	// Standard method to start the applet
	public void start() {
		while( !loadControllerDone ) {
			try { Thread.sleep(500); }
			catch(Exception e) { QRGutils.debug( "Classroom.getEffect", "not sleeping"); QRGutils.debugStackTrace( e ); }
		} // end wait
		super.start();
		try {
			QRGutils.debug("VModel.start", "controllerClass=" + QRGutils.rawClassName(controllerClass().getName()));
			Method method = null;
			if (controllerClass() != null) method = controllerClass().getMethod("go", null);
			QRGutils.debug("VModel.start", "method=" + method + QRGutils.eol() + QRGutils.eol());
			if (method != null) method.invoke(controller(), null);
		} catch (NoSuchMethodException e1) {
			QRGutils.debugStackTrace(e1);
			System.exit(-1);
		} catch (IllegalAccessException e2) {
			QRGutils.debugStackTrace(e2);
			System.exit(-2);
		} catch (InvocationTargetException e3) {
			QRGutils.debugStackTrace(e3);
			System.exit(-3);
		} catch (java.lang.IllegalArgumentException e4) {
			QRGutils.debugStackTrace(e4);
			System.exit(-4);
		} catch (Exception e5) {
			QRGutils.debug("VModel.start", "Trapped unanticipated exception e5=" + e5 );
			QRGutils.debugStackTrace(e5);
			System.exit(-5);
		} // try - catch
		System.gc();
	} // method start

	// Standard method to stop the applet
	public void stop() {
		super.stop();
	} // method stop

	// Standard method to destroy the applet
	public void destroy() {
		super.destroy();
	} // method destroy
	
	// Get info on the applet parameters
	public String[ ][ ] getParameterInfo () {
		String info[ ][ ] = {
			{ "CONTROLLER", "String", "Main MVC Controller." },
			{ "OMNIFRAME", "String", "Contain all windows in a single frame." },
			{ "INSIDE_BROWSER", "String", "View applet inside browser." },
			{ "VIEW", "String", "Main MVC View." }
		};
		return(info);
	} // method getParameterInfo

	// Get applet information
	public String appletInfo = null;
	public String getAppletInfo () {
		if ( appletInfo == null ) {
			String result = "";
			QRGutils.projectName(VMODEL_PROJECT_NAME);
			QRGutils.projectVersion(VMODEL_PROJECT_VERSION);
			for( int i = 0; i < VMODEL_PROJECT_AUTHORS.length; i++ ) 
				QRGutils.projectAuthors().addElement(VMODEL_PROJECT_AUTHORS[i]);
			result += QRGutils.eol() + QRGutils.eol() + QRGutils.projectName();
			result += " v" + QRGutils.projectVersion();
			result += QRGutils.eol() + "Copyright " + QRGutils.projectCopyright();
			result += " " + QRGutils.projectCopyrightAttribution();
			result += QRGutils.eol() + QRGutils.eol() + "Authors:";
			for(Enumeration authors = QRGutils.projectAuthors().elements(); authors.hasMoreElements();)
				result += QRGutils.eol() + authors.nextElement();
			result += QRGutils.eol() + QRGutils.eol() + "java version = " + System.getProperty( "java.version");
			result += QRGutils.eol() + "java vendor  = " + System.getProperty( "java.vendor");
			result += QRGutils.eol() + "java home    = " + System.getProperty( "java.home");
			result += QRGutils.eol() + QRGutils.eol() + "os name    = " + System.getProperty( "os.name");
			result += QRGutils.eol() + "os arch    = " + System.getProperty( "os.arch");
			result += QRGutils.eol() + "os version = " + System.getProperty( "os.version");
			result += QRGutils.eol() + QRGutils.eol() + "file separator = '" + System.getProperty( "file.separator") + "'";
			result += QRGutils.eol() + QRGutils.eol() + "user name = " + System.getProperty( "user.name");
			result += QRGutils.eol() + "user home = " + System.getProperty( "user.home");
			result += QRGutils.eol() + "user dir  = " + System.getProperty( "user.dir");
			// Added (TRH)
			Runtime rt = Runtime.getRuntime();
			result += QRGutils.eol() + QRGutils.eol() + "Total memory (heap) = " + rt.totalMemory();
			result += QRGutils.eol() + "Available memory    = " + rt.freeMemory();
			result += QRGutils.eol() + QRGutils.eol();
			appletInfo = result;
		} // end if
		return(appletInfo);
	} // getAppletInfo
	
	// Required Application Methods
   // ------------------------------------------------------------------------
	// Main entry point when running standalone application
	public static void main(String[] args) {
		VModel instance = new VModel();
		instance.args = args;
		instance.isApplet = false;
		instance.init();
	} // method main


	// Models
	// --------------------------------------------------------------------------
	protected Vector models = null;
	public Vector models () {
		if (models == null) models = new Vector();
		return(models);
	} // reader

	public Vector addModel (Model model) {
		models().addElement(model);
		return(models);
	} // method

	public Vector removeModel (Model model) {
		models().removeElement(model);
		return(models);
	} // method

	protected static int newModelCount = 0;
	public Model makeModel (boolean logp) {
		Model result = new Model();
		result.name("Untitled Model " + (++newModelCount));
		if ((result != null) && logp) result.log("VModel.makeModel", "Model Created id=" + result.id() + " name=" + result.name());
		addModel(result);
		return(result);
	} // method
	
	public Vector addDefinition (Model model, Definition definition) {
		Vector result = new Vector();
		Vector rest = new Vector();
		Instance prototype = null;
		Instance instance = null;
		Vector pair = null;
		Vector pairs = new Vector();
		for(Enumeration kids = definition.children(); kids.hasMoreElements();) {
			prototype = (Instance) kids.nextElement();
			instance = makeInstance(model, prototype);
			instance.removeKey(CONNECTIONS);
			instance.removeKey(CONNECTORS);
			instance.removeKey(CHILDINSTANCES);
			instance.removeKey(PARENTINSTANCES);
			instance.removeKey(INPUTS);
			instance.removeKey(OUTPUTS);
			instance.removeKey(CAUSALINPUTS);
			instance.removeKey(CAUSALOUTPUTS);
			pair = new Vector();
			pair.addElement(prototype);
			pair.addElement(instance);
			pairs.addElement(pair);
			if (prototype.id().equalsIgnoreCase(definition.id())) result.addElement(instance);
			else rest.addElement(instance);
		} // end for
		connectRelatedPrototypes(model, pairs);
		result = QRGutils.mergeVector(result, rest);
		return(result);
	} // method

	protected Vector listRelatedSources (Model model, Definition definition, Instance source, Vector visited) {
		Vector result = new Vector();
		Vector pair = new Vector();
		Instance newPrototype = makeInstance(model, source);
		Instance instance = null;
		if ( visited == null ) visited = new Vector( );
		newPrototype.removeKey(CONNECTIONS);
		newPrototype.removeKey(CONNECTORS);
		newPrototype.removeKey(CHILDINSTANCES);
		newPrototype.removeKey(PARENTINSTANCES);
		newPrototype.removeKey(INPUTS);
		newPrototype.removeKey(OUTPUTS);
		newPrototype.removeKey(CAUSALINPUTS);
		newPrototype.removeKey(CAUSALOUTPUTS);
		pair.addElement(source);
		pair.addElement(newPrototype);
		result.addElement(pair);
		for(Enumeration kids = source.children(); kids.hasMoreElements();) {
			instance = (Instance) kids.nextElement();
			if ( !visited.contains(instance) ) {
				visited.addElement(instance);
				result = QRGutils.mergeVector(result, listRelatedSources(model, definition, instance, visited));
			} // end if
		} // end for
		return(result);
	} // method
	
	protected void connectRelatedSources (Model model, Vector sourcePrototypePairs) {
		Vector pair = null;
		Vector pair2 = null;
		Instance source = null;
		Instance prototype = null;
		Instance mumble = null;
		Instance frotz = null;
		Instance input = null;
		Instance output = null;
		for(Enumeration pairs = sourcePrototypePairs.elements(); pairs.hasMoreElements();) {
			pair = (Vector) pairs.nextElement();
			source = (Instance) pair.elementAt(0);
			prototype = (Instance) pair.elementAt(1);
			if (prototype.typep(STRUCTURE)) {
				input = null;
				output = null;
				for(Enumeration pairs2 = sourcePrototypePairs.elements(); pairs2.hasMoreElements();) {
					pair2 = (Vector) pairs2.nextElement();
					mumble = (Instance) pair2.elementAt(0);
					frotz = (Instance) pair2.elementAt(1);
					if (source.input().equalsIgnoreCase(mumble.id())) {
//						prototype.input(frotz.id());
						input = frotz;
					} // end if
					if (source.output().equalsIgnoreCase(mumble.id())) {
//						prototype.output(frotz.id());
						output = frotz;
					} // end if
				} // end for
				connect(model, prototype, input, output);
			} // end if
		} // end for
		for(Enumeration pairs = sourcePrototypePairs.elements(); pairs.hasMoreElements();) {
			pair = (Vector) pairs.nextElement();
			source = (Instance) pair.elementAt(0);
			prototype = (Instance) pair.elementAt(1);
			if (prototype.typep(CONSTRAINT)) {
				input = null;
				output = null;
				for(Enumeration pairs2 = sourcePrototypePairs.elements(); pairs2.hasMoreElements();) {
					pair2 = (Vector) pairs2.nextElement();
					mumble = (Instance) pair2.elementAt(0);
					frotz = (Instance) pair2.elementAt(1);
					if (source.input().equalsIgnoreCase(mumble.id())) {
//						prototype.input(frotz.id());
						input = frotz;
					} // end if
					if (source.output().equalsIgnoreCase(mumble.id())) {
//						prototype.output(frotz.id());
						output = frotz;
					} // end if
				} // end for
				connect(model, prototype, input, output);
			} // end if
		} // end for
		for(Enumeration pairs = sourcePrototypePairs.elements(); pairs.hasMoreElements();) {
			pair = (Vector) pairs.nextElement();
			source = (Instance) pair.elementAt(0);
			prototype = (Instance) pair.elementAt(1);
			if (prototype.typep(CAUSES)) {
				input = null;
				output = null;
				for(Enumeration pairs2 = sourcePrototypePairs.elements(); pairs2.hasMoreElements();) {
					pair2 = (Vector) pairs2.nextElement();
					mumble = (Instance) pair2.elementAt(0);
					frotz = (Instance) pair2.elementAt(1);
					if (source.input().equalsIgnoreCase(mumble.id())) {
//						prototype.input(frotz.id());
						input = frotz;
					} // end if
					if (source.output().equalsIgnoreCase(mumble.id())) {
//						prototype.output(frotz.id());
						output = frotz;
					} // end if
				} // end for
				connect(model, prototype, input, output);
			} // end if
		} // end for
	} // method
	
	public int save (Model model) {
    	return(save(model, model.getBoolean("SAVEDONCE")));
    } // method save

	public int save (Model model, boolean savedB4) {
		String filename = model.getString("FILEPATH");
		if (! savedB4) filename = QRGutils.QRGuserpath() + model.name();
		return(save(model, filename));
	} // method save
	
	public int save (Model model, String filename) {
		int result = 0;
		model.save(filename);
		return(result);
	} // method save	

	public Model load (String dir, String filename) {
		Model result = null;
		result = load(dir + filename);
		return(result);
	} // method loadModel
	
	public Model load (String filepath) {
		Model result = null;
		result = (Model) Model.load(filepath);
		if (result != null){
			 result.set("SAVEDONCE", true);
		} // end if result
		return(result);
	} // method loadModel


	// Definitions
	// ----------------------------------------------------------------------
	public Definition makeDefinition (Model model, Instance source) {
		Definition result = new Definition();
		Instance newPrototype = null;
		Vector prototypePairs = listRelatedPrototypes(model, result, source, null);
		connectRelatedPrototypes(model, prototypePairs);
		newPrototype = (Instance) ((Vector) prototypePairs.elementAt(0)).elementAt(1);
		// assume all definitions share unique identifier with primary prototype.
		result.plist(result.copyKeyValuePairs(newPrototype).plist());
		result.connections(new Vector()); 
		result.connectors(new Vector()); 
		result.removeKey(CHILDINSTANCES);
		result.removeKey(PARENTINSTANCES);
		result.removeKey(INPUTS);
		result.removeKey(OUTPUTS);
		for(Enumeration pairs = prototypePairs.elements(); pairs.hasMoreElements();) {
			Instance foo = (Instance) ((Vector) pairs.nextElement()).elementAt(1);
			result.add(foo);
		} // end for
		return(result);
	} // method
	
	protected Vector listRelatedPrototypes (Model model, Definition definition, Instance source, Vector visited) {
		Vector result = new Vector();
		Vector pair = new Vector();
		Instance newPrototype = makeInstance(source);
		Instance instance = null;
		if ( visited == null ) visited = new Vector( );
		newPrototype.removeKey(CONNECTIONS);
		newPrototype.removeKey(CONNECTORS);
		newPrototype.removeKey(CHILDINSTANCES);
		newPrototype.removeKey(PARENTINSTANCES);
		newPrototype.removeKey(INPUTS);
		newPrototype.removeKey(OUTPUTS);
		newPrototype.removeKey(CAUSALINPUTS);
		newPrototype.removeKey(CAUSALOUTPUTS);
		pair.addElement(source);
		pair.addElement(newPrototype);
		result.addElement(pair);
		for(Enumeration kids = source.children(); kids.hasMoreElements();) {
			instance = (Instance) kids.nextElement();
			if (!visited.contains(instance)) {
				visited.addElement( instance );
				result = QRGutils.mergeVector(result, listRelatedPrototypes(model, definition, instance, visited));
			} // end if
		} // end for
		return(result);
	} // method
	
	public Definition makeDefinition (Model model) {
		Definition result = new Definition();
		Instance newPrototype = null;
		Vector prototypePairs = listRelatedPrototypes(model, result);
		connectRelatedPrototypes(model, prototypePairs);
		// assume all definitions share unique identifier with primary prototype.
		result.plist(result.copyKeyValuePairs(model).plist());
		result.connections(new Vector()); 
		result.connectors(new Vector()); 
		result.removeKey(CHILDINSTANCES);
		result.removeKey(PARENTINSTANCES);
		result.removeKey(INPUTS);
		result.removeKey(OUTPUTS);
		result.removeKey(CAUSALINPUTS);
		result.removeKey(CAUSALOUTPUTS);
		for(Enumeration pairs = prototypePairs.elements(); pairs.hasMoreElements();) {
			Instance foo = (Instance) ((Vector) pairs.nextElement()).elementAt(1);
			result.add(foo);
		} // end for
		return(result);
	} // method
	
	protected Vector listRelatedPrototypes (Model model, Definition definition) {
		Vector result = new Vector();
		Vector pair = null;
		Vector visited = new Vector( );
		Instance source = null;
		Instance newPrototype = null;
		Instance instance = null;
		for( Enumeration instances = model.instances( ).elements(); instances.hasMoreElements(); ) {
			source = (Instance) instances.nextElement();
			newPrototype = makeInstance(source);
			newPrototype.removeKey(CONNECTIONS);
			newPrototype.removeKey(CONNECTORS);
			newPrototype.removeKey(CHILDINSTANCES);
			newPrototype.removeKey(PARENTINSTANCES);
			newPrototype.removeKey(INPUTS);
			newPrototype.removeKey(OUTPUTS);
			newPrototype.removeKey(CAUSALINPUTS);
			newPrototype.removeKey(CAUSALOUTPUTS);
			pair = new Vector( );
			pair.addElement(source);
			pair.addElement(newPrototype);
			result.addElement(pair);
		} // end for
		return(result);
	} // method
	
	public Definition makeDefinition (Model model, Vector sources) {
		Definition result = new Definition();
		Instance newPrototype = null;
		Vector prototypePairs = listRelatedPrototypes(model, result, sources);
		connectRelatedPrototypes(model, prototypePairs);
		for(Enumeration pairs = prototypePairs.elements(); pairs.hasMoreElements();) {
			Instance foo = (Instance) ((Vector) pairs.nextElement()).elementAt(1);
			result.add(foo);
		} // end for
		return(result);
	} // method
	
	protected Vector listRelatedPrototypes (Model model, Definition definition, Vector sources) {
		Vector result = new Vector();
		Vector pair = null;
		Vector visited = new Vector( );
		Instance source = null;
		Instance newPrototype = null;
		Instance instance = null;
		for( Enumeration instances = sources.elements(); instances.hasMoreElements(); ) {
			source = (Instance) instances.nextElement();
			newPrototype = makeInstance(source);
			newPrototype.removeKey(CONNECTIONS);
			newPrototype.removeKey(CONNECTORS);
			newPrototype.removeKey(CHILDINSTANCES);
			newPrototype.removeKey(PARENTINSTANCES);
			newPrototype.removeKey(INPUTS);
			newPrototype.removeKey(OUTPUTS);
			newPrototype.removeKey(CAUSALINPUTS);
			newPrototype.removeKey(CAUSALOUTPUTS);
			pair = new Vector();
			pair.addElement(source);
			pair.addElement(newPrototype);
			result.addElement(pair);
		} // end for
		return(result);
	} // method
	
	protected void connectRelatedPrototypes (Model model, Vector sourcePrototypePairs) {
		Vector pair = null;
		Vector pair2 = null;
		Instance source = null;
		Instance prototype = null;
		Instance mumble = null;
		Instance frotz = null;
		Instance input = null;
		Instance output = null;
		for(Enumeration pairs = sourcePrototypePairs.elements(); pairs.hasMoreElements();) {
			pair = (Vector) pairs.nextElement();
			source = (Instance) pair.elementAt(0);
			prototype = (Instance) pair.elementAt(1);
			if (prototype.typep(HASATTRIBUTE) || prototype.typep(HASPARTICIPANT) || prototype.typep(DOES)) {
// QRGutils.debug("VModel.connectRelatedPrototypes", "1. prototype=" + prototype.name());
				input = null;
				output = null;
				for(Enumeration pairs2 = sourcePrototypePairs.elements(); pairs2.hasMoreElements();) {
					pair2 = (Vector) pairs2.nextElement();
					mumble = (Instance) pair2.elementAt(0);
					frotz = (Instance) pair2.elementAt(1);
					if (source.input().equalsIgnoreCase(mumble.id())) {
						input = frotz;
					} // end if
					if (source.output().equalsIgnoreCase(mumble.id())) {
						output = frotz;
					} // end if
					if ((input != null) && (output != null)) break;
				} // end for
//				controller().connect( model, prototype, input, output);
				controller().connect( prototype, input, output);
			} // end if
		} // end for
		for(Enumeration pairs = sourcePrototypePairs.elements(); pairs.hasMoreElements();) {
			pair = (Vector) pairs.nextElement();
			source = (Instance) pair.elementAt(0);
			prototype = (Instance) pair.elementAt(1);
			if (prototype.typep(RELATIONSHIP) && (!(prototype.typep(HASATTRIBUTE) || prototype.typep(HASPARTICIPANT) || prototype.typep(DOES)))) {
// QRGutils.debug("VModel.connectRelatedPrototypes", "2. prototype=" + prototype.name());
				input = null;
				output = null;
				for(Enumeration pairs2 = sourcePrototypePairs.elements(); pairs2.hasMoreElements();) {
					pair2 = (Vector) pairs2.nextElement();
					mumble = (Instance) pair2.elementAt(0);
					frotz = (Instance) pair2.elementAt(1);
					if (source.input().equalsIgnoreCase(mumble.id())) {
						input = frotz;
					} // end if
					if (source.output().equalsIgnoreCase(mumble.id())) {
						output = frotz;
					} // end if
					if ((input != null) && (output != null)) break;
				} // end for
//				controller().connect( model, prototype, input, output);
				controller().connect( prototype, input, output);
			} // end if
		} // end for
	} // method
	
	// Ontologies
	// ----------------------------------------------------------------------
	public void addToOntology (Model model, Ontology ontology, Definition definition) {
		ontology.add(definition);
	} // method

	public void removeFromOntology (Model model, Ontology ontology, Definition definition) {
		ontology.remove(definition);
	} // method

	public Definition addToLibrary (Model model, Instance instance) {
		Ontology ontology = myLibrary();
		if (instance.typep(PROCESS)) ontology = myProcesses();
		else if (instance.typep("SUBSTANCE")) ontology = mySubstances();
		else if (instance.typep("MULTIPLETHING")) ontology = myMultipleThings();
		else if (instance.typep("THING")) ontology = myThings();
		else if (instance.typep(ENTITY)) ontology = myEntities();
		else if (instance.typep(MODEL)) ontology = myModels();
		else if (instance.typep(ATTRIBUTE)) ontology = myAttributes();
		return(addToLibrary(model, ontology, instance));
	} // method

	public Definition addToLibrary (Model model, Ontology ontology, Instance instance) {
		Definition result = makeDefinition(model, instance);
		ontology.add(result);
		model.log("ADD-TO-LIBRARY", "ontology=" + ontology.id() + ":" + ontology.name() + " instance=" + instance.id() + ":" + instance.name());
		return(result);
	} // method

	public Definition addToLibrary (Model model, Vector stuff) {
		Ontology ontology = myEntities();
		return(addToLibrary(model, ontology, stuff));
	} // method

	public Definition addToLibrary (Model model, Ontology ontology, Vector stuff) {
		Definition result = makeDefinition(model, stuff);
		ontology.add(result);
		model.log("ADD-TO-LIBRARY", "ontology=" + ontology.id() + ":" + ontology.name() + " stuff=" + stuff);
		return(result);
	} // method

	// Instances
	// ----------------------------------------------------------------------
	public Instance makeInstance (Instance prototype) {
		Instance result = new Instance(prototype);
		return(result);
	} // method
	
	public Instance makeInstance (Model model, Instance prototype) {
		Instance result = new Instance(model, prototype);
		addInstance(model, result);
		return(result);
	} // method
	
	public void addInstance (Model model, Instance instance) {
		model.add(instance);
		instance.model(model);
	} // method
	
	public void addInstance (Model model, Vector instanceVector) {
		Instance instance = null;
		for(Enumeration instances = instanceVector.elements(); instances.hasMoreElements();) {
			instance = (Instance) instances.nextElement();
			addInstance(model, instance);
		} // end for
	} // method
	
	public void removeInstance (Model model, Instance instance) {
		Instance mumble = null;
		Instance frotz = null;
		Vector structure = new Vector(); // 20030131 Leo
		if (instance.typep(RELATIONSHIP)) {
			Instance input = model.findInstance(instance.input());
			Instance output = model.findInstance(instance.output());
			connect(model, instance, null, null);
			if (input != null) input.updatePeer();
			if (output != null) output.updatePeer();
		} // end if
		for(Enumeration inputs = instance.inputs().elements(); inputs.hasMoreElements();) {
			mumble = model.findInstance((String) inputs.nextElement());
			if (mumble != null) {
				if (mumble.typep(STRUCTURE)) structure.addElement(mumble); // 20030131 Leo
				frotz = model.findInstance(mumble.input());
				connect(model, mumble, frotz, null);
				mumble.updatePeer();
			} // end if
		} // end for
		for(Enumeration inputs = structure.elements(); inputs.hasMoreElements();) { // 20030131 Leo
			mumble = (Instance) inputs.nextElement();
			if (mumble != instance) removeInstance(model, mumble);
		} // end for
		for(Enumeration outputs = instance.outputs().elements(); outputs.hasMoreElements();) {
			mumble = model.findInstance((String) outputs.nextElement());
			if (mumble != null) {
				frotz = model.findInstance(mumble.output());
				connect(model, mumble, null, frotz);
				mumble.updatePeer();
			} // end if
		} // end for
		model.remove(instance);
		instance.updatePeer();
	} // method
	
	public void removeInstance (Model model, Vector instanceVector) {
		Instance instance = null;
		for(Enumeration instances = instanceVector.elements(); instances.hasMoreElements();) {
			instance = (Instance) instances.nextElement();
			removeInstance(model, instance);
		} // end for
	} // method
	
	public Instance connect (Model model, Definition relationshipDefinition, Instance source, Instance target) {
		Instance result = (Instance) addDefinition(model, relationshipDefinition).elementAt(0);
		connect(model, result, source, target);
		return(result);
	} // method
	
	public Instance connect (Model model, Instance relationship, Instance source, Instance target) {
		Instance mumble = null;
		// 1. disconnect step
		if ((relationship.input() != null) && (! relationship.input().equalsIgnoreCase(""))) {
			mumble = model.findInstance(relationship.input());
			if (mumble != null) {
				if (relationship.typep(STRUCTURE)) {
					mumble.connections().removeElement(relationship.id());
					mumble.connectors().removeElement(relationship.output());
					mumble.removeChild(relationship);
					mumble.removeChild(model.findInstance(relationship.output()));
				} // end if
				mumble.outputs().removeElement(relationship.id());
				mumble.causalOutputs().removeElement(relationship);
				relationship.input("");
			} else QRGutils.debug("VModel.connect", "Could not find input during disconnect. input=" + relationship.input());
		} // end if
		if ((relationship.output() != null) && (! relationship.output().equalsIgnoreCase(""))) {
			mumble = model.findInstance(relationship.output());
			if (mumble != null) {
				mumble.inputs().removeElement(relationship.id());
				mumble.causalInputs().removeElement(relationship.id());
				relationship.output("");
			} else QRGutils.debug("VModel.connect", "Could not find output during disconnect. output=" + relationship.output());
		} // end if
		// 2. connect step
		connect(relationship, source, target);
		return(relationship);
	} // method
	
	public Instance connect (Instance relationship, Instance source, Instance target) {
		if (source != null) {
			source.outputs().addElement(relationship.id());
			if (relationship.typep(CAUSES)) {
				source.causalOutputs().removeElement(relationship);
				source.causalOutputs().addElement(relationship);
			} // end if
			relationship.input(source.id());
		} // end if
		if (target != null) {
			target.inputs().addElement(relationship.id());
			if (relationship.typep(CAUSES)) target.causalInputs().addElement(relationship.id());
			relationship.output(target.id());
		} // end if
		if ((relationship.typep(STRUCTURE) || relationship.typep(REQUIRES)) && (source != null)) {
			source.connections().removeElement(relationship.id());
			source.connections().addElement(relationship.id());
			source.addChild(relationship);
			if (target != null) source.addChild(target);
			if ((target != null) && target.typep(RELATIONSHIP)) {
				source.connections().removeElement(target.id());
				source.connections().addElement(target.id());
			} else {
				if (target != null) source.connectors().removeElement(target.id());
				if (target != null) source.connectors().addElement(target.id());
			} // end if
		} // end if
		if (source == null) relationship.input("");
		if (target == null) relationship.output("");
		return(relationship);
	} // method
	
	public void rename (Model model, Instance instance, String name, boolean logp) {
		String oldName = instance.id();
		instance.name(name);
		if (logp) model.log("RENAME-INSTANCE", "id=" + instance.id() + " old=" + oldName + " new=" + name);
	} // method
	
	// Simulation
	// ----------------------------------------------------------------------
	public void resetState (Model model) {
		Instance instance = null;
		for(Enumeration instances = model.instances().elements(); instances.hasMoreElements();) {
			instance = (Instance) instances.nextElement();
			if (instance.typep(ATTRIBUTE) || instance.typep(PROCESSRATE)) {
				if (instance.causalInputs().isEmpty() && (instance.initialReason() != null) && (!instance.initialReason().equals(USERASSUMPTION))) instance.state(CONSTANT); 
				else instance.state(instance.initialState());
				instance.reasons(new Vector()).addElement(instance.initialReason());
				instance.set(CAUSALCOUNT, 0);
			} else if (instance.typep(CAUSES)) {
				instance.state(instance.initialState());
				instance.reasons(new Vector()).addElement(instance.initialReason());
			} // end if
		} // end for
	} // method
		
	public boolean simulate (Model model) {
		boolean result = true;
		Vector quantities = new Vector();
		Instance instance = null;
		Instance candidate = null;
		Vector directInfluences = new Vector();
		Vector indirectInfluences = new Vector();
		Vector directlyInfluenced = new Vector();
		Vector indirectlyInfluenced = new Vector();
		Vector attributes = new Vector();
		Vector mumble = new Vector();
		resetState(model);
		quantities = orderAttributes( model );
		for(Enumeration instances = quantities.elements(); instances.hasMoreElements();) {
			instance = (Instance) instances.nextElement();
			if (instance.typep(ATTRIBUTE) || instance.typep(PROCESSRATE)) result = result && resolveState(model, instance); 
			else QRGutils.debug("VModel.simulate", "Unexpected instance=" + instance);
			if (! result) break; // contradiction
		} // end for
// QRGutils.debug("VModel.simulate", "result=" + result);		
		return(result);
	} // method
	
	public boolean resolveState (Model model, Instance attribute) {
		boolean result = true;
		Vector dominant = new Vector();
		Vector normal = new Vector();
		Vector negligable = new Vector();
		Vector causes = null;
		Instance candidate = null;
		Instance source = null;
		String mumble = "";
		String state = attribute.state();
		String effect = "";
		Stack reasons = new Stack();
		Vector reason = new Vector();
		boolean contradiction = false;
		// 1. Sort step.
		for(Enumeration enum = attribute.causalInputs().elements(); enum.hasMoreElements();) {
			mumble = (String) enum.nextElement();
			candidate = model.findInstance(mumble);
			if (candidate.state().equalsIgnoreCase(DOMINANT)) dominant.addElement(candidate);
			else if (candidate.state().equalsIgnoreCase(NEGLIGABLE)) negligable.addElement(candidate);
			else normal.addElement(candidate);
		} // end for
		if (dominant.size() > 0) causes = dominant;
		else causes = normal;
		for(Enumeration enum = causes.elements(); enum.hasMoreElements();) {
			candidate = (Instance) enum.nextElement();
			source = model.findInstance(candidate.input());
			if (source != null) {
//if (source.causalInputs().isEmpty() && (source.reasons() != null) && (!source.reasons().isEmpty()) && ( ! ((String)source.reasons().elementAt(0)).equals(USERASSUMPTION))) {
//								effect = state;
//} else {
								effect = getEffect(model, candidate);
//} // end if else
				reason = new Vector();
				reason.addElement(effect);
				reason.addElement(candidate.id());
				reason.addElement(candidate.name());
				reason.addElement(source.id());
				reason.addElement(source.name());
				reason.addElement(source.reasons());
				reasons.push(reason);
			attribute.set(CAUSALCOUNT2, attribute.getInt(CAUSALCOUNT2) + 1);
				if (state.equalsIgnoreCase(DEFAULT) || state.equalsIgnoreCase(CONSTANT)) {
					state = effect;
				} else if ((! state.equalsIgnoreCase(effect)) && (!state.equalsIgnoreCase(CONSTANT)) && (effect != CONSTANT)) {
					if (! ((String)attribute.reasons().elementAt(0)).equals(USERASSUMPTION)) {
						result = false;
						state = CONTRADICTION;
						contradiction = true;
						attribute.problems("Error #20", "This " + attribute.name() + " as ambiguous inputs. Please assume a value or weight the input relationships. (Right-click.)");
						break;
					} // end if
				} // end if else
			if (attribute.getInt(CAUSALCOUNT2) == attribute.getInt(CAUSALCOUNT)) {
				finalizeState( attribute, state );
			} // end if
			}else QRGutils.debug("VModel.resolveState", "Source not found. candidate=" + candidate + " candidate.input=" + candidate.input());
		} // end for
		if (! ((String)attribute.reasons().elementAt(0)).equals(USERASSUMPTION)) {
			attribute.state(state);
			attribute.reasons(reasons);
		} // end if
		if ((!attribute.predictedState().equalsIgnoreCase(UNKNOWN)) && (!attribute.predictedState().equalsIgnoreCase(attribute.state()))) attribute.problems("Error #21", "You predicted that " + attribute.name() + " would be " + attribute.predictedState() + " but instead it is " + attribute.state());
		attribute.set("CONTRADICTION", contradiction);
		attribute.updatePeer();
		return(result);
	} // method
	
	public void finalizeState ( Instance attribute, String state ) {
	} // method

	public Vector orderAttributes (Model model) {
		Vector result = new Vector( );
		Instance instance = null;
		for(Enumeration instances = model.instances().elements(); instances.hasMoreElements();) {
			instance = (Instance) instances.nextElement();
			if (instance.typep(DIRECTINFLUENCE)) {
				result = QRGutils.mergeVector(result, orderAttributes(model, instance));
			} // end if
		} // end for
		for(Enumeration instances = model.instances().elements(); instances.hasMoreElements();) {
			instance = (Instance) instances.nextElement();
			if ((instance.typep(QUANTITY)) && (!result.contains(instance)) && (instance.causalInputs().isEmpty())) {
// QRGutils.debug("VModel.orderAttributes","instance=" + instance + " name=" + instance.name());
				result = QRGutils.mergeVector(result, orderAttributes(model, instance));
			} // end if
		} // end for
		return(result);
	} // method
	
	public Vector orderAttributes (Model model, Instance instance) {
		Vector result = new Vector();
		Instance candidate = null;
		if ( instance.typep(QUANTITY ) ) {
//			instance.set(CAUSALCOUNT, (instance.getInt(CAUSALCOUNT) + 1));
//			if (instance.getInt(CAUSALCOUNT) == instance.causalInputs().size()) {
				result.addElement(instance);
				for(Enumeration enum = instance.causalOutputs().elements(); enum.hasMoreElements();) {
					candidate = (Instance) enum.nextElement();
					if (candidate != null) {
						result = QRGutils.mergeVector(result, orderAttributes(model, candidate));
					} // end if
				} // end for
//			} // end if
		} else {
			if (instance.typep(REQUIRES)) return(result);
			Instance input = (Instance) model.findInstance(instance.input());
			Instance output = (Instance) model.findInstance(instance.output());
			String mumble = "";
			if (input != null) {
				if (output != null) {
					output.set(CAUSALCOUNT, (output.getInt(CAUSALCOUNT) + 1));
					output.set(CAUSALCOUNT2, 0);
					if (output.getInt(CAUSALCOUNT) == output.causalInputs().size()) {
						result.addElement(output);
						for(Enumeration enum = output.causalOutputs().elements(); enum.hasMoreElements();) {
							candidate = (Instance) enum.nextElement();
							if (candidate != null) {
								result = QRGutils.mergeVector(result, orderAttributes(model, candidate));
							} // end if
						} // end for
					} // end if
				} // end if
			}else {
				QRGutils.debug("Relationship not connected to an input. model=" + model + " instance=" + instance);
			} // end if
		} // end if else
		return(result);
	} // method
	
	public String getEffect (Model model, Instance instance) {
		String result = UNKNOWN;
		Instance relationship = instance;
		Instance inputInstance = model.findInstance(instance.input());
		String inputState = "";
		if (inputInstance != null) {
			inputState = inputInstance.state();
			if ((relationship.typep(IPLUS)) || 
					(relationship.typep(CPLUS))) {
				result = INCREASING;
			} else if ((relationship.typep(IMINUS)) || 
					(relationship.typep(CMINUS))) {
				result = DECREASING;
			} else if (relationship.typep(QPROPPLUS)) {
				if (inputState.equalsIgnoreCase(INCREASING)) result = INCREASING;
				else if (inputState.equalsIgnoreCase(DECREASING)) result = DECREASING;
				else if (inputState.equalsIgnoreCase(CONSTANT)) result = CONSTANT;
				else if (inputState.equalsIgnoreCase(UNKNOWN)) result = CONSTANT;
				else if (inputState.equalsIgnoreCase(DEFAULT)) result = CONSTANT;
				else result = UNKNOWN;
			} else if (relationship.typep(QPROPMINUS)) {
				if (inputState.equalsIgnoreCase(INCREASING)) result = DECREASING;
				else if (inputState.equalsIgnoreCase(DECREASING)) result = INCREASING;
				else if (inputState.equalsIgnoreCase(CONSTANT)) result = CONSTANT;
				else if (inputState.equalsIgnoreCase(UNKNOWN)) result = CONSTANT;
				else if (inputState.equalsIgnoreCase(DEFAULT)) result = CONSTANT;
				else result = UNKNOWN;
			} else {
				QRGutils.debug("VModel.getEffect", "Unknown Relationship. instance=" + instance);
				result = UNKNOWN;
			} // end if else
		} else QRGutils.debug("VModel.getEffect", "No Connected Input. instance=" + instance);
//QRGutils.debug("VModel.getEffect", "result=" + result);
		return(result);
	} // method
	
	public boolean checkModel( Model model, long checks, boolean clear ) { return( model.checkModel( checks, clear ) ); }
	
	public void updatePeers( Model model ) { model.updatePeers( ); }
	
} // class VModel

// ---------------------------------------------------------------------------
// End of File                                   2003 Northwestern University