package edu.qrg.vmodel;

import edu.qrg.utils.*;
import edu.qrg.utils.CMLio.CMLable;
import edu.qrg.utils.KeyValuePair;
import edu.qrg.utils.KeyValuePair.KVPinterface;
import edu.qrg.vmodel.Ontology.Element;
import edu.qrg.utils.PList.PLinterface;
import edu.qrg.utils.XMLio.XMLable;
import java.io.Serializable;
import java.lang.Class;
import java.lang.ClassNotFoundException;
import java.lang.Cloneable;
import java.lang.IllegalAccessException;
import java.lang.InstantiationException;
import java.lang.Math;
import java.lang.Object;
import java.lang.String;
import java.util.Enumeration;
import java.util.Date;
import java.util.Vector;
import javax.swing.tree.TreeNode;

/** <PRE> Sun Java 1.3.1 -Sourcepath QRG/VModel/development/code/library/definitions
* ----------------------------------------------------------------------------
*  2003 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
* ----------------------------------------------------------------------------
* File name: Definition.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* ----------------------------------------------------------------------------
*   Class: Definition
* Purpose: Definitions are basic ontological parts.  Definitions are reusable
*          abstractions from which we can compose instances in a model. 
* ----------------------------------------------------------------------------
* </PRE> 
*/
public class Definition extends PList
		implements Cloneable, Serializable, VModelConstants, Ontology.Element, 
			CMLio.CMLable, XMLio.XMLable {
	// Constructors
	// ------------------------------------------------------------------------
	/** General Intialization. 
	* A unique identifier is provided for each definition.
	*/
	public Definition ( ) {
		id( composeUniqueIdentifier( ) );
//		CMLcommand( "definition" );
    } // constructor
    
    public Definition( Definition original ) {
    	this( );
    	String id = id( );
    	plist( copyKeyValuePairs( original ).plist() );
    	id( id );
    	typeChain().addElement( new String( original.type( ) ) );
    	typeNameChain().addElement( new String( original.typeName( ) ) );
    	copyInstances( original );
    } // constructor
		
	// Local Properties & Accessors
	// ------------------------------------------------------------------------
	/** List of relationships connected to the definition. */
	public Vector connections ( ) {
		return( getVector( CONNECTIONS ) ); 
	} // reader
	public Vector connections ( Vector v ) {
		return( (Vector) set( CONNECTIONS, v ) ); 
	} // reader
	
	/** List of instances connected by a relationship. */
	public Vector connectors ( ) {
		return( getVector( CONNECTORS ) ); 
	} // reader
	public Vector connectors ( Vector v ) {
		return( (Vector) set( CONNECTORS, v ) ); 
	} // reader
	
	// Jtype
	/** Java type */
	public String Jtype ( ) { return( getClass( ).getName( ) ); }

	/** Inheritence chain of definition types. */
	public Vector typeChain ( ) { return( getVector( TYPECHAIN ) ); }
	/** @see #typeChain( ) */
	public Vector typeChain ( Vector v ) { return( (Vector) set( TYPECHAIN, v ) ); }

	/** Inheritence chain of definition names. */
	public Vector typeNameChain ( ) { return( getVector( SUBCLASSOF ) ); }
	/** @see #typeNameChain( ) */
	public Vector typeNameChain ( Vector v ) { return( (Vector) set( SUBCLASSOF, v ) ); }

	// Local Methods
	// ------------------------------------------------------------------------
	// Uniquify
	/** Counter of the number of reference objects created in session. */
	protected static long sessionCount = 0;

	/** Generates string for id uniquification. */
	public String composeUniqueIdentifier ( ) {
		String result =  "VMODEL"; //getClass( ).getName( ); 
		result += "-" + ( sessionCount++ );
		result += "-" + ( new Date( ) ).getTime( ); 
		result += "-" + hashCode( );
		result += "-" + String.valueOf( (char) ( ( Math.random( ) * 25 ) + 65 ) );
		return( result );
	} // method 
	
	// Type
	/** Jtype check - like instanceof */
	public boolean Jtypep ( String typeID ){
		boolean result = Jtype( ).equalsIgnoreCase( typeID );
		if ( ! result ) {
			try {
				Class c = Class.forName( typeID );
				result = c.isInstance( this );
			} catch ( ClassNotFoundException e ) { }
		} // end if
		return( result );
	} // method
	
	public void add ( Instance instance ) {
		Model model = instance.model( );
		Instance target = null;
		if ( connections( ).contains( instance ) ||
				connectors().contains( instance ) ) return;
		if ( instance.typep( RELATIONSHIP ) ) {
			connections( ).addElement( instance );
/*			
			if ( model != null ) {
				if ( instance.input() != null ) {
					target = model.findInstance( instance.input( ) );
					if ( target != null ) add( target );
				} // end if
				if ( instance.output() != null ) {
					target = model.findInstance( instance.output( ) );
					if ( target != null ) add( target );
				} // end if
			} // end if
*/			
		} else {
			connectors( ).addElement( instance );
/*
			if ( model != null ) {
				for( Enumeration enum = instance.outputs( ).elements( ); enum.hasMoreElements(); ) {
					target = model.findInstance((String) enum.nextElement());
					if ( target.typep( STRUCTURE ) || 
							target.typep( DIRECTINFLUENCE ) ||
							target.typep( REQUIRES ) ) {
						add( target );
						if ( target.typep( DIRECTINFLUENCE ) && ( target.output( ) != null ) ) {
// This is where we need to search for the attribute's owner.							
						} // end if
					} // end for
				} // end for
			} // end if
*/			
		} // end if else
	} // method
	
	public void remove ( Instance instance ) {
		if ( instance.typep( RELATIONSHIP ) ) connections( ).removeElement( instance );
		else connectors( ).removeElement( instance );
	} // method
	
	public void removeAll ( ) {
		connectors( new Vector( ) );
		connections( new Vector( ) );
	} // method
	
	public void copyInstances ( Definition original ) {
		Instance originalInstance = null;
		Instance newInstance = null;
		Vector pairs = new Vector( );
		Vector pair = null;
		for( Enumeration instances = original.connectors( ).elements( ); instances.hasMoreElements(); ) {
			originalInstance = (Instance) instances.nextElement();
			newInstance = new Instance( originalInstance );
			add( newInstance );
			pair = new Vector( );
			pair.addElement( originalInstance );
			pair.addElement( newInstance );
			pairs.addElement( pair );
		} // end for
		for( Enumeration instances = original.connections( ).elements( ); instances.hasMoreElements(); ) {
			originalInstance = (Instance) instances.nextElement();
			newInstance = new Instance( originalInstance );
			add( newInstance );
			pair = new Vector( );
			pair.addElement( originalInstance );
			pair.addElement( newInstance );
			pairs.addElement( pair );
		} // end for
		reconnect( pairs );
	} // method
	
	public void reconnect ( Vector pairVector ) {
		Instance originalInstance = null;
		Instance newInstance = null;
		Instance inputInstance = null;
		Instance outputInstance = null;
		Instance original = null;
		Instance duplicate = null;
		String inputID = null;
		String outputID = null;
		Vector pair = null;
		Vector candidate = null;
		for( Enumeration pairs = pairVector.elements( ); pairs.hasMoreElements(); ) {
			pair = (Vector) pairs.nextElement();
			originalInstance = (Instance) pair.elementAt( 0 );
			newInstance = (Instance) pair.elementAt( 1 );
			if ( originalInstance.typep( RELATIONSHIP ) ) {
				inputID = originalInstance.input();
				outputID = originalInstance.output();
				inputInstance = null;
				outputInstance = null;
				for( Enumeration candidates = pairVector.elements( ); candidates.hasMoreElements(); ) {
					candidate = (Vector) candidates.nextElement();
					original = (Instance) pair.elementAt( 0 );
					duplicate = (Instance) pair.elementAt( 1 );
					if ( original.id( ).equalsIgnoreCase( inputID ) ) {
						inputInstance = duplicate;
					} // end if
					if ( original.id( ).equalsIgnoreCase( outputID ) ) {
						outputInstance = duplicate;
					} // end if
				} // end for
				if ( inputInstance != null ) {
					newInstance.input( inputInstance.id( ) );
					inputInstance.connections( ).removeElement( originalInstance.id( ) );
					inputInstance.connections( ).addElement( newInstance.id( ) );
				} // end if
				if ( outputInstance != null ) {
					newInstance.output( outputInstance.id( ) );
					outputInstance.connections( ).removeElement( originalInstance.id( ) );
					outputInstance.connections( ).addElement( newInstance.id( ) );
				} // end if
			} // end if
		} // end for
	} // method
	
	/** <PRE>
	* ------------------------------------------------------------------------
	* Purpose: Find the instance with the specified id in the model,
	*          if one exists there.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public Instance findInstance ( String id ) {
		Instance result = null;
		Instance mumble = null;
		if ( ! ( ( id == null ) || ( id.equalsIgnoreCase( "" ) ) ) ) {
			for (Enumeration o = children( ); o.hasMoreElements() ;) {
				mumble =  (Instance) o.nextElement();
				if ( id.equalsIgnoreCase( mumble.id( ) ) ) {
					result = mumble;
					break;
				} // end if
			} // end for
		} // end if
		if ( result == null ) QRGutils.debug( "Model.findInstance", "ID not found. id=" + id + " result=" + result );
		return( result );
	} // method
	
	// Ontology.Element methods
	// -----------------------------------------------------------------------
	// Unique Identifier
	/** String identifier for object. */
	public String id ( ) { return( getString( ID ) ); }
	/** @see #id( ) */
	public String id ( String s ) { return( (String) set( ID, s ) ); }
	
	/** String label for object.
	* * May contain spaces.
	* * Doesn't have to be unique.
	*/
	public String name ( ) { return( getString( NAME ) ); }
	/** @see #name( ) */
	public String name ( String s ) { return( (String) set( NAME, s ) ); }
	
	// Help
	/** Documentation string for object. */
	public String documentation ( ) { return( getString( DESCRIPTION ) ); }
	/** @see #documentation( ) */
	public String documentation ( String s ) { return( (String) set( DESCRIPTION, s ) ); }
	
	/** URL of web-based help file. */
	public String helpURL ( ) { return( getString( HELPURL ) ); }
	/** @see #helpURL( ) */
	public String helpURL ( String s ) { return( (String) set( HELPURL, s ) ); }
	
	/** Purpose: Returns the string filename of an image representing
	             the Definition. */
	public String icon ( ) { return( getString( ICON ) ); }
	/** @see #icon( ) */
	public String icon ( String s ) { return( (String) set( ICON, s ) ); }

	// Category
	/** Purpose: Returns the string id of the category to which this
	             category belongs. */
	public String category ( ) { return( getString( CATEGORY ) ); }
	/** @see #category( ) */
	public String category ( String s ) { return( (String) set( CATEGORY, s ) ); }
	
	// type
	/** Type of definition. */
	public String type ( ) { return( getString( TYPE ) ); }
	/** @see #type( ) */
	public String type ( String s ) { return( (String) set( TYPE, s ) ); }
	
	/** Returns the name of type definition. */
	public String typeName ( ) {
		return( getString( TYPENAME ) );
	} // method
	/** @see #typeName( ) */
	public String typeName ( String s ) {
		return( (String) set( TYPENAME, s ) );
	} // method
	
	/** Type check */
	public boolean typep ( String typeID ) {
		boolean result = false;
		Definition mumble = null;
		result = id( ).equalsIgnoreCase( typeID ) ||
			name( ).equalsIgnoreCase( typeID ) ||
			category( ).equalsIgnoreCase( typeID ) ||
			type( ).equalsIgnoreCase( typeID ) ||
			typeChain( ).contains( typeID ) ||
			typeNameChain( ).contains( typeID ) ||
			Jtypep( typeID );
		return( result );
	} // method
	
	// TreeNode Methods
	// -----------------------------------------------------------------------
	/** Returns a list of all contained instances. */
	public Enumeration children ( ) {
		Vector mumble = new Vector( );
		mumble.addAll( connections( ) );
		mumble.addAll( connectors( ) );
		return( mumble.elements( ) );
	} // method
	
	/** Definitions are leaf nodes in an ontology. */
	public boolean getAllowsChildren( ) {
		return( false );
	} // method
	
	/** No children so return null. */
	public TreeNode getChildAt(int childIndex) {
		return( null );
	} // method
	
	/** No children. */
	public int getChildCount( ) {
		return( 0 );
	} // method
	
	/** No children so return -1. */
	public int getIndex(TreeNode node) {
		return( -1 );
	} // method
	
	/** Return owning ontological category. */
	public TreeNode getParent( ) {
		Object result = null;
		// Hmmm... This doesn't seem like it is correct.
		QRGutils.debug( "Definition", "getParent" );
		if ( category( ) != null ) result = Ontology.find( category( ) );
		if ( ! ( result instanceof TreeNode ) ) result = null;
		QRGutils.debug( "Definition", "getParent", "result=" + result );
		return( (TreeNode) result );
	} // method
	
	/** Definitions are all leaves. */
	public boolean isLeaf() {
		return( true );
	} // method

	// Stringification.
	// -----------------------------------------------------------------------
	/** Stringifies the object's KeyValuePairs with indentation.
	* @see PList
	* @see KeyValuePair
	*/
	public String stringifyKeyValuePairs ( int indent ) {
		String result = "";
		int j = 0;
		KeyValuePair candidate = null;
		if (keyValuePairs() != null) {
			j = 0;
			while( j < keyValuePairs().size( ) ) {
				candidate = (KeyValuePair) keyValuePairs().elementAt(j);
				if ( ! ( candidate.value( ) instanceof Vector ) ) {
					if (j > 0) result += QRGutils.eol( ) + QRGutils.space(indent);
					else result += " "; 
					result += candidate;
				} // end if
				j++;
			} // while
		} // end if
		return(result);
	} // method toString
	
	/** Stringifies the object in XML format with indentation. */
	public String stringify ( int indent ) {
		String result = "";
		KeyValuePair pair = null;
		Object mumble = null;
		final VectorElement grumble = new VectorElement( );
		boolean flag = true;
		result = QRGutils.space( indent ) + "<" + "Definition"; //this.getClass( ).getName( );
		result += stringifyKeyValuePairs( result.length( ) + 1 );
		for (Enumeration e = keyValuePairs( ).elements( ) ; e.hasMoreElements() ;) {
			pair = (KeyValuePair) e.nextElement( );
			if ( ( pair.value( ) instanceof Vector ) &&
					( ! ( (Vector) pair.value( ) ).isEmpty( ) ) ) {
				flag = false;
				break;
			} // end if
		} // end for
		int tabs = indent + 3;
		if ( flag ) result += " />" + QRGutils.eol( );
		else {
			result += " >" + QRGutils.eol( );
			for (Enumeration e = keyValuePairs( ).elements( ) ; e.hasMoreElements() ;) {
				pair = (KeyValuePair) e.nextElement( );
				if ( pair.value( ) instanceof Vector ) {
					for (Enumeration p = ((Vector) pair.value( )).elements( ) ; p.hasMoreElements() ;) {
						mumble = p.nextElement( );
						grumble.set( TAG, pair.key( ) );
						grumble.set( VALUE, mumble );
						result += QRGutils.space( tabs ) + grumble.xmlify( );
					} // end for
				} // end if
			} // end for
			result += QRGutils.space( indent ) + "</" + "Definition" /* this.getClass( ).getName( ) */ + ">" + QRGutils.eol( );
		} // end if 
		return( result );
	} // method toString
	
	/** Stringifies the object. */
	public String stringify ( ) { 
		return(stringify( 0 )); 
	} // method toString
	
	/** Returns the name of the Definition. */
	public String toString ( ) {
		return( name( ) );
	} // method
	
	// CMLio.CMLable
	// -----------------------------------------------------------------------
	/** Returns the CML command used to represent the object. */
	public String CMLcommand ( ) { return( "DEFINITION" ); }
	/** @see #CMLcommand( ) */
//	public String CMLcommand ( String s ) { return( (String) set( "CMLCOMMAND" ,s ) ); }

	/** Stringifies the object into CML format. */
	public String toCML ( ) {
		String result = "";
		KeyValuePair mumble = null;
		for( Enumeration instances = connectors( ).elements( ); instances.hasMoreElements(); ) {
			result += ((Instance) instances.nextElement()).toCML() + QRGutils.eol( );
		} // end for
		for( Enumeration instances = connections( ).elements( ); instances.hasMoreElements(); ) {
			result += ((Instance) instances.nextElement()).toCML() + QRGutils.eol( );
		} // end for
		result += QRGutils.eol( ) + "(" + CMLcommand( ) + " " + id( );
		for (Enumeration e = keyValuePairs( ).elements( ) ; e.hasMoreElements() ;) {
			mumble = (KeyValuePair) e.nextElement();
			if ( ! mumble.key( ).equalsIgnoreCase( ID ) ) {
				if ( mumble.value( ) instanceof Vector ) {
					if ( ((Vector) mumble.value( ) ).size( ) > 0 ) {
						result += QRGutils.eol( ) + "   :" + mumble.key( ) + " (";
						for ( Enumeration values = ((Vector) mumble.value( )).elements( ); values.hasMoreElements( ); ) {
							result += values.nextElement( );
							if ( values.hasMoreElements( ) ) result += " ";
						} // end for
						result += ")";
					} // end if
				} else result += QRGutils.eol( ) + "   :" + mumble.key( ) + " \"" + mumble.value( ) + "\"";
			} // end if
		} // end for
		result += ")" + QRGutils.eol( );
		return( result );
	} // method
	
	/** Saves the ontology to filename in XML format. */
	public void exportCML ( String filename ) {
		CMLio.save(  QRGutils.QRGuserpath( ) + filename + "." + CML_SUFFIX , this );
	} // method
	
	// XMLio.XMLable
	// -----------------------------------------------------------------------
	/** Stringify the an XML definition for the object. */
	public String xmlDef ( ) { 
		String result = 	"";
		KeyValuePair mumble = null;
		String classname = this.getClass( ).getName( );
		result += "<!DOCTYPE " + classname + " [\r\n";
		result += "<!ELEMENT " + classname + " EMPTY >\r\n";
		for (Enumeration e = keyValuePairs( ).elements( ) ; e.hasMoreElements() ;) {
			mumble = (KeyValuePair) e.nextElement();
			result += "<!ATTLIST " + classname + " " + mumble.key( ).toUpperCase( ) + " CDATA #REQUIRED >\r\n";
		} // end for
		result += "]>";
		return( result ); 
	} // method
	
	/** Stringify the object into the XML format. */
	public String xmlify ( ) {
		String result = stringify( );
		return(result);
	} // method
	
	/** Initializes slots after object is created by XML loader. */
	public void XMLioObjectInit ( Vector attr, Vector elmnt ) {
		Object mumble = null;
		VectorElement grumble = null;
		Definition humble = null;
		keyValuePairs( attr );
		for( Enumeration enum = elmnt.elements( ); enum.hasMoreElements( ); ) {
			mumble = enum.nextElement( );
			if ( mumble instanceof VectorElement ) {
				grumble = (VectorElement) mumble;
				getVector( grumble.getString( TAG ) ).addElement( grumble.get( VALUE ) );
			} // end if
		} // end for
	} // constructor PList
	
	/** Saves the ontology to filename in XML format. */
	public void exportXML ( String filename ) {
		XMLio.save(  QRGutils.QRGuserpath( ) + filename + "." + XML_SUFFIX , this );
	} // method
} // class

// --------------------------------------------------------------------------
// End of File: Definition.java              (c) 2003 Northwestern University