//package edu.qrg.vmodel.library.ontologies;

import CMLio.CMLable;
import KeyValuePair.KVPinterface;
import Ontology.Category;
import Ontology.Element;
import Ontology.OntInterface;
import PList.PLinterface;
import XMLio.XMLable;
import java.io.Serializable;
import java.lang.Cloneable;
import java.lang.Object;
import java.lang.String;
import java.lang.System;
import java.util.Enumeration;
import java.util.Vector;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeModelEvent;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

/** <PRE> 
* Sun Java 1.3.1 -Sourcepath QRG/VModel/development/code/library/ontologies
* ---------------------------------------------------------------------------
*  2003 Northwestern University, Qualitative Reasoning Group, Ken Forbus
* ---------------------------------------------------------------------------
* File name: Ontology.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* ---------------------------------------------------------------------------
*   Class: Ontology
* Purpose: Defines an library of elements.  May contain categories.
*          All derived ontological elements are contained in libraries.
* ---------------------------------------------------------------------------
* Notes:
* * Libraries implement TreeModel so they can be displayed as a tree with
*   minimal fuss. (We already have a data model - shouldn't need another.)
* * Libraries implement TreeNode so nested libraries can be displayed as
*   nodes in a tree list.
* ---------------------------------------------------------------------------
* </PRE> */
public class Ontology extends Ontology
		implements Cloneable, Serializable, Ontology.OntInterface {
	// Constructors
	// -----------------------------------------------------------------------
	/** General initialization. */
	public Ontology ( ) {
		super( );
   } // default constructor
		
	/** Initializes most frequently used slots. */
   public Ontology( String id, String name, String documentation, String helpURL ) {
    	this( );
    	id( id );
    	name( name );
    	documentation( documentation );
    	helpURL( helpURL );
    } // constructor

	// Interfaces
	// -----------------------------------------------------------------------
	/** <PRE>
	* ------------------------------------------------------------------------
	* Interface: Ontology.OntInterface
	* Purpose: In general, anything implementing this interface should be
	*          acceptable as a Ontology.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public interface OntInterface extends Cloneable, Serializable, 
		Ontology.OntInterface, Ontology.Element, Ontology.Category {
		// Properties & Accessors
		// --------------------------------------------------------------------
		/** Keeps track of the selected element in the tree model.
		* * Is this still being used?
		*/
		public String selectedElement ( );
		/** @see #selectedElement( ) */
		public String selectedElement ( String definitionID );
	    public void reload(TreeNode node);
	} // interface
	
	/** <PRE>
	* ------------------------------------------------------------------------
	* Interface: Ontology.Element
	* Purpose: In general, anything implementing this interface may be
	*          added to a Ontology as an ontological element.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public interface Element 
			extends Cloneable, Serializable, VModelConstants, 
				Ontology.Element, TreeNode {

	} // interface
	
	/** <PRE>
	* ------------------------------------------------------------------------
	* Interface: Ontology.Category
	* Purpose: Allows libraries to be nested.
	* ------------------------------------------------------------------------
	* </PRE> 
	*/
	public interface Category 
			extends Cloneable, Serializable, VModelConstants, 
				Ontology.Category, TreeModel {
						
	} // interface
	
	// Properties & Accessors
	// -----------------------------------------------------------------------
	/** @serial */
	protected String selectedElement = null;
	/** Stashes the id of the ontological element currently
	*   selected in the tree model.
	*/
	public String selectedElement ( ) { return( selectedElement ); }
	/** @see #selectedElement( ) */
	public String selectedElement ( String definitionID ) { 
		return( selectedElement = definitionID ); 
	} // writer
	
	/** Removes an ontological element from the ontology. */
	public void remove ( Ontology.Element a ) {
		super.remove( a );
		reload( this );
	} // method
	
	/** Adds an ontological element to the ontology. */
	public void add ( Ontology.Element a ) {
		super.add( a );
		reload( this );
	} // method
	
	/** Purpose: Returns the string id of the category to which this
	             category belongs. */
	public String category ( ) { return( getString( CATEGORY ) ); }
	/** @see #category( ) */
	public String category ( String s ) { return( (String) set( CATEGORY, s ) ); }
	
	/** Purpose: Returns the string filename of an image representing
	             the library. */
	public String icon ( ) { return( getString( ICON ) ); }
	/** @see #icon( ) */
	public String icon ( String s ) { return( (String) set( ICON, s ) ); }
	/** Purpose: Returns the string filename of an image representing
	             the library tree node in an Open or Close state. */
	public String icon ( boolean expanded ) { 
		String result = icon( );
		if ( expanded ) result += "Open";
		else result += "Closed";
		return( result ); 
	} // method
	
	/** Type of ontological element. */
	public String type ( ) { return( LIBRARY ); }
	/** @see #type( ) */
	public String type ( String s ) { return( LIBRARY ); }

	/** Type of ontological element. */
	public String typeName ( ) { return( LIBRARY ); }
	/** @see #type( ) */
	public String typeName ( String s ) { return( LIBRARY ); }

	/** Type check */
	public boolean typep ( String typeID ) {
		boolean result = false;
		Definition mumble = null;
		result = id( ).equalsIgnoreCase( typeID ) ||
			name( ).equalsIgnoreCase( typeID ) ||
			category( ).equalsIgnoreCase( typeID ) ||
			type( ).equalsIgnoreCase( typeID );
		return( result );
	} // method
	
	// TreeModel Methods
	// -----------------------------------------------------------------------
	/** Use accessors. */
	protected transient Vector treeModelListeners = null;
	/** Stash for listeners. Default is new Vector( ). */
	public Vector treeModelListeners ( ) {
		if ( treeModelListeners == null ) treeModelListeners = new Vector( ); 
		return( treeModelListeners ); 
	} // reader
	
	/** Add a tree model listener.  Wow! These are exciting comments! */
	public void addTreeModelListener(TreeModelListener l) {
		treeModelListeners( ).addElement( l );
	} // method
	
	/** Remove tree model listener. */ 
	public void removeTreeModelListener(TreeModelListener l) {
		treeModelListeners( ).removeElement( l );
	} // method
	
	/** Returns child tree node. */
	public Object getChild(Object category, int index) {
		return( ((TreeNode) category).getChildAt( index ) );
	} // method
	
	/** Returns number of child nodes. */
	public int getChildCount(Object category) {
		return( ((TreeNode) category).getChildCount( ) );
	} // method
	
	/** Returns location of child node in tree. */
	public int getIndexOfChild(Object category, Object child) {
		return( ((TreeNode) category).getIndex( (TreeNode) child ) );
	} // method
	
	/** Return root node of tree. */
	public Object getRoot() { return( this ); } // method
	
	/** Return true if node has no children. */
	public boolean isLeaf(Object node) {
		return( ((TreeNode) node).isLeaf( ) );
	} // method
	
	/** Does this ever get called?  How can we use it to allow renaming of
	    ontological elements in the tree display? */
	public void valueForPathChanged(TreePath path, Object newValue) {
		System.out.println( "Ontology.valueForPathChanged path=" + path + " newValue=" + newValue );
	} // method

    /*
     * Notify all listeners that have registered interest for
     * notification on this event type.  The event instance 
     * is lazily created using the parameters passed into 
     * the fire method.
     * @see EventListenerList
     */
    protected void fireTreeNodesChanged(Object source, Object[] path, 
                                        int[] childIndices, 
                                        Object[] children) {
       	TreeModelEvent e = null;
       	TreeModelListener l = null;
		for ( Enumeration enum = treeModelListeners( ).elements( ); enum.hasMoreElements( ); ) {
			l = (TreeModelListener) enum.nextElement( );
                if (e == null)
                    e = new TreeModelEvent(source, path, 
                                           childIndices, children);
                l.treeNodesChanged(e);
        }
    }

    /*
     * Notify all listeners that have registered interest for
     * notification on this event type.  The event instance 
     * is lazily created using the parameters passed into 
     * the fire method.
     * @see EventListenerList
     */
    protected void fireTreeNodesInserted(Object source, Object[] path, 
                                        int[] childIndices, 
                                        Object[] children) {
       	TreeModelEvent e = null;
       	TreeModelListener l = null;
		for ( Enumeration enum = treeModelListeners( ).elements( ); enum.hasMoreElements( ); ) {
			l = (TreeModelListener) enum.nextElement( );
                // Lazily create the event:
                if (e == null)
                    e = new TreeModelEvent(source, path, 
                                           childIndices, children);
                l.treeNodesInserted(e);
        }
    }

    /*
     * Notify all listeners that have registered interest for
     * notification on this event type.  The event instance 
     * is lazily created using the parameters passed into 
     * the fire method.
     * @see EventListenerList
     */
    protected void fireTreeNodesRemoved(Object source, Object[] path, 
                                        int[] childIndices, 
                                        Object[] children) {
       	TreeModelEvent e = null;
       	TreeModelListener l = null;
		for ( Enumeration enum = treeModelListeners( ).elements( ); enum.hasMoreElements( ); ) {
			l = (TreeModelListener) enum.nextElement( );
                if (e == null)
                    e = new TreeModelEvent(source, path, 
                                           childIndices, children);
                l.treeNodesRemoved(e);
        }
    }

    /*
     * Notify all listeners that have registered interest for
     * notification on this event type.  The event instance 
     * is lazily created using the parameters passed into 
     * the fire method.
     * @see EventListenerList
     */
    protected void fireTreeStructureChanged(Object source, Object[] path, 
                                        int[] childIndices, 
                                        Object[] children) {
       	TreeModelEvent e = null;
       	TreeModelListener l = null;
		for ( Enumeration enum = treeModelListeners( ).elements( ); enum.hasMoreElements( ); ) {
			l = (TreeModelListener) enum.nextElement( );
                if (e == null)
                    e = new TreeModelEvent(source, path, 
                                           childIndices, children);
                l.treeStructureChanged(e);
        }
    }

    /**
      * Invoke this method after you've changed how node is to be
      * represented in the tree.
      */
 //   public void nodeChanged(TreeNode node) {
 //       if(listenerList != null && node != null) {
 //           TreeNode         parent = node.getParent();
//
//            if(parent != null) {
//                int        anIndex = parent.getIndex(node);
//                if(anIndex != -1) {
//                    int[]        cIndexs = new int[1];
//
//                    cIndexs[0] = anIndex;
//                   nodesChanged(parent, cIndexs);
//                }
//            }
//	    else if (node == getRoot()) {
//		nodesChanged(node, null);
//	    }
//        }
//    }

    /**
     * Invoke this method if you've modified the TreeNodes upon which this
     * model depends.  The model will notify all of its listeners that the
     * model has changed below the node <code>node</code> (PENDING).
     */
    public void reload(TreeNode node) {
        if(node != null) {
            fireTreeStructureChanged(this, getPathToRoot(node), null, null);
        }
    }

    /**
      * Invoke this method after you've inserted some TreeNodes into
      * node.  childIndices should be the index of the new elements and
      * must be sorted in ascending order.
      */
//    public void nodesWereInserted(TreeNode node, int[] childIndices) {
//        if(listenerList != null && node != null && childIndices != null
//           && childIndices.length > 0) {
//            int               cCount = childIndices.length;
//            Object[]          newChildren = new Object[cCount];
//
//            for(int counter = 0; counter < cCount; counter++)
//                newChildren[counter] = node.getChildAt(childIndices[counter]);
//            fireTreeNodesInserted(this, getPathToRoot(node), childIndices, 
//                                  newChildren);
//        }
//    }
    
    /**
      * Invoke this method after you've removed some TreeNodes from
      * node.  childIndices should be the index of the removed elements and
      * must be sorted in ascending order. And removedChildren should be
      * the array of the children objects that were removed.
      */
    public void nodesWereRemoved(TreeNode node, int[] childIndices,
                                 Object[] removedChildren) {
        if(node != null && childIndices != null) {
            fireTreeNodesRemoved(this, getPathToRoot(node), childIndices, 
                                 removedChildren);
        }
    }

    /**
      * Invoke this method after you've changed how the children identified by
      * childIndicies are to be represented in the tree.
      */
    public void nodesChanged(TreeNode node, int[] childIndices) {
        if(node != null) {
	    if (childIndices != null) {
		int            cCount = childIndices.length;

		if(cCount > 0) {
		    Object[]       cChildren = new Object[cCount];

		    for(int counter = 0; counter < cCount; counter++)
			cChildren[counter] = node.getChildAt
			    (childIndices[counter]);
		    fireTreeNodesChanged(this, getPathToRoot(node),
					 childIndices, cChildren);
		}
	    }
	    else if (node == getRoot()) {
		fireTreeNodesChanged(this, getPathToRoot(node), null, null);
	    }
        }
    }

    /**
      * Invoke this method if you've totally changed the children of
      * node and its childrens children...  This will post a
      * treeStructureChanged event.
      */
    public void nodeStructureChanged(TreeNode node) {
        if(node != null) {
           fireTreeStructureChanged(this, getPathToRoot(node), null, null);
        }
    }

    /**
     * Builds the parents of node up to and including the root node,
     * where the original node is the last element in the returned array.
     * The length of the returned array gives the node's depth in the
     * tree.
     * 
     * @param aNode the TreeNode to get the path for
     * @param an array of TreeNodes giving the path from the root to the
     *        specified node. 
     */
    public TreeNode[] getPathToRoot(TreeNode aNode) {
        return getPathToRoot(aNode, 0);
    }

    /**
     * Builds the parents of node up to and including the root node,
     * where the original node is the last element in the returned array.
     * The length of the returned array gives the node's depth in the
     * tree.
     * 
     * @param aNode  the TreeNode to get the path for
     * @param depth  an int giving the number of steps already taken towards
     *        the root (on recursive calls), used to size the returned array
     * @return an array of TreeNodes giving the path from the root to the
     *         specified node 
     */
    protected TreeNode[] getPathToRoot(TreeNode aNode, int depth) {
        TreeNode[]              retNodes;
	// This method recurses, traversing towards the root in order
	// size the array. On the way back, it fills in the nodes,
	// starting from the root and working back to the original node.

        /* Check for null, in case someone passed in a null node, or
           they passed in an element that isn't rooted at root. */
        if(aNode == null) {
            if(depth == 0)
                return null;
            else
                retNodes = new TreeNode[depth];
        }
        else {
            depth++;
            if(aNode == getRoot( ))
                retNodes = new TreeNode[depth];
            else
                retNodes = getPathToRoot(aNode.getParent(), depth);
            retNodes[retNodes.length - depth] = aNode;
        }
        return retNodes;
    }

	// TreeNode Methods
	// -----------------------------------------------------------------------
	/** Enumerate child nodes. */
	public Enumeration children( ) {
		return( elements( ).elements( ) );
	} // method
	
	/** Yes, allow child nodes. */
	public boolean getAllowsChildren( ) {
		return( true );
	} // method
	
	/** Return child node at index. */
	public TreeNode getChildAt(int childIndex) {
		TreeNode result = (TreeNode) elements( ).elementAt( childIndex );
		return( result );
	} // method
	
	/** Return count of child nodes. */
	public int getChildCount( ) {
		int result = elements( ).size( );
		return( result );
	} // method
	
	/** Return the location of the specified node. */
	public int getIndex(TreeNode node) {
		return( elements( ).indexOf( node ) );
	} // method
	
	/** Return the parent category. */
	public TreeNode getParent( ) {
		Object result = null;
		if ( category( ) != null ) result = find( category( ) );
		if ( ! ( result instanceof TreeNode ) ) result = null;
		return( (TreeNode) result );
	} // method
	
	/** true if this node a leaf? */
	public boolean isLeaf() {
		boolean result = elements( ).isEmpty( );
		return( result );
	} // method
} // class Ontology

// --------------------------------------------------------------------------
// End of File: Ontology.java                 (c) 2003 Northwestern University
