package edu.qrg.utils;

import java.io.FileDescriptor;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.lang.Exception;
import java.lang.Object;
import java.lang.String;

/** <PRE> Sun Java 1.3.1 -Sourcepath QRG/VModel/development/code/utils
* ----------------------------------------------------------------------------
*  2003 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
* ----------------------------------------------------------------------------
* File name: CMLio.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* ----------------------------------------------------------------------------
*   Class: CMLio
* Purpose: Load and Save objects in the CML format.
* ----------------------------------------------------------------------------
* Notes:
* * Load not yet implemented.
* * No constructors - methods are called statically.
* ----------------------------------------------------------------------------
* </PRE> */
public class CMLio {
	// Interface
	// ------------------------------------------------------------------------
	/** <PRE>
	* ------------------------------------------------------------------------
	* Interface: CMLio.CMLable
	* Purpose: In general, anything implementing this interface 
	*          may be saved or loaded by the methods in CMLio.
	* ------------------------------------------------------------------------
	* </PRE> */
	public interface CMLable {
		/** Returns the CML command used to describe the object. ex: DEFENTITY */
		public String CMLcommand ();
		/** Stringify the object into cml format. */
		public String toCML ();
		/** Save the calling object to filename in CML format. */
		public void exportCML (String filename);
	} // interface Interface

	// Methods for saving.
	// -----------------------------------------------------------------------
	/** Save the specified thing to the specified file using cml format. */
	public static CMLio.CMLable save (String filename, CMLio.CMLable thing) {
		CMLio.CMLable result = null;
		try {
			FileWriter fw = new FileWriter(filename);
			PrintWriter pw = new PrintWriter(fw);
			pw.print(thing.toCML());
			pw.close();
			fw.close();
			result = thing;
		} catch (Exception e) {
			QRGutils.debug("Save CML", " Error: \n" + e);
			QRGutils.debugStackTrace(e);
		   //  Potential leak here.
		} // try catch
		return(result);
	} // method save
	
	/** Save the specified thing to the specified FileOutputStream. */
	public static CMLio.CMLable save (FileOutputStream fout, CMLio.CMLable thing) {
		CMLio.CMLable result = null;
		try { result = save(fout.getFD(), thing);}
		catch (Exception e) {
		   	QRGutils.debug("Save CML", " Error: \n" + e);
			QRGutils.debugStackTrace(e);
		} // try catch
		return(result);
	} // method save
	
	/** Save the specified thing to the specified FileDescriptor. */
	public static CMLio.CMLable save (FileDescriptor fout, CMLio.CMLable thing) {
		CMLio.CMLable result = null;
		try {
			FileWriter fw = new FileWriter(fout);
			PrintWriter pw = new PrintWriter(fw);
			pw.print(thing.toCML());
			pw.close();
			fw.close();
			result = thing;
		} catch (Exception e) {
		   	QRGutils.debug("Save CML", " Error: \n" + e);
			QRGutils.debugStackTrace(e);
		   //  Potential leak here.
		} // try catch
		return(result);
	} // method save
	
} // class CMLio

// ---------------------------------------------------------------------------
// End of File: CMLio.java                       2003 Northwestern University