package edu.qrg.utils;

import edu.qrg.vmodel.*;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.lang.Cloneable;
import java.lang.Exception;
import java.lang.Object;
import java.lang.String;
import java.util.Vector;
import javax.swing.ImageIcon;

/** <PRE> Sun Java 1.3.1 -Sourcepath QRG/VModel/development/code/utils
* ---------------------------------------------------------------------------
*  2003 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
* ---------------------------------------------------------------------------
* File name: ImageCache.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* ---------------------------------------------------------------------------
*   Class: ImageCache
* Purpose: Cache for commonly used images.
* Notes:
* * No constructors - methods are called statically.
* ---------------------------------------------------------------------------
* </PRE> */
public class ImageCache implements VModelConstants {
	// images
	/** Cache Vector of images keyed by filename.
	* * Implement as HashTable?
	* * Use the accessors.
	* @see #images()
	*/
   protected static Vector images = null;
   /** @see #images */
	public static Vector images() {
		if (images == null) images = new Vector(); 
		return(images); 
	} // reader

    // IMAGE CACHE
	/** Finds an image in the cache whose key matches filename.
	*   Returns null if no matching image is found.
	*/
    public static KeyValuePair findImage(String filename) {
		KeyValuePair attribute = null;
		for(int i = 0; i < images().size(); i++) {
		    attribute = (KeyValuePair) images().elementAt(i);
		    if (attribute.key().equalsIgnoreCase(filename)) break;
		    else attribute = null;
		} // end for i
		return(attribute);
    } // method getImage

   /** Loads and caches an image file.
	*  Overwrites previously cached load of file.
	*/
    public static ImageIcon loadImage (String filename) {
		String mumble = QRGutils.QRGroot()+IMAGEPATH+filename;
		KeyValuePair grumble = findImage(filename);
		ImageIcon result = null;
		if ((!QRGutils.isApplet()) && (grumble == null) && 
				(!((new File(mumble)).exists()))) {
			mumble = QRGutils.replaceExtension(mumble, "gif");
			if (!((new File(mumble)).exists())) {
				mumble = QRGutils.replaceExtension(mumble, "jpg");
			} // end if
		} // end if
		if ((!QRGutils.isApplet()) && (grumble == null) && 
			(!((new File(mumble)).exists())))
			QRGutils.debug("loadImage", "File Not Found: " + mumble);
		else {
			result = QRGutils.getImageIcon(mumble);
			if (result != null) {
			    if (grumble == null)
					images().addElement(new KeyValuePair(filename, result));
			    else grumble.value(result);
			} else if (grumble != null) result = (ImageIcon) grumble.value();
		} // end if exists
		return(result);
    } // method loadImage

   /** Gets/Loads a cached image file.
	*  Loads file if not in the cache.
	*/
    public static ImageIcon getImage(String filename) {
		ImageIcon result = null;
		KeyValuePair attribute = findImage(filename);
		if (attribute == null) result = loadImage(filename);
		else result = (ImageIcon) attribute.value();
		return(result);
    } // method getImage

   /** Dumps load code for cached images. */
    public static void dumpImages(String filename) {

		try {
			FileWriter fw = new FileWriter(filename);
			PrintWriter pw = new PrintWriter(fw);
			KeyValuePair pair = null;
			for(int i = 0; i < images().size(); i++) {
				pair = (KeyValuePair) images().elementAt(i);
				pw.print("ImageCache.loadImage(\"" + pair.key() +"\");" + QRGutils.eol());
			} // end for i
			pw.close();
			fw.close();
		} catch (Exception e) {
			QRGutils.debug("ImageCache", "dumpImages", " Error: \n" + e);
			QRGutils.debugStackTrace(e);
		   //  Potential leak here.
		} // try catch
    } // method getImage
} // class ImageCache
        
// --------------------------------------------------------------------------
// End of File: ImageCache.java                  2003 Northwestern University