package edu.qrg.utils;

import java.io.File;
import java.io.FilenameFilter;
import java.lang.Cloneable;
import java.lang.Object;
import java.lang.String;

/** <PRE> Sun Java 1.3.1 -Sourcepath QRG/VModel/development/code/utils
* -------------------------------------------------------------------------
* 2003 Northwestern University, Qualitative Reasoning Group, Ken Forbus
* -------------------------------------------------------------------------
* File name: QRGfilefilter.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* -------------------------------------------------------------------------
*   Class: QRGfilefilter
* Purpose: This is a simple string based filename filter. It filters based on
*          a suffix value.  (For example, xml)  The suffix may be changed, but
*          filtration is only over a single suffix.
* -------------------------------------------------------------------------
* </PRE> */
public class QRGfilefilter implements QRGconstants, FilenameFilter {
	// Constructors
	// -----------------------------------------------------------------------
	/** General initialization.  Suffix is not initialized. */
	public QRGfilefilter () { super(); }
	/** General initialization.  Suffix is set to arg. */
	public QRGfilefilter (String suffix) {
		this();
		suffix(suffix);
	} // constructor
	
	// Constants
	// -----------------------------------------------------------------------
	// Should these be moved to QRGconstants?
	public static final String NOFILTRATION = "*";
	public static final String CLASS = "class";
	public static final String JAVA = "java";
	public static final String XML = "xml";
	public static final String CML = "cml";
	public static final String VModel = "vmf";
	public static final String LISP = "lisp";
	public static final String LSP = "lsp";
	public static final String FASL = "fasl";
	public static final String FSL = "fsl";
	
	// Properties & Accessors
	// -----------------------------------------------------------------------
	/** Stash suffix used in filtration.
	* * USE THE ACCESSORS.
	* * Does not include leading dot.
	* @see #suffix()
	* @see #suffix(String)
	*/
	protected String suffix = NOFILTRATION;
	/** @see #suffix */
	public String suffix () { return(suffix); }
	/** @see #suffix */
	public String suffix (String s) { return(suffix = s); }
	
	// filter method
	// -----------------------------------------------------------------------
	/** Accept names that end with specified suffix.
	* @see #suffix(String)
	*/
	public boolean accept(File dir, String name) {
		boolean result = ((suffix() == NOFILTRATION) ||
			((name.length() > (1 + suffix.length())) &&
				name.toUpperCase().endsWith("." + suffix().toUpperCase())));
		return(result);
	} // method
} // class

// ------------------------------------------------------------------------
// End of File: QRGfilefilter.java            2003 Northwestern University