package edu.qrg.vmodel;

import edu.qrg.utils.*;
import java.awt.Color;
import java.lang.Cloneable;
import java.lang.String;
import java.lang.System;

/** <PRE> Sun Java 1.3.1 -Sourcepath QRG/VModel/development/code/engine
* ----------------------------------------------------------------------------
* 2003 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
* ----------------------------------------------------------------------------
* File name: VModelConstants.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* ----------------------------------------------------------------------------
* Interface: VModelConstants
*   Purpose: Constants used throughout the VModel executable class. Every
*            class should implement this to gain access to these properties.
* ----------------------------------------------------------------------------
* </PRE> */
public interface VModelConstants extends Cloneable, QRGconstants  {
	// VMODEL
	public static String VMODEL_PROJECT_NAME = "VModel";
	public static double VMODEL_PROJECT_VERSION = 0.6;
	public static String [] VMODEL_PROJECT_AUTHORS = {
		"Kenneth D. Forbus", "Bruce Sherin", "Karen E. Carney",
		"Leo C. Ureel II", "Robert T. Harris", "Tara Teich",
		"Tom Hinrichs", "Danielle Fisher", "Marisa K. Ketzler"
	};


	// Modeling	
	public static final boolean ENFORCE_MODELING_GRAMMAR = false;

	// Events	
	public static final String ADD_OBJECT = "addObject";
	public static final String ADD_GROUP = "addGroup";
	public static final String ADD_ATTRIBUTE = "addAttribute";
	public static final String ADD_RELATIONSHIP = "addRelationship";
	public static final String ADD_MODEL = "addModel";

	public static final String REMOVE_OBJECT = "removeObject";
	public static final String REMOVE_GROUP = "removeGroup";
	public static final String REMOVE_ATTRIBUTE = "removeAttribute";
	public static final String REMOVE_RELATIONSHIP = "removeRelationship";
	public static final String REMOVE_MODEL = "removeModel";

	public static final String NEW_MODEL = "newModel";
	public static final String SAVE_MODEL = "saveModel";
	public static final String OPEN_MODEL = "openModel";
	public static final String MAKE_GROUP = "makeGroup";
	public static final String ADD_TO_GROUP = "addToGroup";
	public static final String AUTO_ADD_TO_GROUP = "autoAddToGroup";
	public static final String AUTOGROUP = "autoGroup";
	public static final String REMOVE_FROM_GROUP = "removeFromGroup";

	public static final String VARIABLIZE = "variablize";
	public static final String BIND = "bind";
	public static final String UNBIND = "unbind";

	public static final String CONNECT = "connect";
	public static final String CONNECT_INPUT = "connectInput";
	public static final String CONNECT_OUTPUT = "connectOutput";
	public static final String DISCONNECT = "disconnect";
	public static final String DISCONNECT_INPUT = "disconnectInput";
	public static final String DISCONNECT_OUTPUT = "disconnectOutput";

	public static final String ADD_TO_LIBRARY = "addToLibrary";

	public static final String UPDATE_PROBLEM_STATEMENT = "updateProblemStatement";
	public static final String UPDATE_DESCRIPTION = "updateDescription";
	public static final String UPDATE_NOTES = "updateNotes";
	public static final String UPDATE_LABEL = "updateLabel";

	public static final String ADD_TO_ONTOLOGY = "addToLibrary";
	public static final String REMOVE_FROM_ONTOLOGY = "removeFromLibrary";

	// Files
	public static final String SEPARATOR = System.getProperty("file.separator");
	public static final String VMODELROOT = "./";

	public static final String VMODEL_SUFFIX = "vmf";
	public static final String VMODEL_MODEL_SUFFIX = "vmf";
	public static final String VMODEL_ONTOLOGY_SUFFIX = "ont";
	public static final String VMODEL_PROJECT_SUFFIX = "vmp";
	public static final String VMODEL_LIBRARY_SUFFIX = "vlb";
	public static final String VMODEL_DEBUG_SUFFIX = "dbg";
	
	public static final String ONT_SUFFIX = "ont";
	public static final String INI_SUFFIX = "ini";
	public static final String JPEG_SUFFIX = "jpg";
	public static final String TIFF_SUFFIX = "tif";
	public static final String BMP_SUFFIX = "bmp";
	public static final String PNM_SUFFIX = "ppm";
	public static final String GIF_SUFFIX = "gif";
	public static final String HTML_SUFFIX = "htm";
	public static final String CML_SUFFIX = "cml";
	public static final String XML_SUFFIX = "xml";
	
	public static final String IMAGE_JPEG = "JPEG";
	public static final String IMAGE_TIFF = "TIFF";
	public static final String IMAGE_BMP = "BMP";
	public static final String IMAGE_PNM = "PNM";
	
	public static final String DATFILE = "vmodel.dat";
	public static final String INIFILE = "vmodel.ini";
	public static final String DEFAULTPROJECT = "default.vmp";
	public static final String ONTFILE = "vmodel.ont";
	public static final String CLASSFILE = "vmodel.class";
	public static final String JARFILE = "vmodel.jar";
	public static final String BATFILE = "vmodel.bat";

	public static final String USERPATH = "users/";
	public static final String MODELDIR = "models";
	public static final String WEBSPATH = "webpages/";
	public static final String MODELSPATH = "models/";
	public static final String IMAGEPATH = "resources/icons/";
	public static final String SOUNDPATH = "resources/sounds/";
	public static final String ONTOLOGYPATH = "resources/ontology/";
	public static final String HELPPATH = "resources/help/";
	public static final String PROJECTPATH = "resources/projects/";
	
	public static String userdir = "users";
	public static String vmodelpath = "models/";
	public static String modelpath = "models/";
	
	// Qualitative States
	public static final String IGNORE = "IGNORE";
	public static final String INCREASING = "INCREASING";
	public static final String DECREASING = "DECREASING";
	public static final String CONSTANT = "CONSTANT";
	public static final String UNKNOWN = "UNKNOWN";
	public static final String UNCONNECTED = "UNCONNECTED";
	public static final String DEFAULT = "UNCHANGED";
	public static final String CONTRADICTION = "AMBIGUOUS";
	public static final String AMBIGUOUS = "AMBIGUOUS2";
	public static final String NORMAL = "NORMAL";
	public static final String DOMINANT = "DOMINANT";
	public static final String NEGLIGABLE = "NEGLIGABLE";
	public static final String USERASSUMPTION = "USERASSUMPTION";
	public static final String DEFAULTASSUMPTION = "DEFAULTASSUMPTION";

	// Element
	public static final String TYPE = "TYPE";
	public static final String TYPENAME = "TYPENAME";
	public static final String QPTYPE = "QPTYPE";
	public static final String OTYPE = "OTYPE";
	public static final String CATEGORY = "CATEGORY";
	public static final String LIBRARY = "LIBRARY";

	public static final String CONNECTIONS = "CONNECTIONS";
	public static final String CONNECTORS = "CONNECTORS";

	public static final String PARTICIPANTS = "PARTICIPANTS";
	public static final String QUANTITIES = "QUANTITIES";
	public static final String CONDITIONS = "CONDITIONS";
	public static final String CONSEQUENCES = "CONSEQUENCES";
	public static final String STRUCTURE = "STRUCTURE";

	// Instances	
	public static final String TARGET = "TARGET";
	public static final String STATE = "STATE";
	public static final String REASONS = "REASONS";
	public static final String INITIALSTATE = "INITIALSTATE";
	public static final String INITIALREASON = "INITIALREASON";
	public static final String NOTES = "NOTES";

	// Basic Ontologyt IDs
	public static final String BASEONTOLOGY 		= "CORE-ONTOLOGY";
	public static final String QPONTOLOGY 			= "QP-ONTOLOGY";
	public static final String CLASSROOMONTOLOGY 	= "CLASSROOM-ONTOLOGY";
	public static final String ENTITIES 			= "ENTITIES";
	public static final String PROCESSES 			= "PROCESSES";
	public static final String ATTRIBUTES 			= "ATTRIBUTES";
	public static final String NUMBERATTRIBUTES 	= "QUANTITY-ATTRIBUTES";
	public static final String RELATIONSHIPS 		= "RELATIONSHIPS";
	public static final String STRUCTURALRELATIONSHIPS = "STRUCTURAL-RELATIONSHIPS";
	public static final String SPATIALRELATIONSHIPS 	= "SPATIAL-RELATIONSHIPS";
	public static final String CONSTRAINTRELATIONSHIPS = "CONSTRAINT-RELATIONSHIPS";
	public static final String CAUSALRELATIONSHIPS 		= "CAUSAL-RELATIONSHIPS";

	// Basic Ontological Element IDs
	public static final String UNKNOWNELEMENT		= "UNKNOWN-ONTOLOGICAL-ELEMENT";
	public static final String ENTITY 				= "ENTITY";
	public static final String COLLECTION 			= "COLLECTION";
	public static final String COLLECTIONAMOUNT		= "COLLECTIONAMOUNT";
	public static final String AGENT		 		= "AGENT";
	public static final String PROCESS 				= "PROCESS";
	public static final String PROCESSRATE 			= "PROCESSRATE";
	public static final String DATA 				= "DATA";
	public static final String MODEL				= "MODEL";
	public static final String MODELFRAGMENT		= "MODELFRAGMENT";
	public static final String ATTRIBUTE 			= "ATTRIBUTE";
	public static final String QUANTITY 	      	= "QUANTITY-ATTRIBUTE";
	public static final String AMOUNT 				= "AMOUNT";
	public static final String LEVEL 				= "LEVEL";
	public static final String MAGNITUDE			= "MAGNITUDE";
	public static final String RATE	 				= "RATE";
	public static final String CHANCE 				= "CHANCE-OF-HAPPENING";
	public static final String RELATIONSHIP 		= "RELATIONSHIP";
	public static final String ISRELATEDTO 			= "IS-RELATED-TO";
	public static final String HASATTRIBUTE 		= "HAS-ATTRIBUTE";
	public static final String HASPARTICIPANT 		= "HAS-PARTICIPANT";
	public static final String ISBOUNDTO 			= "IS-BOUND-TO";
	public static final String ISA 					= "IS-A";
	public static final String AKO 					= "A-KO";
	public static final String INITIATES 			= "INITIATES";
	public static final String SUPPORTS 			= "SUPPORTS";
	public static final String PREDICTS 			= "PREDICTS";
	public static final String CONSTRAINS 			= "CONSTRAINS";
	public static final String CONSTRAINT 			= "CONSTRAINT";
	public static final String ISGREATERTHAN 		= "IS-GREATER-THAN";
	public static final String ISLESSTHAN 			= "IS-LESS-THAN";
	public static final String ISEQUALTO 			= "IS-EQUAL-TO";
	public static final String ISTHESAMEAS 			= "IS-THE-SAME-AS";
	public static final String ISDIFFERENTFROM 		= "IS-DIFFERENT-FROM";
	public static final String MAKESNEGLIGABLE 		= "MAKES-NEGLIGABLE";
	public static final String SPATIALLYRELATED 	= "SPATIALLY-RELATED";
	public static final String TOUCHES 				= "TOUCHES";
	public static final String CONTAINS 			= "CONTAINS";
	public static final String ISPARTOF 			= "IS-PART-OF";
	public static final String REQUIRES 			= "REQUIRES";
	public static final String CAUSES 				= "CAUSES";
	public static final String I		 			= "I";
	public static final String IPLUS 				= "I+";
	public static final String IMINUS 				= "I-";
	public static final String QPROP		 		= "QPROP";
	public static final String QPROPPLUS 			= "QPROP+";
	public static final String QPROPMINUS 			= "QPROP-";
	public static final String CPLUS 				= "C+";
	public static final String CMINUS 				= "C-";
	
	public static final String TAG = "TAG";
	public static final String VALUE = "VALUE";
	
	public static final String VMODEL = "VMODEL";
	public static final String ELEMENT = "ELEMENT";
	public static final String REFERENCE = "REFERENCE";
	public static final String ID = "ID";
	public static final String NAME = "NAME";
	public static final String DESCRIPTION = "DOCUMENTATION";
	public static final String HELPURL = "HELPURL";
	public static final String ICON = "ICON";
	public static final String TYPECHAIN = "TYPECHAIN";
	public static final String PEER = "PEER";
	public static final String PARENT = "PARENT";
	public static final String OWNER = "OWNER";
	public static final String SUBCLASSOF = "SUBCLASSOF";

	public static final String JUSTIFICATIONENTITIES = "JUSTIFICATIONENTITIES";
	public static final String JUSTIFICATION = "JUSTIFICATION";
	public static final String HYPOTHESIS = "HYPOTHESIS";
	public static final String PARTICIPATESIN = "PARTICIPATESIN";
	public static final String DOES = "DOES";
	public static final String BOUNDTO = "BOUNDTO";
	public static final String DIRECTINFLUENCE = "DIRECTINFLUENCE";
	public static final String INDIRECTINFLUENCE = "INDIRECTINFLUENCE";
	public static final String JUSTIFICATIONRELATIONSHIPS = "JUSTIFICATIONRELATIONSHIPS";
	
	public static final String CAUSALRELATIONSHIP = "CAUSAL-RELATIONSHIP";
	public static final String CONSTRAINTRELATIONSHIP = "CONSTRAINT-RELATIONSHIP";
	public static final String STRUCTURALRELATIONSHIP = "STRUCTURAL-RELATIONSHIP";

	public static final String INPUT = "INPUT";
	public static final String OUTPUT = "OUTPUT";

	public static final String INPUTS = "INPUTS";
	public static final String OUTPUTS = "OUTPUTS";

	public static final String CAUSALINPUTS = "CAUSALINPUTS";
	public static final String CAUSALOUTPUTS = "CAUSALOUTPUTS";

	public static final String PARENTINSTANCES = "PARENTINSTANCES";
	public static final String CHILDINSTANCES = "CHILDINSTANCES";
	public static final String CAUSALCOUNT = "CAUSALCOUNT";
	public static final String CAUSALCOUNT2 = "CAUSALCOUNT2";

	// Modes
	public static final String MODE = "MODE";
	public static final int NULL_MODE = 0;
	public static final int PROBLEM_MODE = 100;
	public static final int BUILD_MODE = 101;
	public static final int ANALYZE_MODE = 102;
	public static final int JUSTIFY_MODE = 103;
	public static final String TARGETMODE = "Target Mode";
	public static final String BUILDMODE = "Build Mode";
	public static final String ANALYSISMODE = "Analysis Mode";
	
	public static final String MODES[] = {"PROBLEM", "BUILD", "ANALYZE", "JUSTIFY"};
	
	// Views
	public static final int DESCRIBE = 0;
	public static final int SITUATION = 0;
	public static final int EXPLAIN = 1;
	public static final int CAUSAL = 1;
	public static final int JUSTIFY = 2;
	public static final int EVIDENCE = 2;
	
	// ORIENTATION
	public static final int WEST2EAST = 0;
	public static final int NORTH2SOUTH = 1;
	public static final int EAST2WEST = 2;
	public static final int SOUTH2NORTH = 3;
	
	// Colors
	public static final Color SPLASH = new Color(174, 191, 199);
	public static final Color BACKGROUND_COLOR = new Color(-8084279);
	
	// Component Display Modes
	public static final int UNKNOWN_MODE = -1;
	public static final int RELATIONSHIP_MODE = 0;
	public static final int OBJECT_MODE = 1;
	public static final int BOX_MODE = 2;
	public static final int IMAGE_MODE = 3;
	public static final int GROUP_MODE = 4;
	public static final int MODEL_MODE = 5;
	public static final int ATTRIBUTE_MODE = 6;
	
	// XML
	public static final String URI_PREFIX = "file://";
	public static final String XML_HEADING = "<?xml version = \"1.0\" encoding =\"UTF-8\" standalone = \"no\"?>"+ '\r';
	public static final String MODEL_DOCTYPE = "<!DOCTYPE Model SYSTEM \"vmodel.dtd\">"+'\r';
	public static final String PREFERENCES_DOCTYPE = "<!DOCTYPE Preferences SYSTEM \"vmodel.dtd\">"+'\r';
	public static final String ONTOLOGY_DOCTYPE = "<!DOCTYPE Ontology SYSTEM \"vmodel.dtd\">"+'\r';
	
	// Misc
	public static final int VIEWALL = -1;
	public static final int BLOCKINCREMENT = 1;
	public static final int BACKWARD = -1;
	public static final int FORWARD = 1;
	
} // Interface 

// ---------------------------------------------------------------------------
// END OF FILE: VModelConstants.java             2003 Northwestern University