package edu.qrg.vmodel;

import edu.qrg.utils.*;
import edu.qrg.vmodel.*;
import edu.qrg.utils.PList.PLinterface;
import edu.qrg.utils.XMLio.XMLable;
import java.io.Serializable;
import java.lang.Cloneable;
import java.lang.Object;
import java.lang.String;
import java.util.Enumeration;
import java.util.Vector;

/** <PRE> Sun Java 1.3.1 -Sourcepath QRG/VModel/development/code/ontology
* ----------------------------------------------------------------------------
*  2003 Northwestern University Qualitative Reasoning Group Kenneth D. Forbus
* ----------------------------------------------------------------------------
* File name: VectorElement.java
*    System: VModel
*     Since: v0.6
*    Author: Leo C. Ureel II
* ----------------------------------------------------------------------------
*   Class: VectorElement
* Purpose: This is a wrapper class used to contain objects such as those
*          within lists that are keyed values in a PList.  Such objects
*          do not lend themselves to writing as an attribute value
*          in an XML file.  
* ----------------------------------------------------------------------------
* </PRE>
* @see PList
* @see KeyValuePair
*/
public class VectorElement extends PList 
		implements Cloneable, Serializable, PList.PLinterface, 
			VModelConstants, XMLio.XMLable {
	// Constructors
	// -------------------------------------------------------------------------
	/** The empty constructor should only be called internally. */
	public VectorElement() {
	   super();
	} // default constructor 
	
	/** Call this constructor to set a TAG and VALUE. */
	public VectorElement(String tag, Object value) {
	   this();
	   set(TAG, tag);
	   set(VALUE, value);
	} // constructor 
	
	// XMLio.
	// -----------------------------------------------------------------------
	/** Used to stash XMLio objects targeted for stringification.
	* @see #xmlify
	* @see #stringifyNonXmlValues
	*/
	protected transient Vector xmlElmnts = null;

	/** Stringifies the XML definition for this class.
	* Note: I'm not an XML specialist - this may be incomplete or wrong -Leo 
	*/
	public String xmlDef () { 
		String result = 	"";
		String classname = this.getClass().getName();
		result += "<!DOCTYPE " + classname + " [\r\n";
		result += "<!ELEMENT " + classname + " EMPTY >\r\n";
		result += "<!ATTLIST " + classname + " TAG CDATA #REQUIRED >\r\n";
		result += "<!ATTLIST " + classname + " VALUE CDATA #REQUIRED >\r\n";
		result += "]>";
		return(result); 
	} // method
	
	/** Stringifies the class into XML format.
	* Note: I'm not an XML specialist - this may be incomplete or incorrect.
	*       Especially with regard to illegal characters.
	* @see #stringifyNonXmlValues
	* @see #xmlElmnts
	*/
	public String xmlify () {
		String result = "";
		KeyValuePair kvp = null;
		int i = 0, j = 0;
		result = "<" + "VectorElement"; //this.getClass().getName();
		result += stringifyNonXmlValues(result.length() + 1);
		if (xmlElmnts.isEmpty()) result += " />" + QRGutils.eol();
		else {
			 result += " >" + QRGutils.eol();
			for(Enumeration enum = xmlElmnts.elements(); enum.hasMoreElements();) {
				kvp = (KeyValuePair) enum.nextElement();
				result += ((XMLio.XMLable) kvp.value()).xmlify();
			} // end if
			result += "</" + "VectorElement" /* this.getClass().getName() */ + ">" + QRGutils.eol();
		} // end if
		return(result);
	} // method

	/** This method is called by the XMLio load function 
	*   to initialize and restore values. 
	* @see XMLio
	*/
	public void XMLioObjectInit (Vector attr, Vector elmnt) {
		keyValuePairs(attr);
		if ((elmnt != null) && (! elmnt.isEmpty())) set(VALUE, elmnt.elementAt(0));
	} // constructor 
	
	/** Saves the ontology to filename in XML format. */
	public void exportXML (String filename) {
		XMLio.save(QRGutils.QRGuserpath() + filename + "." + XML_SUFFIX , this);
	} // method
	
	/** Stringifies non XML values using XML format.
	*   If a value is an instanceof XMLio, 
	*   then it is thrown on the xmlElmnts list
	*   for stringification later.
	* @see #xmlify
	* @see #xmlElmnts
	*/
	protected String stringifyNonXmlValues (int indent) {
		String result = "";
		xmlElmnts = new Vector();
		KeyValuePair kvp = null;
		int j = 0;
		if (keyValuePairs() != null) {
			j = 0;
			while(j < keyValuePairs().size()) {
				if (j > 0) result += QRGutils.eol() + QRGutils.space(indent);
				else result += " "; 
				kvp = (KeyValuePair) keyValuePairs().elementAt(j);
				if (kvp.value() instanceof XMLio.XMLable) xmlElmnts.addElement(kvp);
				else result += kvp;
				j++;
			} // while
		} // end if
		return(result);
	} // method
} // class VectorElement

// ---------------------------------------------------------------------------
// End of File: VectorElement.java               2003 Northwestern University
